@@ -583,13 +583,11 @@ int nv_drm_dumb_map_offset(struct drm_file *file,
     return ret;
 }
 
-#if defined(NV_DRM_DRIVER_HAS_DUMB_DESTROY)
 int nv_drm_dumb_destroy(struct drm_file *file,
                         struct drm_device *dev,
                         uint32_t handle)
 {
     return drm_gem_handle_delete(file, handle);
 }
-#endif /* NV_DRM_DRIVER_HAS_DUMB_DESTROY */
 
 #endif
