@@ -24,6 +24,7 @@
 #include "uvm_channel.h"
 #include "uvm_global.h"
 #include "uvm_hal.h"
+#include "uvm_kvmalloc.h"
 #include "uvm_push.h"
 #include "uvm_test.h"
 #include "uvm_tracker.h"
@@ -179,7 +180,7 @@ static NV_STATUS test_membar(uvm_gpu_t *gpu)
 
     for (i = 0; i < REDUCTIONS; ++i) {
         uvm_push_set_flag(&push, UVM_PUSH_FLAG_NEXT_MEMBAR_NONE);
-        gpu->parent->ce_hal->semaphore_reduction_inc(&push, host_mem_gpu_va, REDUCTIONS);
+        gpu->parent->ce_hal->semaphore_reduction_inc(&push, host_mem_gpu_va, REDUCTIONS + 1);
     }
 
     // Without a sys membar the channel tracking semaphore can and does complete
@@ -528,7 +529,7 @@ static NV_STATUS test_semaphore_reduction_inc(uvm_gpu_t *gpu)
 
     for (i = 0; i < REDUCTIONS; i++) {
         uvm_push_set_flag(&push, UVM_PUSH_FLAG_NEXT_MEMBAR_NONE);
-        gpu->parent->ce_hal->semaphore_reduction_inc(&push, gpu_va, REDUCTIONS);
+        gpu->parent->ce_hal->semaphore_reduction_inc(&push, gpu_va, i+1);
     }
 
     status = uvm_push_end_and_wait(&push);
@@ -655,9 +656,11 @@ static NV_STATUS test_ce(uvm_va_space_t *va_space, bool skipTimestampTest)
         TEST_NV_CHECK_RET(test_memcpy_and_memset(gpu));
         TEST_NV_CHECK_RET(test_semaphore_reduction_inc(gpu));
         TEST_NV_CHECK_RET(test_semaphore_release(gpu));
+
         if (!skipTimestampTest)
             TEST_NV_CHECK_RET(test_semaphore_timestamp(gpu));
-    }
+
+   }
 
     return NV_OK;
 }
