@@ -26,19 +26,11 @@
 #include "nv-reg.h"
 #include "nv-frontend.h"
 
-#if defined(MODULE_LICENSE)
 MODULE_LICENSE("Dual MIT/GPL");
-#endif
-#if defined(MODULE_INFO)
-MODULE_INFO(supported, "external");
-#endif
-#if defined(MODULE_VERSION)
-MODULE_VERSION(NV_VERSION_STRING);
-#endif
 
-#ifdef MODULE_ALIAS_CHARDEV_MAJOR
+MODULE_INFO(supported, "external");
+MODULE_VERSION(NV_VERSION_STRING);
 MODULE_ALIAS_CHARDEV_MAJOR(NV_MAJOR_DEVICE_NUMBER);
-#endif
 
 /*
  * MODULE_IMPORT_NS() is added by commit id 8651ec01daeda
@@ -79,9 +71,6 @@ int          nvidia_frontend_mmap(struct file *, struct vm_area_struct *);
 static struct file_operations nv_frontend_fops = {
     .owner     = THIS_MODULE,
     .poll      = nvidia_frontend_poll,
-#if defined(NV_FILE_OPERATIONS_HAS_IOCTL)
-    .ioctl     = nvidia_frontend_ioctl,
-#endif
     .unlocked_ioctl = nvidia_frontend_unlocked_ioctl,
 #if NVCPU_IS_X86_64 || NVCPU_IS_AARCH64
     .compat_ioctl = nvidia_frontend_compat_ioctl,
