@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -125,7 +125,6 @@ namespace DisplayPort
         bool    bPConConnected;                 // HDMI2.1-Protocol Converter (Support SRC control mode) connected.
         bool    bSkipAssessLinkForPCon;         // Skip assessLink() for PCON. DD will call assessFRLLink later.
         bool    bHdcpAuthOnlyOnDemand;          // True if only initiate Hdcp authentication on demand and MST won't auto-trigger authenticate at device attach.
-        bool    bReassessMaxLink;               // Retry assessLink() if the first assessed link config is lower than the panel max config.
 
         bool    constructorFailed;
 
@@ -300,8 +299,6 @@ namespace DisplayPort
         bool        bEnableFastLT;
         NvU32       maxLinkRateFromRegkey;
 
-        bool        bEnableOuiRestoring;
-
         //
         // Latency(ms) to apply between link-train and FEC enable for bug
         // 2561206.
@@ -336,14 +333,6 @@ namespace DisplayPort
         //
         bool        bPowerDownPhyBeforeD3;
 
-        //
-        // Reset the MSTM_CTRL registers on Synaptics branch device irrespective of
-        // IRQ VECTOR register having stale message. Synaptics device needs to reset
-        // the topology before issue of new LAM message if previous LAM was not finished
-        // bug 3928070
-        //
-        bool bForceClearPendingMsg;
-
         void sharedInit();
         ConnectorImpl(MainLink * main, AuxBus * auxBus, Timer * timer, Connector::EventSink * sink);
 
