@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2009-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2009 - 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -448,6 +448,7 @@ static const PNPVendorId PNPVendorIds[] =
     { "SEA",    _VENDOR_NAME_ENTRY("Segate")                            },
     { "SEC",    _VENDOR_NAME_ENTRY("Seiko/Epson")                       },
     { "SEN",    _VENDOR_NAME_ENTRY("Sencore")                           },
+    { "SFL",    _VENDOR_NAME_ENTRY("Shiftall Inc")                      },
     { "SGT",    _VENDOR_NAME_ENTRY("Stargate Technology/AT&T")          },
     { "SGX",    _VENDOR_NAME_ENTRY("SGI")                               },
     { "SHP",    _VENDOR_NAME_ENTRY("Sharp")                             },
@@ -504,6 +505,7 @@ static const PNPVendorId PNPVendorIds[] =
     { "TEI",    _VENDOR_NAME_ENTRY("TECO")                              },
     { "TGI",    _VENDOR_NAME_ENTRY("TriGem")                            },
     { "TGS",    _VENDOR_NAME_ENTRY("Torus")                             },
+    { "TMA",    _VENDOR_NAME_ENTRY("Tianma Microelectronics")           },
     { "TOS",    _VENDOR_NAME_ENTRY("Toshiba")                           },
     { "TRI",    _VENDOR_NAME_ENTRY("Tricord")                           },
     { "TRM",    _VENDOR_NAME_ENTRY("Tekram")                            },
