@@ -315,8 +315,7 @@ NvHdmi_QueryFRLConfig(NvHdmiPkt_Handle                      libHandle,
     }
 
     // if there is no FRL capability reported fail this call
-    if ((pSrcCaps->linkMaxFRLRate == HDMI_FRL_DATA_RATE_NONE) ||
-        (pSinkCaps->linkMaxFRLRate == HDMI_FRL_DATA_RATE_NONE))
+    if (pSinkCaps->linkMaxFRLRate == HDMI_FRL_DATA_RATE_NONE)
     {
         return NVHDMIPKT_FAIL;
     }
