@@ -26,18 +26,14 @@
 #include "nvlink_lock.h"
 
 //
-// Only enabling top level locking for linux as required by Bug 4108674.
-// Per link locking is still disabled at all times. It will be enabled 
-// after other locking related clean up is done.
-// 
+// Only enabling locking for testing purposes at the moment.
+// Disabled at all other times.
+//
+#define LOCKING_DISABLED 1 
 
 static void   _sort_links(nvlink_link **, NvU32, NvBool (*)(void *, void *));
 static NvBool _compare(void *, void *);
 
-#if defined(NV_LINUX)
-#undef TOP_LEVEL_LOCKING_DISABLED 
-#   define TOP_LEVEL_LOCKING_DISABLED 0
-#endif  /* defined(NV_LINUX) */
 /*
  * Allocate top level lock. Return NVL_SUCCESS if 
  * the lock was allocated else return NVL_ERR_GENERIC.
@@ -45,7 +41,7 @@ static NvBool _compare(void *, void *);
 NvlStatus
 nvlink_lib_top_lock_alloc(void)
 {
-    if (TOP_LEVEL_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -86,7 +82,7 @@ nvlink_lib_top_lock_alloc(void)
 NvlStatus
 nvlink_lib_top_lock_free(void)
 {
-    if (TOP_LEVEL_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -119,7 +115,7 @@ nvlink_lib_link_lock_alloc
     nvlink_link *link
 )
 {
-    if (PER_LINK_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -162,7 +158,7 @@ nvlink_lib_link_lock_free
     nvlink_link *link
 )
 {
-    if (PER_LINK_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -192,7 +188,7 @@ nvlink_lib_link_lock_free
 NvlStatus
 nvlink_lib_top_lock_acquire(void)
 {
-    if (TOP_LEVEL_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -227,7 +223,7 @@ nvlink_lib_top_lock_acquire(void)
 NvlStatus
 nvlink_lib_top_lock_release(void)
 {
-    if (TOP_LEVEL_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
@@ -269,12 +265,13 @@ nvlink_lib_link_locks_acquire
     int           numLinks
 )
 {
-    if (PER_LINK_LOCKING_DISABLED)
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
 
     int i;
+
     nvlink_link *link_prev  = NULL;
 
     // Check if array of links is already empty before attempting to release. 
@@ -331,12 +328,13 @@ nvlink_lib_link_locks_release
     int           numLinks
 )
 {
-    if (PER_LINK_LOCKING_DISABLED)
+    int i;
+
+    if (LOCKING_DISABLED)
     {
         return NVL_SUCCESS;
     }
 
-    int i;
     nvlink_link *link_prev  = NULL;
 
     // Check if array of links is already empty before attempting to release. 
