@@ -642,6 +642,12 @@ nvswitch_os_get_platform_time
     void
 );
 
+NvU64
+nvswitch_os_get_platform_time_epoch
+(
+    void
+);
+
 #define NVSWITCH_PRINT_ATTRIB(str, arg1)             \
     __attribute__ ((format (printf, (str), (arg1))))
 
@@ -798,6 +804,14 @@ nvswitch_os_strncmp
     NvLength length
 );
 
+char*
+nvswitch_os_strncat
+(
+    char *s1,
+    const char *s2,
+    NvLength length
+);
+
 void *
 nvswitch_os_memset
 (
@@ -881,7 +895,6 @@ nvswitch_os_vsnprintf
 void
 nvswitch_os_assert_log
 (
-    int cond,
     const char *pFormat,
     ...
 );
