@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -32,7 +32,6 @@
 #include "nvVer.h"
 #include "regkey_nvswitch.h"
 #include "inforom/inforom_nvl_v3_nvswitch.h"
-#include "soe/soeififr.h"
 
 //
 // TODO: Split individual object hals to their own respective files
@@ -1281,14 +1280,3 @@ nvswitch_bbx_get_sxid_lr10
     return -NVL_ERR_NOT_SUPPORTED;
 }
 
-NvlStatus
-nvswitch_bbx_get_data_lr10
-(
-    nvswitch_device *device,
-    NvU8 dataType,
-    void *params
-)
-{
-    return -NVL_ERR_NOT_SUPPORTED;
-}
-
