@@ -28,7 +28,6 @@
 #include "ls10/therm_ls10.h"
 #include "error_nvswitch.h"
 #include "soe/soeiftherm.h"
-#include "rmflcncmdif_nvswitch.h"
 
 #include "flcn/flcnable_nvswitch.h"
 #include "flcn/flcn_nvswitch.h"
@@ -362,105 +361,8 @@ nvswitch_monitor_thermal_alert_ls10
     return;
 }
 
-/*
- * @brief Callback function to recieve thermal messages from SOE.
- */
-void
-nvswitch_therm_soe_callback_ls10
-(
-    nvswitch_device *device,
-    RM_FLCN_MSG *pGenMsg,
-    void *pParams,
-    NvU32 seqDesc,
-    NV_STATUS status
-)
-{
-    RM_SOE_THERM_MSG_SLOWDOWN_STATUS slowdown_status;
-    RM_SOE_THERM_MSG_SHUTDOWN_STATUS shutdown_status;
-    RM_FLCN_MSG_SOE *pMsg = (RM_FLCN_MSG_SOE *)pGenMsg;
-    NvU32 temperature;
-    NvU32 threshold;
-
-    switch (pMsg->msg.soeTherm.msgType)
-    {
-        case RM_SOE_THERM_MSG_ID_SLOWDOWN_STATUS:
-        {
-            slowdown_status = pMsg->msg.soeTherm.slowdown;
-            if (slowdown_status.bSlowdown)
-            {
-                if (slowdown_status.source.bTsense) // TSENSE_THERM_ALERT
-                {
-                    temperature = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(slowdown_status.maxTemperature);
-                    threshold   = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(slowdown_status.warnThreshold);
-
-                    NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_START,
-                        "NVSWITCH Temperature %dC | TSENSE WARN Threshold %dC\n",
-                        temperature, threshold);
-
-                    NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_START,
-                        "Thermal Slowdown Engaged | Temp higher than WARN Threshold\n");
-                }
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_START,
-                    "Thermal Slowdown Engaged | Links Thermal Mode %s\n", (slowdown_status.bLinksL1Status ? "ON" : "OFF"));
-
-                if (slowdown_status.source.bPmgr) // PMGR_THERM_ALERT
-                {
-                    NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_START,
-                        "Thermal Slowdown Engaged | PMGR WARN Threshold reached\n");
-                }
-            }
-            else // REVERT_SLOWDOWN
-            {
-                temperature = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(slowdown_status.maxTemperature);
-                threshold   = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(slowdown_status.warnThreshold);
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_END,
-                    "NVSWITCH Temperature %dC | TSENSE WARN Threshold %dC\n",
-                    temperature, threshold);
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_END,
-                    "Thermal Slowdown Disengaged | Links Thermal Mode %s\n", (slowdown_status.bLinksL1Status ? "ON" : "OFF"));
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_END,
-                    "Thermal slowdown Disengaged\n");
-            }
-            break;
-        }
-
-        case RM_SOE_THERM_MSG_ID_SHUTDOWN_STATUS:
-        {
-            shutdown_status = pMsg->msg.soeTherm.shutdown;
-            if (shutdown_status.source.bTsense) // TSENSE_THERM_SHUTDOWN
-            {
-                temperature = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(shutdown_status.maxTemperature);
-                threshold   = RM_SOE_NV_TEMP_TO_CELSIUS_TRUNCED(shutdown_status.overtThreshold);
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_SHUTDOWN,
-                    "NVSWITCH Temperature %dC | OVERT Threshold %dC\n",
-                    temperature, threshold);
-
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_SHUTDOWN,
-                    "TSENSE OVERT Threshold reached. Shutting Down\n");
-            }
-
-            if (shutdown_status.source.bPmgr) // PMGR_THERM_SHUTDOWN
-            {
-                NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_HOST_THERMAL_EVENT_START,
-                    "PMGR OVERT Threshold reached. Shutting Down\n");
-            }
-            break;
-        }
-        default:
-        {
-            NVSWITCH_PRINT(device, ERROR, "%s Unknown message Id\n", __FUNCTION__);
-            NVSWITCH_ASSERT(0);
-        }
-    }
-}
-
 //
-// nvswitch_ctrl_therm_read_voltage
+// nvswitch_therm_read_voltage
 //
 // Temperature and voltage are only available on SKUs which have thermal and
 // voltage sensors.
@@ -543,86 +445,3 @@ nvswitch_ctrl_therm_read_voltage_ls10
     return NVL_SUCCESS;
 }
 
-//
-// nvswitch_ctrl_therm_read_power
-//
-// Power is only available on SKUs which have thermal and
-// voltage sensors.
-//
-NvlStatus
-nvswitch_ctrl_therm_read_power_ls10
-(
-    nvswitch_device *device,
-    NVSWITCH_GET_POWER_PARAMS *pParams
-)
-{
-    FLCN                *pFlcn;
-    NvU32               cmdSeqDesc;
-    NV_STATUS           status;
-    NvU8                flcnStatus;
-    RM_FLCN_CMD_SOE     cmd;
-    RM_FLCN_MSG_SOE     msg;
-    RM_SOE_CORE_CMD_GET_POWER *pGetPowerCmd;
-    NVSWITCH_TIMEOUT    timeout;
-
-    if (!nvswitch_is_soe_supported(device))
-    {
-        return -NVL_ERR_NOT_SUPPORTED;
-    }
-
-    if (pParams == NULL)
-    {
-        return -NVL_BAD_ARGS;
-    }
-
-    pFlcn = device->pSoe->pFlcn;
-
-    nvswitch_os_memset(pParams, 0, sizeof(NVSWITCH_GET_POWER_PARAMS));
-    nvswitch_os_memset(&cmd, 0, sizeof(RM_FLCN_CMD_SOE));
-    nvswitch_os_memset(&msg, 0, sizeof(RM_FLCN_MSG_SOE));
-
-    cmd.hdr.unitId = RM_SOE_UNIT_CORE;
-    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, GET_POWER);
-
-    msg.hdr.unitId = RM_SOE_UNIT_CORE;
-    msg.hdr.size   = RM_SOE_MSG_SIZE(CORE, GET_POWER);
-
-    pGetPowerCmd = &cmd.cmd.core.getPower;
-    pGetPowerCmd->cmdType = RM_SOE_CORE_CMD_GET_POWER_VALUES;
-
-    cmdSeqDesc = 0;
-
-    nvswitch_timeout_create(NVSWITCH_INTERVAL_1SEC_IN_NS * 5, &timeout);
-    status = flcnQueueCmdPostBlocking(device, pFlcn,
-                                      (PRM_FLCN_CMD)&cmd,
-                                      (PRM_FLCN_MSG)&msg,   // pMsg
-                                      NULL, // pPayload
-                                      SOE_RM_CMDQ_LOG_ID,
-                                      &cmdSeqDesc,
-                                      &timeout);
-    if (status != NV_OK)
-    {
-        NVSWITCH_PRINT(device, ERROR, "%s: Failed to read power 0x%x\n", 
-                       __FUNCTION__, status);
-        return -NVL_ERR_INVALID_STATE;
-    }
-
-    flcnStatus = msg.msg.core.getPower.flcnStatus;
-    if (flcnStatus != FLCN_OK)
-    {
-        if (flcnStatus == FLCN_ERR_MORE_PROCESSING_REQUIRED)
-        {
-            return -NVL_MORE_PROCESSING_REQUIRED;
-        }
-        else
-        {
-            return -NVL_ERR_GENERIC;
-        }
-    }
-
-    pParams->vdd_w = msg.msg.core.getPower.vdd_w;
-    pParams->dvdd_w = msg.msg.core.getPower.dvdd_w;
-    pParams->hvdd_w = msg.msg.core.getPower.hvdd_w;
-
-    return NVL_SUCCESS;
-}
\ No newline at end of file
