@@ -200,8 +200,8 @@ extern "C" {
 #define NV2080_NOTIFIERS_POSSIBLE_ERROR                            (164)
 #define NV2080_NOTIFIERS_NVLINK_INFO_LINK_UP                       (165)
 #define NV2080_NOTIFIERS_NVLINK_INFO_LINK_DOWN                     (176)
-#define NV2080_NOTIFIERS_HDMI_FRL_RETRAINING_REQUEST               (177)
-#define NV2080_NOTIFIERS_AUX_POWER_STATE_CHANGE                    (178)
+#define NV2080_NOTIFIERS_NVPCF_EVENTS                              (177)
+#define NV2080_NOTIFIERS_HDMI_FRL_RETRAINING_REQUEST               (178)
 #define NV2080_NOTIFIERS_MAXCOUNT                                  (179)
 
 // Indexed GR notifier reference
