@@ -114,74 +114,93 @@ static NVDpyIdList UpdateConnectedDpys(NVDispEvoPtr pDispEvo)
 
 static void FlipBaseToNull(NVDevEvoPtr pDevEvo)
 {
-    struct NvKmsFlipParams *pParams = nvCalloc(1, sizeof(*pParams));
-    struct NvKmsFlipRequest *pRequest;
+    struct NvKmsFlipRequestOneHead *pFlipApiHead = NULL;
+    NvU32 numFlipApiHeads = 0, i;
     NvU32 sd;
     NVDispEvoPtr pDispEvo;
     NvBool ret = TRUE;
 
-    if (!pParams) {
+    /* First count the number of active heads. */
+    FOR_ALL_EVO_DISPLAYS(pDispEvo, sd, pDevEvo) {
+        NvU32 apiHead;
+        for (apiHead = 0; apiHead < pDevEvo->numApiHeads; apiHead++) {
+            if (!nvApiHeadIsActive(pDispEvo, apiHead)) {
+                continue;
+            }
+            numFlipApiHeads++;
+        }
+    }
+
+    if (numFlipApiHeads == 0) {
+        // If no heads require changes, there's nothing to do.
+        return;
+    }
+
+    /* Allocate an array of head structures */
+    pFlipApiHead = nvCalloc(numFlipApiHeads, sizeof(pFlipApiHead[0]));
+
+    if (!pFlipApiHead) {
         nvEvoLogDevDebug(pDevEvo, EVO_LOG_WARN,
             "Failed to allocate flip parameters for console restore base flip "
             "to NULL");
         return;
     }
 
-    pRequest = &pParams->request;
-
+    i = 0;
     FOR_ALL_EVO_DISPLAYS(pDispEvo, sd, pDevEvo) {
-        struct NvKmsFlipRequestOneSubDevice *pRequestSd =
-            &pRequest->sd[sd];
-        NvU32 head;
-        for (head = 0; head < pDevEvo->numHeads; head++) {
-            struct NvKmsFlipCommonParams *pRequestHead =
-                &pRequestSd->head[head];
+        NvU32 apiHead;
+        for (apiHead = 0; apiHead < pDevEvo->numApiHeads; apiHead++) {
+            struct NvKmsFlipCommonParams *pRequestApiHead = NULL;
             NvU32 layer;
 
-            if (!nvHeadIsActive(pDispEvo, head)) {
+            if (!nvApiHeadIsActive(pDispEvo, apiHead)) {
                 continue;
             }
 
-            pRequestSd->requestedHeadsBitMask |= NVBIT(head);
+            pFlipApiHead[i].sd = sd;
+            pFlipApiHead[i].head = apiHead;
+            pRequestApiHead = &pFlipApiHead[i].flip;
+            i++;
+            nvAssert(i <= numFlipApiHeads);
 
-            for (layer = 0; layer < pDevEvo->head[head].numLayers; layer++) {
-                pRequestHead->layer[layer].surface.specified = TRUE;
+            for (layer = 0; layer < pDevEvo->apiHead[apiHead].numLayers; layer++) {
+                pRequestApiHead->layer[layer].surface.specified = TRUE;
                 // No need to specify sizeIn/sizeOut as we are flipping NULL surface.
-                pRequestHead->layer[layer].compositionParams.specified = TRUE;
-                pRequestHead->layer[layer].completionNotifier.specified = TRUE;
-                pRequestHead->layer[layer].syncObjects.specified = TRUE;
+                pRequestApiHead->layer[layer].compositionParams.specified = TRUE;
+                pRequestApiHead->layer[layer].completionNotifier.specified = TRUE;
+                pRequestApiHead->layer[layer].syncObjects.specified = TRUE;
 
                 // Disable HDR
-                pRequestHead->tf.val = NVKMS_OUTPUT_TF_NONE;
-                pRequestHead->tf.specified = TRUE;
-                pRequestHead->layer[layer].hdr.enabled = FALSE;
-                pRequestHead->layer[layer].hdr.specified = TRUE;
-                pRequestHead->layer[layer].colorspace.val =
+                pRequestApiHead->tf.val = NVKMS_OUTPUT_TF_NONE;
+                pRequestApiHead->tf.specified = TRUE;
+                pRequestApiHead->layer[layer].hdr.enabled = FALSE;
+                pRequestApiHead->layer[layer].hdr.specified = TRUE;
+                pRequestApiHead->layer[layer].colorspace.val =
                     NVKMS_INPUT_COLORSPACE_NONE;
-                pRequestHead->layer[layer].colorspace.specified = TRUE;
+                pRequestApiHead->layer[layer].colorspace.specified = TRUE;
             }
-
-            pRequest->commit = TRUE;
         }
     }
 
-    // If no heads require changes, there's nothing to do.
-    if (pRequest->commit) {
-        ret = nvFlipEvo(pDevEvo, pDevEvo->pNvKmsOpenDev, pRequest,
-                        &pParams->reply, FALSE /* skipUpdate */,
-                        FALSE /* allowFlipLock */);
-    }
-    nvFree(pParams);
+    ret = nvFlipEvo(pDevEvo, pDevEvo->pNvKmsOpenDev,
+                    pFlipApiHead,
+                    numFlipApiHeads,
+                    TRUE  /* commit */,
+                    FALSE /* allowVrr */,
+                    NULL  /* pReply */,
+                    FALSE /* skipUpdate */,
+                    FALSE /* allowFlipLock */);
+    nvFree(pFlipApiHead);
 
     if (!ret) {
         nvAssert(!"Console restore failed to flip base to NULL");
     }
 
     FOR_ALL_EVO_DISPLAYS(pDispEvo, sd, pDevEvo) {
-        NvU32 head;
-        for (head = 0; head < pDevEvo->numHeads; head++) {
+        NvU32 apiHead;
+        for (apiHead = 0; apiHead < pDevEvo->numApiHeads; apiHead++) {
             NvBool stoppedBase;
-            ret = nvRMIdleBaseChannel(pDevEvo, head, sd, &stoppedBase);
+            ret = nvIdleBaseChannelOneApiHead(pDispEvo, apiHead, &stoppedBase);
             if (!ret) {
                 nvAssert(!"Console restore failed to idle base");
             }
@@ -189,13 +208,55 @@ static void FlipBaseToNull(NVDevEvoPtr pDevEvo)
     }
 }
 
+/*!
+ * Return the mask of active api heads on this pDispEvo.
+ */
+static NvU32 GetActiveApiHeadMask(NVDispEvoPtr pDispEvo)
+{
+    NvU32 apiHead;
+    NvU32 apiHeadMask = 0;
+
+    for (apiHead = 0; apiHead < NVKMS_MAX_HEADS_PER_DISP; apiHead++) {
+        if (nvApiHeadIsActive(pDispEvo, apiHead)) {
+            apiHeadMask |= 1 << apiHead;
+        }
+    }
+
+    return apiHeadMask;
+}
+
+static NvU32 PickApiHead(const NVDpyEvoRec *pDpyEvo,
+                         const NvU32 availableApiHeadsMask)
+{
+    const NvU32 possibleApiHeads = availableApiHeadsMask &
+                                   pDpyEvo->pConnectorEvo->validApiHeadMask;
+    const NvU32 activeApiHeadsMask =
+        GetActiveApiHeadMask(pDpyEvo->pDispEvo);
+
+    if (possibleApiHeads == 0) {
+        return NV_INVALID_HEAD;
+    }
+
+    if ((pDpyEvo->apiHead != NV_INVALID_HEAD) &&
+            ((NVBIT(pDpyEvo->apiHead) & possibleApiHeads) != 0x0)) {
+        return pDpyEvo->apiHead;
+    }
+
+    if ((possibleApiHeads & ~activeApiHeadsMask) != 0x0) {
+        return BIT_IDX_32(LOWESTBIT(possibleApiHeads &
+            ~activeApiHeadsMask));
+    }
+
+    return BIT_IDX_32(LOWESTBIT(possibleApiHeads));
+}
+
 static NvBool InitModeOneHeadRequest(
     NVDpyEvoRec *pDpyEvo,
     NVSurfaceEvoPtr pSurfaceEvo,
     const struct NvKmsMode *pOverrideMode,
     const struct NvKmsSize *pOverrideViewPortSizeIn,
     const struct NvKmsPoint *pOverrideViewPortPointIn,
-    const NvU32 head,
+    const NvU32 apiHead,
     struct NvKmsSetModeOneHeadRequest *pRequestHead)
 {
 
@@ -242,7 +303,7 @@ static NvBool InitModeOneHeadRequest(
         pFlip->layer[NVKMS_MAIN_LAYER].sizeIn.val;
 
     /* Disable other layers except Main */
-    for (layer = 0; layer < pDevEvo->head[head].numLayers; layer++) {
+    for (layer = 0; layer < pDevEvo->apiHead[apiHead].numLayers; layer++) {
 
         if (layer == NVKMS_MAIN_LAYER) {
             pFlip->layer[layer].csc.matrix = NVKMS_IDENTITY_CSC_MATRIX;
@@ -272,24 +333,21 @@ ConstructModeOneHeadRequestForOneDpy(NVDpyEvoRec *pDpyEvo,
                                      NVSurfaceEvoPtr pSurfaceEvo,
                                      struct NvKmsSetModeParams *pParams,
                                      const NvU32 dispIndex,
-                                     NvU32 *pAvailableHeadsMask)
+                                     NvU32 *pAvailableApiHeadsMask)
 {
     NvBool ret = FALSE;
-    const NvU32 possibleHeads = *pAvailableHeadsMask &
-                                 pDpyEvo->pConnectorEvo->validHeadMask;
+    const NvU32 apiHead = PickApiHead(pDpyEvo, *pAvailableApiHeadsMask);
 
-    if (possibleHeads == 0 || pDpyEvo->isVrHmd) {
+    if ((apiHead == NV_INVALID_HEAD) || pDpyEvo->isVrHmd) {
         goto done;
     }
 
-    const NvU32 head = BIT_IDX_32(LOWESTBIT(possibleHeads));
-
     NVDispEvoRec *pDispEvo = pDpyEvo->pDispEvo;
     struct NvKmsSetModeRequest *pRequest = &pParams->request;
     struct NvKmsSetModeOneDispRequest *pRequestDisp =
         &pRequest->disp[dispIndex];
     struct NvKmsSetModeOneHeadRequest *pRequestHead =
-        &pRequestDisp->head[head];
+        &pRequestDisp->head[apiHead];
 
     NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
 
@@ -298,7 +356,7 @@ ConstructModeOneHeadRequestForOneDpy(NVDpyEvoRec *pDpyEvo,
                                 NULL /* Use default Mode */,
                                 NULL /* Use default ViewPortSizeIn */,
                                 NULL /* Use default ViewPortPointIn */,
-                                head,
+                                apiHead,
                                 pRequestHead)) {
         goto done;
     }
@@ -331,7 +389,7 @@ ConstructModeOneHeadRequestForOneDpy(NVDpyEvoRec *pDpyEvo,
         pRequestHead->viewPortOutSpecified = TRUE;
     }
 
-    *pAvailableHeadsMask &= ~NVBIT(head);
+    *pAvailableApiHeadsMask &= ~NVBIT(apiHead);
 
     ret = TRUE;
 
@@ -492,7 +550,7 @@ ConstructModeRequestForTiledDisplay(const NVDispEvoRec *pDispEvo,
                                     struct NvKmsSetModeParams *pParams,
                                     const NvU32 dispIndex,
                                     NVDpyIdList tiledDisplayDpysList,
-                                    NvU32 *pAvailableHeadsMask)
+                                    NvU32 *pAvailableApiHeadsMask)
 {
     NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
     /*
@@ -516,16 +574,16 @@ ConstructModeRequestForTiledDisplay(const NVDispEvoRec *pDispEvo,
     struct NvKmsSetModeRequest *pRequest = &pParams->request;
     struct NvKmsSetModeOneDispRequest *pRequestDisp =
         &pRequest->disp[dispIndex];
-    NvU32 firstClaimedHead = NV_INVALID_HEAD;
-    NvU32 claimedHeadMask = 0x0;
+    NvU32 firstClaimedApiHead = NV_INVALID_HEAD;
+    NvU32 claimedApiHeadMask = 0x0;
     NVDpyEvoRec *pDpyEvo;
-    NvU32 head;
+    NvU32 apiHead;
 
     /*
      * Return failure if not enough number of heads available to construct
      * modeset request for Tiled-Display.
      */
-    if (nvPopCount32(*pAvailableHeadsMask) <
+    if (nvPopCount32(*pAvailableApiHeadsMask) <
         nvCountDpyIdsInDpyIdList(tiledDisplayDpysList)) {
         return FALSE;
     }
@@ -546,20 +604,20 @@ ConstructModeRequestForTiledDisplay(const NVDispEvoRec *pDispEvo,
             .x = pDpyDisplayIdInfo->tile_location.x * viewPortSizeIn.width,
             .y = pDpyDisplayIdInfo->tile_location.y * viewPortSizeIn.height
         };
-        const NvU32 possibleHeads = *pAvailableHeadsMask &
-                                     pDpyEvo->pConnectorEvo->validHeadMask &
-                                     ~claimedHeadMask;
+        const NvU32 localAvailableApiHeadsMask =
+            *pAvailableApiHeadsMask & ~claimedApiHeadMask;
+        const NvU32 apiHead = PickApiHead(pDpyEvo,
+            localAvailableApiHeadsMask);
 
-        if (possibleHeads == 0 || pDpyEvo->isVrHmd) {
+        if ((apiHead == NV_INVALID_HEAD) || pDpyEvo->isVrHmd) {
             goto failed;
         }
 
-        const NvU32 head = BIT_IDX_32(LOWESTBIT(possibleHeads));
         struct NvKmsSetModeOneHeadRequest *pRequestHead =
-            &pRequestDisp->head[head];
+            &pRequestDisp->head[apiHead];
         struct NvKmsMode mode;
 
-        if (firstClaimedHead == NV_INVALID_HEAD) {
+        if (firstClaimedApiHead == NV_INVALID_HEAD) {
             /*
              * Find mode of native dimensions reported in Tiled-Display
              * information.
@@ -572,20 +630,20 @@ ConstructModeRequestForTiledDisplay(const NVDispEvoRec *pDispEvo,
                 goto failed;
             }
 
-            firstClaimedHead = head;
+            firstClaimedApiHead = apiHead;
         } else {
             /* All tiles should support same set of modes */
-            mode = pRequestDisp->head[firstClaimedHead].mode;
+            mode = pRequestDisp->head[firstClaimedApiHead].mode;
         }
 
-        claimedHeadMask |= NVBIT(head);
+        claimedApiHeadMask |= NVBIT(apiHead);
 
         if (!InitModeOneHeadRequest(pDpyEvo,
                                     pSurfaceEvo,
                                     &mode,
                                     &viewPortSizeIn,
                                     &viewPortPointIn,
-                                    head,
+                                    apiHead,
                                     pRequestHead)) {
             goto failed;
         }
@@ -601,19 +659,19 @@ ConstructModeRequestForTiledDisplay(const NVDispEvoRec *pDispEvo,
                           FALSE /* doRasterLock */)) {
         goto failed;
     }
-    *pAvailableHeadsMask &= ~claimedHeadMask;
+    *pAvailableApiHeadsMask &= ~claimedApiHeadMask;
 
     return TRUE;
 
 failed:
 
-    for (head = 0; head < ARRAY_LEN(pRequestDisp->head); head++) {
-        if ((NVBIT(head) & claimedHeadMask) == 0x0) {
+    for (apiHead = 0; apiHead < ARRAY_LEN(pRequestDisp->head); apiHead++) {
+        if ((NVBIT(apiHead) & claimedApiHeadMask) == 0x0) {
             continue;
         }
-        nvkms_memset(&pRequestDisp->head[head],
+        nvkms_memset(&pRequestDisp->head[apiHead],
                      0,
-                     sizeof(pRequestDisp->head[head]));
+                     sizeof(pRequestDisp->head[apiHead]));
     }
 
     return FALSE;
@@ -625,12 +683,15 @@ static NvBool isDpMSTModeActiveOnAnyConnector(NVDevEvoPtr pDevEvo)
     NVDispEvoPtr pDispEvo;
 
     FOR_ALL_EVO_DISPLAYS(pDispEvo, i, pDevEvo) {
-        NvU32 head;
+        NvU32 apiHead;
 
-        for (head = 0; head < pDevEvo->numHeads; head++) {
-            const NVDispHeadStateEvoRec *pHeadState =
-                &pDispEvo->headState[head];
-            const NVConnectorEvoRec *pConnectorEvo = pHeadState->pConnectorEvo;
+        for (apiHead = 0; apiHead < pDevEvo->numApiHeads; apiHead++) {
+            const NVDispApiHeadStateEvoRec *pApiHeadState =
+                &pDispEvo->apiHeadState[apiHead];
+            const NVDpyEvoRec *pDpyEvo =
+                nvGetOneArbitraryDpyEvo(pApiHeadState->activeDpys, pDispEvo);
+            const NVConnectorEvoRec *pConnectorEvo = (pDpyEvo != NULL) ?
+                pDpyEvo->pConnectorEvo : NULL;
 
             if ((pConnectorEvo != NULL) &&
                     nvConnectorUsesDPLib(pConnectorEvo)) {
@@ -655,12 +716,14 @@ static NvBool isDpMSTModeActiveOnAnyConnector(NVDevEvoPtr pDevEvo)
  * If a framebuffer console surface was successfully imported from RM, then use
  * the core channel to set a mode that displays it.
  *
- * Enables as many heads as possible in a clone configuration. In first pass
- * for connected boot dpys and in second pass for other remaining dpys:
+ * This enables as many heads as possible in a clone configuration.
+ * In the first pass we select connected active dpys, in the second pass
+ * any other connected boot dpys, and in a third pass any other
+ * remaining connected dpys:
  *
- *   1. Populate modeset request to enable given dpy.
+ *   1. Populate modeset request to enable the given dpy.
  *
- *   2. Do modeset request validation, if fails then disable scaling. If
+ *   2. Do modeset request validation, if it fails then disable scaling. If
  *   modeset request validation fails even after disabling scaling then do not
  *   enable that dpy.
  *
@@ -724,13 +787,17 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
     // Construct the request.
     //
     // To start with, try to enable as many connected dpys as possible,
-    // preferring boot displays first.
+    // preferring the connected active displays first.
     struct NvKmsSetModeRequest *pRequest = &params->request;
     NvBool foundDpysConfigForConsoleRestore = FALSE;
 
     FOR_ALL_EVO_DISPLAYS(pDispEvo, dispIndex, pDevEvo) {
-        NvU32 availableHeadsMask = NVBIT(pDevEvo->numHeads) - 1;
+        NvU32 availableApiHeadsMask = NVBIT(pDevEvo->numApiHeads) - 1;
         NVDpyIdList connectedDpys = UpdateConnectedDpys(pDispEvo);
+        const NVDpyIdList activeDpys = nvActiveDpysOnDispEvo(pDispEvo);
+        const NVDpyIdList connectedActiveDpys =
+            nvIntersectDpyIdListAndDpyIdList(connectedDpys,
+                                             activeDpys);
         const NVDpyIdList connectedBootDpys =
             nvIntersectDpyIdListAndDpyIdList(connectedDpys,
                                              pDispEvo->bootDisplays);
@@ -739,7 +806,7 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
         int pass;
 
         pRequest->requestedDispsBitMask |= NVBIT(dispIndex);
-        pRequestDisp->requestedHeadsBitMask = availableHeadsMask;
+        pRequestDisp->requestedHeadsBitMask = availableApiHeadsMask;
 
         // Only enable heads on the subdevice that actually contains the
         // console.
@@ -749,19 +816,24 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
 
         NVDpyIdList handledDpysList = nvEmptyDpyIdList();
 
-        for (pass = 0; pass < 2; pass++) {
+        for (pass = 0; pass < 3; pass++) {
             NVDpyIdList candidateDpys;
             NVDpyEvoPtr pDpyEvo;
 
-            if (availableHeadsMask == 0) {
+            if (availableApiHeadsMask == 0) {
                 break;
             }
 
             if (pass == 0) {
-                candidateDpys = connectedBootDpys;
-            } else {
+                candidateDpys = connectedActiveDpys;
+            } else if (pass == 1) {
+                candidateDpys = nvDpyIdListMinusDpyIdList(connectedBootDpys,
+                    connectedActiveDpys);
+            } else if (pass == 2) {
                 candidateDpys = nvDpyIdListMinusDpyIdList(connectedDpys,
                                                           connectedBootDpys);
+                candidateDpys = nvDpyIdListMinusDpyIdList(candidateDpys,
+                                                          connectedActiveDpys);
             }
 
             FOR_ALL_EVO_DPYS(pDpyEvo, candidateDpys, pDispEvo) {
@@ -773,7 +845,7 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
                         &pDpyEvo->parsedEdid.info.ext_displayid : NULL;
                 NvBool done = FALSE;
 
-                if (availableHeadsMask == 0) {
+                if (availableApiHeadsMask == 0) {
                     break;
                 }
 
@@ -807,7 +879,7 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
                                                     params,
                                                     dispIndex,
                                                     tiledDisplayInfo.detectedDpysList,
-                                                    &availableHeadsMask);
+                                                    &availableApiHeadsMask);
                     isTiledDisplayEnable = done;
                 }
 
@@ -828,7 +900,7 @@ NvBool nvEvoRestoreConsole(NVDevEvoPtr pDevEvo, const NvBool allowMST)
                                                     pSurfaceEvo,
                                                     params,
                                                     dispIndex,
-                                                    &availableHeadsMask);
+                                                    &availableApiHeadsMask);
                     isTiledDisplayEnable =
                         done && tiledDisplayInfo.isCapToScaleSingleTile;
                 }
@@ -875,7 +947,7 @@ done:
 
     /* If console restore failed then simply shut down all heads */
     if (!ret) {
-        nvShutDownHeads(pDevEvo, NULL /* pTestFunc, shut down all heads */);
+        nvShutDownApiHeads(pDevEvo, NULL /* pTestFunc, shut down all heads */);
     }
 
     // If restoring the console from here succeeded, then skip triggering RM's
