@@ -89,13 +89,13 @@ NvBool nvGetCursorImageSurfaces(
 }
 
 static void
-SetCursorImage(NVDispEvoPtr pDispEvo,
-               const NvU32 head,
-               NVSurfaceEvoRec *pSurfaceEvoNew,
-               const struct NvKmsCompositionParams *pCursorCompParams)
+SetCursorImageOneHead(NVDispEvoPtr pDispEvo,
+                      const NvU32 head,
+                      NVSurfaceEvoRec *pSurfaceEvoNew,
+                      const struct NvKmsCompositionParams *pCursorCompParams,
+                      NVEvoUpdateState *pUpdateState)
 {
     NVDevEvoPtr pDevEvo = pDispEvo->pDevEvo;
-    NVEvoUpdateState updateState = { };
     const NvU32 sd = pDispEvo->displayOwner;
     NvBool changed = FALSE;
 
@@ -131,51 +131,79 @@ SetCursorImage(NVDispEvoPtr pDispEvo,
             pDevEvo,
             head,
             pDevEvo->gpus[sd].headState[head].cursor.pSurfaceEvo,
-            &updateState,
+            pUpdateState,
             &pDevEvo->gpus[sd].headState[head].cursor.cursorCompParams);
-        nvEvoUpdateAndKickOff(pDispEvo, FALSE, &updateState,
-                              TRUE /* releaseElv */);
         nvPopEvoSubDevMask(pDevEvo);
     }
+}
 
-    if (pSurfaceEvoNew) {
-        nvTriggerVrrUnstallSetCursorImage(pDispEvo, changed);
+static void
+SetCursorImage(NVDispEvoPtr pDispEvo,
+               const NvU32 apiHead,
+               NVSurfaceEvoRec *pSurfaceEvoNew,
+               const struct NvKmsCompositionParams *pCursorCompParams)
+{
+    NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
+    const NVDispApiHeadStateEvoRec *pApiHeadState =
+        &pDispEvo->apiHeadState[apiHead];
+    NvU32 head;
+    NVEvoUpdateState updateState = { };
+    NvBool changed = FALSE;
+
+    FOR_EACH_EVO_HW_HEAD_IN_MASK(pApiHeadState->hwHeadsMask, head) {
+        SetCursorImageOneHead(pDispEvo,
+                              head,
+                              pSurfaceEvoNew,
+                              pCursorCompParams,
+                              &updateState);
     }
+
+    if (!nvIsUpdateStateEmpty(pDevEvo, &updateState)) {
+        nvEvoUpdateAndKickOff(pDispEvo, FALSE, &updateState,
+                              TRUE /* releaseElv */);
+        changed = TRUE;
+    }
+
+    /*
+     * Unconditionally trigger an unstall: even if the cursor image or
+     * composition didn't change, clients setting the cursor image would expect
+     * a VRR unstall.  Also, if the cursor changed from an image to no image
+     * (i.e., hiding the cursor), that should trigger a VRR unstall, too.
+     */
+    nvTriggerVrrUnstallSetCursorImage(pDispEvo, changed);
 }
 
 static NvBool
 FlipCursorImage(NVDispEvoPtr pDispEvo,
                 const struct NvKmsPerOpenDev *pOpenDevice,
-                NvU32 head,
+                NvU32 apiHead,
                 const struct NvKmsSetCursorImageCommonParams *pImageParams)
 {
     const NvU32 sd = pDispEvo->displayOwner;
     NvBool ret;
-    struct NvKmsFlipParams *pFlipParams;
-    struct NvKmsFlipRequest *pFlipRequest;
+    struct NvKmsFlipRequestOneHead *pFlipHead =
+        nvCalloc(1, sizeof(*pFlipHead));
 
-    pFlipParams = nvCalloc(1, sizeof(*pFlipParams));
-    if (pFlipParams == NULL) {
+    if (pFlipHead == NULL) {
         return FALSE;
     }
 
-    pFlipRequest = &pFlipParams->request;
-
-    pFlipRequest->sd[sd].head[head].cursor.image = *pImageParams;
-    pFlipRequest->sd[sd].head[head].cursor.imageSpecified = TRUE;
-
-    pFlipRequest->sd[sd].requestedHeadsBitMask = NVBIT(head);
-
-    pFlipRequest->commit = TRUE;
+    pFlipHead->sd = sd;
+    pFlipHead->head = apiHead;
+    pFlipHead->flip.cursor.image = *pImageParams;
+    pFlipHead->flip.cursor.imageSpecified = TRUE;
 
     ret = nvFlipEvo(pDispEvo->pDevEvo,
                     pOpenDevice,
-                    pFlipRequest,
-                    &pFlipParams->reply,
+                    pFlipHead,
+                    1     /* numFlipHeads */,
+                    TRUE  /* commit */,
+                    FALSE /* allowVrr */,
+                    NULL  /* pReply */,
                     FALSE /* skipUpdate */,
                     FALSE /* allowFlipLock */);
 
-    nvFree(pFlipParams);
+    nvFree(pFlipHead);
 
     return ret;
 }
@@ -184,7 +212,7 @@ NvBool nvSetCursorImage(
     NVDispEvoPtr pDispEvo,
     const struct NvKmsPerOpenDev *pOpenDevice,
     const NVEvoApiHandlesRec *pOpenDevSurfaceHandles,
-    NvU32 head,
+    NvU32 apiHead,
     const struct NvKmsSetCursorImageCommonParams *pParams)
 {
     NVDevEvoPtr pDevEvo = pDispEvo->pDevEvo;
@@ -206,25 +234,32 @@ NvBool nvSetCursorImage(
             pDevEvo->capsBits,
             NV5070_CTRL_SYSTEM_CAPS_BUG_2052012_GLITCHY_MCLK_SWITCH)) {
         const NvU32 sd = pDispEvo->displayOwner;
+        const NVDispApiHeadStateEvoRec *pApiHeadState =
+            &pDispEvo->apiHeadState[apiHead];
+        NvU32 head;
 
-        NVSurfaceEvoPtr pSurfaceEvoOld =
-            pDevEvo->gpus[sd].headState[head].cursor.pSurfaceEvo;
+        FOR_EACH_EVO_HW_HEAD_IN_MASK(pApiHeadState->hwHeadsMask, head) {
+            NVSurfaceEvoPtr pSurfaceEvoOld =
+                pDevEvo->gpus[sd].headState[head].cursor.pSurfaceEvo;
 
-        if ((pSurfaceEvoOld != pSurfaceEvoNew) &&
-            (pSurfaceEvoOld == NULL || pSurfaceEvoNew == NULL)) {
-            flipCursorImage = TRUE;
+            if ((pSurfaceEvoOld != pSurfaceEvoNew) &&
+                (pSurfaceEvoOld == NULL || pSurfaceEvoNew == NULL)) {
+                flipCursorImage = TRUE;
+                break;
+            }
         }
     }
 
     if (flipCursorImage) {
         return FlipCursorImage(pDispEvo,
-                               pOpenDevice, head, pParams);
+                               pOpenDevice, apiHead, pParams);
     }
 
     SetCursorImage(pDispEvo,
-                   head,
+                   apiHead,
                    pSurfaceEvoNew,
                    &pParams->cursorCompParams);
+
     return TRUE;
 }
 
@@ -244,19 +279,27 @@ void nvEvoMoveCursorInternal(NVDispEvoPtr pDispEvo,
     }
 }
 
-void nvEvoMoveCursor(NVDispEvoPtr pDispEvo, NvU32 head,
-                     const struct NvKmsMoveCursorCommonParams *pParams)
+void nvMoveCursor(NVDispEvoPtr pDispEvo, const NvU32 apiHead,
+                  const struct NvKmsMoveCursorCommonParams *pParams)
 {
     NVDevEvoPtr pDevEvo = pDispEvo->pDevEvo;
-    const NvU32 sd = pDispEvo->displayOwner;
+    const NVDispApiHeadStateEvoRec *pApiHeadState =
+        &pDispEvo->apiHeadState[apiHead];
+    NvU32 head;
 
     /* XXX NVKMS TODO: validate x,y against current viewport in? */
 
-    pDevEvo->gpus[sd].headState[head].cursor.x = pParams->x;
-    pDevEvo->gpus[sd].headState[head].cursor.y = pParams->y;
+    nvAssert(apiHead != NV_INVALID_HEAD);
 
-    nvEvoMoveCursorInternal(pDispEvo,
-                            head, pParams->x, pParams->y);
+    FOR_EACH_EVO_HW_HEAD_IN_MASK(pApiHeadState->hwHeadsMask, head) {
+        const NvU32 sd = pDispEvo->displayOwner;
+
+        pDevEvo->gpus[sd].headState[head].cursor.x = pParams->x;
+        pDevEvo->gpus[sd].headState[head].cursor.y = pParams->y;
+
+        nvEvoMoveCursorInternal(pDispEvo,
+                                head, pParams->x, pParams->y);
+    }
 }
 
 // Allocate and map cursor position PIO channels
