@@ -225,6 +225,7 @@ NV_STATUS  NV_API_CALL nv_vgpu_get_type_info(
 {
     THREAD_STATE_NODE threadState;
     OBJSYS *pSys = SYS_GET_INSTANCE();
+    OBJGPU *pGpu = NULL;
     KernelVgpuMgr *pKernelVgpuMgr = SYS_GET_KERNEL_VGPUMGR(pSys);
     NV_STATUS rmStatus = NV_OK;
     VGPU_TYPE *vgpuTypeInfo;
@@ -237,6 +238,14 @@ NV_STATUS  NV_API_CALL nv_vgpu_get_type_info(
     // LOCK: acquire API lock
     if ((rmStatus = rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_HYPERVISOR)) == NV_OK)
     {
+        pGpu = NV_GET_NV_PRIV_PGPU(pNv);
+        if (pGpu == NULL)
+        {
+            NV_PRINTF(LEVEL_ERROR, "%s GPU handle is not valid \n", __FUNCTION__);
+            rmStatus = NV_ERR_INVALID_STATE;
+            goto exit;
+        }
+
         if ((rmStatus = kvgpumgrGetPgpuIndex(pKernelVgpuMgr, pNv->gpu_id, &pgpuIndex)) ==
             NV_OK)
         {
@@ -438,11 +447,14 @@ NV_STATUS  NV_API_CALL nv_vgpu_create_request(
     const NvU8 *pMdevUuid,
     NvU32 vgpuTypeId,
     NvU16 *vgpuId,
-    NvU32 gpuPciBdf
+    NvU32 gpuPciBdf,
+    NvBool *is_driver_vm
 )
 {
     THREAD_STATE_NODE threadState;
+    OBJSYS        *pSys        = SYS_GET_INSTANCE();
     void          *fp          = NULL;
+    OBJHYPERVISOR *pHypervisor = SYS_GET_HYPERVISOR(pSys);
     NV_STATUS     rmStatus     = NV_OK;
 
     NV_ENTER_RM_RUNTIME(sp,fp);
@@ -454,6 +466,8 @@ NV_STATUS  NV_API_CALL nv_vgpu_create_request(
         rmStatus = kvgpumgrCreateRequestVgpu(pNv->gpu_id, pMdevUuid,
                                              vgpuTypeId, vgpuId, gpuPciBdf);
 
+        *is_driver_vm = pHypervisor->getProperty(pHypervisor, PDB_PROP_HYPERVISOR_DRIVERVM_ENABLED);
+
         // UNLOCK: release API lock
         rmapiLockRelease();
     }
@@ -903,7 +917,7 @@ NV_STATUS osVgpuRegisterMdev
 )
 {
     NV_STATUS status = NV_OK;
-    vgpu_vfio_info vgpu_info = {0};
+    vgpu_vfio_info vgpu_info;
     OBJSYS *pSys = SYS_GET_INSTANCE();
     KernelVgpuMgr *pKernelVgpuMgr = SYS_GET_KERNEL_VGPUMGR(pSys);
     KERNEL_PHYS_GPU_INFO *pPhysGpuInfo;
@@ -921,22 +935,12 @@ NV_STATUS osVgpuRegisterMdev
     status = os_alloc_mem((void **)&vgpu_info.vgpuTypeIds,
                           ((vgpu_info.numVgpuTypes) * sizeof(NvU32)));
     if (status != NV_OK)
-        goto free_mem;
-
-    status = os_alloc_mem((void **)&vgpu_info.vgpuNames,
-                          ((vgpu_info.numVgpuTypes) * sizeof(char *)));
-    if (status != NV_OK)
-        goto free_mem;
+        return status;
 
     vgpu_info.nv = pOsGpuInfo;
     for (i = 0; i < pPhysGpuInfo->numVgpuTypes; i++)
     {
-        status = os_alloc_mem((void *)&vgpu_info.vgpuNames[i], (VGPU_STRING_BUFFER_SIZE * sizeof(char)));
-        if (status != NV_OK)
-            goto free_mem;
-
         vgpu_info.vgpuTypeIds[i] = pPhysGpuInfo->vgpuTypes[i]->vgpuTypeId;
-        os_snprintf((char *) vgpu_info.vgpuNames[i], VGPU_STRING_BUFFER_SIZE, "%s\n", pPhysGpuInfo->vgpuTypes[i]->vgpuName);
     }
 
     if ((!pPhysGpuInfo->sriovEnabled) || 
@@ -966,22 +970,7 @@ NV_STATUS osVgpuRegisterMdev
         }
     }
 
-free_mem:
-    if (vgpu_info.vgpuTypeIds)
-        os_free_mem(vgpu_info.vgpuTypeIds);
-
-    if (vgpu_info.vgpuNames)
-    {
-        for (i = 0; i < pPhysGpuInfo->numVgpuTypes; i++)
-        {
-            if (vgpu_info.vgpuNames[i])
-            {
-                os_free_mem(vgpu_info.vgpuNames[i]);
-            }
-        }
-        os_free_mem(vgpu_info.vgpuNames);
-    }
-
+    os_free_mem(vgpu_info.vgpuTypeIds);
     return status;
 }
 
@@ -995,6 +984,9 @@ NV_STATUS osIsVgpuVfioPresent(void)
 void initVGXSpecificRegistry(OBJGPU *pGpu)
 {
     NvU32 data32;
+    osWriteRegistryDword(pGpu, NV_REG_STR_RM_POWER_FEATURES, 0x55455555);
+    osWriteRegistryDword(pGpu, NV_REG_STR_RM_INFOROM_DISABLE_BBX,
+                               NV_REG_STR_RM_INFOROM_DISABLE_BBX_YES);
 #if !defined(NVCPU_X86_64)
     osWriteRegistryDword(pGpu, NV_REG_STR_RM_BAR2_APERTURE_SIZE_MB, 4);
 #endif
