@@ -42,19 +42,6 @@
 #include "platform/chipset/chipset.h" // BUSINFO
 #include "gpu/nvbitmask.h" // NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX
 
-// VF related info for GSP-RM
-typedef struct GSP_VF_INFO
-{
-    NvU32  totalVFs;
-    NvU32  firstVFOffset;
-    NvU64  FirstVFBar0Address;
-    NvU64  FirstVFBar1Address;
-    NvU64  FirstVFBar2Address;
-    NvBool b64bitBar0;
-    NvBool b64bitBar1;
-    NvBool b64bitBar2;
-} GSP_VF_INFO;
-
 typedef struct GspSMInfo_t
 {
     NvU32 version;
@@ -111,6 +98,8 @@ typedef struct GspStaticConfigInfo_t
     NvBool bIsTitan;
     NvBool bIsTesla;
     NvBool bIsMobile;
+    NvBool bIsGc6Rtd3Allowed;
+    NvBool bIsGcOffRtd3Allowed;
 
     NvU64 bar1PdeBase;
     NvU64 bar2PdeBase;
@@ -166,8 +155,6 @@ typedef struct GspSystemInfo
     BUSINFO FHBBusInfo;
     BUSINFO chipsetIDInfo;
     ACPI_METHOD_DATA acpiMethodData;
-    NvU64 sysTimerOffsetNs;
-    GSP_VF_INFO gspVFInfo;
 } GspSystemInfo;
 
 
