@@ -28,6 +28,7 @@
 
 #include "nvport/nvport.h"
 #include "resserv/resserv.h"
+#include "resserv/rs_client.h"
 #include "nvoc/runtime.h"
 
 #ifdef __cplusplus
@@ -181,6 +182,11 @@ struct RsServer
 
     NvU32                     activeClientCount;
     NvU64                     activeResourceCount;
+
+    /// List of clients that are de-activated and pending free
+    RsDisabledClientList      disabledClientList;
+    RsClient                 *pNextDisabledClient;
+    PORT_SPINLOCK            *pDisabledClientListLock;
 };
 
 /**
@@ -262,20 +268,35 @@ NV_STATUS serverAllocClient(RsServer *pServer, RS_RES_ALLOC_PARAMS_INTERNAL *pPa
 NV_STATUS serverFreeClient(RsServer *pServer, RS_CLIENT_FREE_PARAMS* pParams);
 
 /**
- * Free a list of client handles. All resources references owned by the client will be
- * freed. All priority resources will be freed first across all listed clients.
+ * Mark a list of client handles as disabled. All CPU mappings owned by that
+ * client will be unmapped immediate, and the client will be marked as disabled.
+ * A call to @ref serverFreeDisabledClients will then free all such clients.
  *
  * It is invalid to attempt to free a client from a user other than the one
  * that allocated it.
  *
  * @param[in]   pServer This server instance
- * @param[in]   phClientList The list of client handles to free
+ * @param[in]   phClientList The list of client handles to disable
  * @param[in]   numClients The number of clients in the list
  * @param[in]   freeState User-defined free state
  * @param[in]   pSecInfo Security Info
  *
  */
-NV_STATUS serverFreeClientList(RsServer *pServer, NvHandle *phClientList, NvU32 numClients, NvU32 freeState, API_SECURITY_INFO *pSecInfo);
+NV_STATUS serverMarkClientListDisabled(RsServer *pServer, NvHandle *phClientList, NvU32 numClients, NvU32 freeState, API_SECURITY_INFO *pSecInfo);
+
+/**
+ * Frees all currently disabled clients. All resources references owned by 
+ * any of the clients will be freed.
+ * All priority resources will be freed first across all listed clients.
+ *
+ * NOTE: may return NV_WARN_MORE_PROCESSING_REQUIRED if not all clients were freed
+ *
+ * @param[in]   pServer   This server instance
+ * @param[in]   freeState User-defined free state
+ * @param[in]   limit     Max number of iterations to make returning; 0 means no limit
+ *
+ */
+NV_STATUS serverFreeDisabledClients(RsServer *pServer, NvU32 freeState, NvU32 limit);
 
 /**
  * Allocate a resource.
@@ -351,8 +372,67 @@ NvBool serverShareIterNext(RS_SHARE_ITERATOR*);
  * @param[in] pServer
  * @param[in] clientHandleBase
  */
- NV_STATUS serverSetClientHandleBase(RsServer *pServer, NvU32 clientHandleBase);
+NV_STATUS serverSetClientHandleBase(RsServer *pServer, NvU32 clientHandleBase);
 
+/**
+ * Deserialize parameters for servicing command
+ *
+ * @param[in] pCallContext
+ * @param[in] cmd
+ * @param[in] pParams
+ * @param[in] paramsSize
+ * @param[in] flags
+ */
+NV_STATUS serverDeserializeCtrlDown(CALL_CONTEXT *pCallContext, NvU32 cmd, void *pParams, NvU32 paramsSize, NvU32 *flags);
+
+/**
+ * Serialize parameters for servicing command
+ *
+ * @param[in] pCallContext
+ * @param[in] cmd
+ * @param[in] pParams
+ * @param[in] paramsSize
+ * @param[in] flags
+ */
+NV_STATUS serverSerializeCtrlDown(CALL_CONTEXT *pCallContext, NvU32 cmd, void *pParams, NvU32 paramsSize, NvU32 *flags);
+
+/**
+ * Deserialize parameters for returning from command
+ *
+ * @param[in] pCallContext
+ * @param[in] cmd
+ * @param[in] pParams
+ * @param[in] paramsSize
+ * @param[in] flags
+ */
+NV_STATUS serverDeserializeCtrlUp(CALL_CONTEXT *pCallContext, NvU32 cmd, void *pParams, NvU32 paramsSize, NvU32 *flags);
+
+/**
+ * Serialize parameters for returning from command
+ *
+ * @param[in] pCallContext
+ * @param[in] cmd
+ * @param[in] pParams
+ * @param[in] paramsSize
+ * @param[in] flags
+ */
+NV_STATUS serverSerializeCtrlUp(CALL_CONTEXT *pCallContext, NvU32 cmd, void *pParams, NvU32 paramsSize, NvU32 *flags);
+
+/**
+ * Unset flag for reserializing control before going to GSP
+ * Used if kernel control servicing passes params to GSP without changing them
+ *
+ * @param[in] pCallContext
+ */
+void serverDisableReserializeControl(CALL_CONTEXT *pCallContext);
+
+/**
+ * Free finn structures allocated for serializing/deserializing
+ *
+ * @param[in] pCallContext
+ * @param[in] pParams
+ */
+void serverFreeSerializeStructures(CALL_CONTEXT *pCallContext, void *pParams);
 
 /**
  * Return an available client handle for new client allocation
