@@ -62,7 +62,6 @@ static inline void NV_RM_RPC_ALLOC_EVENT(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_CONTROL(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_MANAGE_HW_RESOURCE_ALLOC(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_MANAGE_HW_RESOURCE_FREE(OBJGPU *pGpu, ...) { return; }
-static inline void NV_RM_RPC_MANAGE_HW_RESOURCE_BIND(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_SET_GUEST_SYSTEM_INFO(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_PERF_GET_PSTATE_INFO(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_PERF_GET_VIRTUAL_PSTATE_INFO(OBJGPU *pGpu, ...) { return; }
@@ -92,7 +91,6 @@ static inline void NV_RM_RPC_FREE_ON_ERROR(OBJGPU *pGpu, ...) { return; }
 
 // Simulation stubs
 static inline void NV_RM_RPC_SIM_LOAD_ESCAPE_FUNCTIONS(OBJOS *pOS, ...) { return; }
-static inline void NV_RM_RPC_SIM_ADD_DISP_CONTEXT_DMA(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_SIM_UPDATE_DISP_CONTEXT_DMA(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_SIM_DELETE_DISP_CONTEXT_DMA(OBJGPU *pGpu, ...) { return; }
 static inline void NV_RM_RPC_SIM_UPDATE_DISP_CHANNEL_INFO(OBJGPU *pGpu, ...) { return; }
@@ -440,19 +438,6 @@ static inline void NV_RM_RPC_SIM_UPDATE_DISP_CHANNEL_INFO(OBJGPU *pGpu, ...) { r
             status = NV_ERR_INSUFFICIENT_RESOURCES;                             \
     } while(0)
 
-#define NV_RM_RPC_MANAGE_HW_RESOURCE_BIND(pGpu, hclient, hdevice, hresource,    \
-                                          virtaddr, physaddr, status)           \
-    do                                                                          \
-    {                                                                           \
-        OBJRPC *pRpc = GPU_GET_RPC(pGpu);                                       \
-        NV_ASSERT(pRpc != NULL);                                                \
-        if ((status == NV_OK) && (pRpc != NULL))                                \
-            status = RmRpcHwResourceBind(pGpu, pRpc, hclient, hdevice,          \
-                                         hresource, virtaddr, physaddr);        \
-        if (pRpc == NULL)                                                       \
-            status = NV_ERR_INSUFFICIENT_RESOURCES;                             \
-    } while(0)
-
 #define NV_RM_RPC_SIM_LOAD_ESCAPE_FUNCTIONS(pos)                         \
     do                                                                   \
     {                                                                    \
@@ -464,17 +449,6 @@ static inline void NV_RM_RPC_SIM_UPDATE_DISP_CHANNEL_INFO(OBJGPU *pGpu, ...) { r
     while(0)
 
 /* outgoing updates to the plugin */
-#define NV_RM_RPC_SIM_ADD_DISP_CONTEXT_DMA(pGpu, hclient, pcontextdma, channelnum)      \
-    do                                                                                  \
-    {                                                                                   \
-        NV_STATUS status;                                                               \
-        SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)                                          \
-        status = RmRpcSimAddDisplayContextDma(pGpu, hclient, pcontextdma, channelnum);  \
-        NV_ASSERT(status == NV_OK);                                                     \
-        SLI_LOOP_END                                                                    \
-    }                                                                                   \
-    while(0)
-
 #define NV_RM_RPC_SIM_UPDATE_DISP_CONTEXT_DMA(pGpu, hclient, pcontextdma, physaddrnew,  \
                                               physlimitnew, pagesize, ptekind)          \
     do                                                                                  \
@@ -545,13 +519,13 @@ static inline void NV_RM_RPC_SIM_UPDATE_DISP_CHANNEL_INFO(OBJGPU *pGpu, ...) { r
             status = NV_ERR_INSUFFICIENT_RESOURCES;                             \
     } while(0)
 
-#define NV_RM_RPC_UNLOADING_GUEST_DRIVER(pGpu, status, bSuspend, bGc6Entering, newPMLevel)        \
+#define NV_RM_RPC_UNLOADING_GUEST_DRIVER(pGpu, status, bInPMTransition, bGc6Entering, newPMLevel)        \
     do                                                                                            \
     {                                                                                             \
         OBJRPC *pRpc = GPU_GET_RPC(pGpu);                                                         \
         NV_ASSERT(pRpc != NULL);                                                                  \
         if ((status == NV_OK) && (pRpc != NULL))                                                  \
-            status = rpcUnloadingGuestDriver_HAL(pGpu, pRpc, bSuspend, bGc6Entering, newPMLevel); \
+            status = rpcUnloadingGuestDriver_HAL(pGpu, pRpc, bInPMTransition, bGc6Entering, newPMLevel); \
         else if (pRpc == NULL)                                                                    \
             status = NV_ERR_INSUFFICIENT_RESOURCES;                                               \
     }                                                                                             \
@@ -703,13 +677,11 @@ static inline void NV_RM_RPC_SIM_UPDATE_DISP_CHANNEL_INFO(OBJGPU *pGpu, ...) { r
                             testData3, status)  do {} while(0)
 
 static inline NV_STATUS RmRpcSimFreeInfra(OBJGPU *pGpu, ...)                            { return NV_OK; }
-static inline NV_STATUS RmRpcSimAddDisplayContextDma(OBJGPU *pGpu, ...)                 { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcSimUpdateDisplayContextDma(OBJGPU *pGpu, ...)              { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcSimDeleteDisplayContextDma(OBJGPU *pGpu, ...)              { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcSimUpdateDispChannelInfo(OBJGPU *pGpu, ...)                { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcHwResourceAlloc(OBJGPU *pGpu, ...)                         { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcHwResourceFree(OBJGPU *pGpu, ...)                          { return NV_ERR_NOT_SUPPORTED; }
-static inline NV_STATUS RmRpcHwResourceBind(OBJGPU *pGpu, ...)                          { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcPerfGetPstateInfo(OBJGPU *pGpu, ...)                       { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcPerfGetCurrentPstate(OBJGPU *pGpu, ...)                    { return NV_ERR_NOT_SUPPORTED; }
 static inline NV_STATUS RmRpcPerfGetVirtualPstateInfo(OBJGPU *pGpu, ...)                { return NV_ERR_NOT_SUPPORTED; }
