@@ -33,6 +33,7 @@
 #include "core/locks.h"
 #include "gpu/gpu.h"
 #include "gpu/bif/kernel_bif.h"
+#include "gpu/subdevice/subdevice.h"
 #include "gpu/mem_mgr/mem_desc.h"
 #include "nvVer.h"
 #include "nvBldVer.h"
@@ -50,9 +51,7 @@
 #include "vgpu/rpc.h"
 #include "vgpu/vgpu_events.h"
 #include "virtualization/hypervisor/hypervisor.h"
-#include "finn_rm_api.h"
 #include "os/os.h"
-#include "objtmr.h"
 
 #define SDK_ALL_CLASSES_INCLUDE_FULL_HEADER
 #include "g_allclasses.h"
@@ -74,6 +73,7 @@
 
 #include "g_rpc_private.h"
 
+#include "g_finn_rm_api.h"
 
 #include "gpu/gsp/message_queue_priv.h"
 
@@ -101,7 +101,6 @@ typedef struct rpc_vgx_version
 } RPC_VGX_VERSION;
 
 static RPC_VGX_VERSION rpcVgxVersion;
-static NvBool bSkipRpcVersionHandshake = NV_FALSE;
 
 void rpcSetIpVersion(OBJGPU *pGpu, OBJRPC *pRpc, NvU32 ipVersion)
 {
@@ -147,7 +146,6 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
 
     // should not be called in broadcast mode
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
-    NV_CHECK(LEVEL_ERROR, rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     if (bProfileRPC)
     {
@@ -179,9 +177,9 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
     status = rpcSendMessage(pGpu, pRpc);
     if (status != NV_OK)
     {
-        NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-            "rpcSendMessage failed with status 0x%08x for fn %d!\n",
-            status, vgpu_rpc_message_header_v->function);
+        NV_PRINTF(LEVEL_ERROR, "rpcSendMessage failed with status 0x%08x for fn %d!\n",
+                  status, vgpu_rpc_message_header_v->function);
+        NV_ASSERT(0);
         //
         // It has been observed that returning NV_ERR_BUSY_RETRY in a bad state (RPC
         // buffers full and not being serviced) can make things worse, i.e. turn RPC
@@ -196,16 +194,15 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
     {
         if (status == NV_ERR_TIMEOUT)
         {
-            NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-                "rpcRecvPoll timedout for fn %d!\n",
-                 vgpu_rpc_message_header_v->function);
+            NV_PRINTF(LEVEL_ERROR, "rpcRecvPoll timedout for fn %d!\n",
+                      vgpu_rpc_message_header_v->function);
         }
         else
         {
-            NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-                "rpcRecvPoll failed with status 0x%08x for fn %d!\n",
-                 status, vgpu_rpc_message_header_v->function);
+            NV_PRINTF(LEVEL_ERROR, "rpcRecvPoll failed with status 0x%08x for fn %d!\n",
+                      status, vgpu_rpc_message_header_v->function);
         }
+        NV_ASSERT(0);
         return status;
     }
 
@@ -365,8 +362,7 @@ static NV_STATUS _issueRpcLarge
 
     pBuf8 = (NvU8 *)pBuffer;
     remainingSize = bufSize;
-    entryLength = NV_MIN(bufSize, vgpu_rpc_message_header_v->length);
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength <= pRpc->maxRpcSize, NV_ERR_INVALID_STATE);
+    entryLength = NV_MIN(bufSize, pRpc->maxRpcSize);
 
     if (((NvU8 *)vgpu_rpc_message_header_v != pBuf8) && bBidirectional)
       portMemCopy(pBuf8, entryLength, vgpu_rpc_message_header_v, entryLength);
@@ -397,11 +393,8 @@ static NV_STATUS _issueRpcLarge
                 NV_ASSERT(0);
                 return nvStatus;
             }
-
-            entryLength = vgpu_rpc_message_header_v->length;
+            entryLength = vgpu_rpc_message_header_v->length - sizeof(rpc_message_header_v);
             NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength <= pRpc->maxRpcSize, NV_ERR_INVALID_STATE);
-            NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength >= sizeof(rpc_message_header_v), NV_ERR_INVALID_STATE);
-            entryLength -= sizeof(rpc_message_header_v);
 
             if (entryLength > remainingSize)
                 entryLength = remainingSize;
@@ -679,17 +672,10 @@ NV_STATUS RmRpcSetGuestSystemInfo(OBJGPU *pGpu, OBJRPC *pRpc)
     {
         if (rpcVgxVersion.majorNum != 0)
         {
-			if (pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_PM_RESUME_CODEPATH) && !bSkipRpcVersionHandshake)
-			{
-				bSkipRpcVersionHandshake = NV_TRUE;
-			}
-			else
-			{
-				NV_PRINTF(LEVEL_INFO,
-						  "NVRM_RPC: Skipping RPC version handshake for instance 0x%x\n",
-						  gpuGetInstance(pGpu));
-				goto skip_ver_handshake;
-			}
+            NV_PRINTF(LEVEL_INFO,
+                      "NVRM_RPC: Skipping RPC version handshake for instance 0x%x\n",
+                      gpuGetInstance(pGpu));
+            goto skip_ver_handshake;
         }
         else
         {
@@ -835,7 +821,7 @@ skip_ver_handshake:
     return status;
 }
 
-NV_STATUS rpcUnloadingGuestDriver_v03_00(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bSuspend, NvBool bGc6Entering, NvU32 newPMLevel)
+NV_STATUS rpcUnloadingGuestDriver_v03_00(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bInPMTransition, NvBool bGc6Entering, NvU32 newPMLevel)
 {
     NV_STATUS status = NV_OK;
 
@@ -849,7 +835,7 @@ NV_STATUS rpcUnloadingGuestDriver_v03_00(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bSus
 }
 
 
-NV_STATUS rpcUnloadingGuestDriver_v1F_07(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bSuspend, NvBool bGc6Entering, NvU32 newPMLevel)
+NV_STATUS rpcUnloadingGuestDriver_v1F_07(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bInPMTransition, NvBool bGc6Entering, NvU32 newPMLevel)
 {
     NV_STATUS status = NV_OK;
     NvU32 headerLength = sizeof(rpc_message_header_v) + sizeof(rpc_unloading_guest_driver_v1F_07);
@@ -866,7 +852,7 @@ NV_STATUS rpcUnloadingGuestDriver_v1F_07(OBJGPU *pGpu, OBJRPC *pRpc, NvBool bSus
     status = rpcWriteCommonHeader(pGpu, pRpc, NV_VGPU_MSG_FUNCTION_UNLOADING_GUEST_DRIVER, sizeof(rpc_unloading_guest_driver_v1F_07));
     if (status != NV_OK)
         return status;
-    rpc_message->unloading_guest_driver_v1F_07.bSuspend = bSuspend;
+    rpc_message->unloading_guest_driver_v1F_07.bInPMTransition = bInPMTransition;
     rpc_message->unloading_guest_driver_v1F_07.bGc6Entering = bGc6Entering;
     rpc_message->unloading_guest_driver_v1F_07.newLevel = newPMLevel;
 
@@ -1319,19 +1305,6 @@ NV_STATUS rpcGspSetSystemInfo_v17_00
         rpcInfo->bUpstreamL1PorMobileOnly = pGpu->getProperty(pGpu, PDB_PROP_GPU_UPSTREAM_PORT_L1_POR_MOBILE_ONLY);
         rpcInfo->upstreamAddressValid     = pGpu->gpuClData.upstreamPort.addr.valid;
 
-        // Fill in VF related GPU flags
-        rpcInfo->gspVFInfo.totalVFs           = pGpu->sriovState.totalVFs;
-        rpcInfo->gspVFInfo.firstVFOffset      = pGpu->sriovState.firstVFOffset;
-        rpcInfo->gspVFInfo.FirstVFBar0Address = pGpu->sriovState.firstVFBarAddress[0];
-        rpcInfo->gspVFInfo.FirstVFBar1Address = pGpu->sriovState.firstVFBarAddress[1];
-        rpcInfo->gspVFInfo.FirstVFBar2Address = pGpu->sriovState.firstVFBarAddress[2];
-        rpcInfo->gspVFInfo.b64bitBar0         = pGpu->sriovState.b64bitVFBar0;
-        rpcInfo->gspVFInfo.b64bitBar1         = pGpu->sriovState.b64bitVFBar1;
-        rpcInfo->gspVFInfo.b64bitBar2         = pGpu->sriovState.b64bitVFBar2;
-
-        OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
-        rpcInfo->sysTimerOffsetNs = pTmr->sysTimerOffsetNs;
-
         status = _issueRpcAsync(pGpu, pRpc);
     }
 
@@ -1354,7 +1327,7 @@ NV_STATUS rpcSetRegistry_v17_00
 
     if (IS_GSP_CLIENT(pGpu))
     {
-        NvU32 regTableSize;
+        NvU32 regTableSize = 0;
         NvU32 totalSize;
         NvU32 remainingMessageSize;
         PACKED_REGISTRY_TABLE *pRegTable;
@@ -1400,18 +1373,20 @@ NV_STATUS rpcSetRegistry_v17_00
 
         status = osPackageRegistry(pGpu, pRegTable, &regTableSize);
         if (status != NV_OK)
-            return status;
+            goto fail;
 
         if (largeRpcBuffer != NULL)
         {
             status = _issueRpcAsyncLarge(pGpu, pRpc, totalSize, largeRpcBuffer);
-            portMemFree(largeRpcBuffer);
         }
         else
         {
             vgpu_rpc_message_header_v->length = totalSize;
             status = _issueRpcAsync(pGpu, pRpc);
         }
+
+    fail:
+        portMemFree(largeRpcBuffer);
     }
 
     return status;
@@ -1544,17 +1519,17 @@ NV_STATUS rpcRmApiControl_GSP
     NvU32 rpc_params_size;
     NvU32 total_size;
 
-    const NvU32 interface_id = (DRF_VAL(XXXX, _CTRL_CMD, _CLASS, cmd) << 8) |
-                               DRF_VAL(XXXX, _CTRL_CMD, _CATEGORY, cmd);
-    const NvU32 message_id = DRF_VAL(XXXX, _CTRL_CMD, _INDEX, cmd);
-    NvU8 *pSerBuffer = NULL;
-    NvU32 serializedSize = 0;
     NvU32 origParamsSize = paramsSize;
     NvU32 gpuMaskRelease = 0;
     NvU32 ctrlFlags = 0;
     NvU32 ctrlAccessRight = 0;
     NvBool bCacheable;
 
+    CALL_CONTEXT *pCallContext;
+    CALL_CONTEXT newContext;
+    NvU32 resCtrlFlags = NVOS54_FLAGS_NONE;
+    NvBool preSerialized = NV_FALSE;
+
     if (!rmDeviceGpuLockIsOwner(pGpu->gpuInstance))
     {
         NV_PRINTF(LEVEL_WARNING, "Calling RPC RmControl 0x%08x without adequate locks!\n", cmd);
@@ -1568,15 +1543,37 @@ NV_STATUS rpcRmApiControl_GSP
     rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight);
     bCacheable = rmapiControlIsCacheable(ctrlFlags, ctrlAccessRight, NV_TRUE);
 
-    // Attempt to calculate the serialized size of the param struct using FINN.
-    serializedSize = FinnRmApiGetSerializedSize(interface_id, message_id, pParamStructPtr);
-
-    // If successful, this is a serializable API and rpc_params->params is a serialized buffer.
-    // otherwise this is a flat API and paramsSize is the param struct size
-    if (serializedSize != 0)
+    pCallContext = resservGetTlsCallContext();
+    if (pCallContext == NULL || pCallContext->bReserialize)
     {
-        // Allocate twice the amount to account for the return buffer
-        paramsSize = 2 * serializedSize;
+        // This should only happen when using the internal physical RMAPI
+        NV_ASSERT_OR_RETURN(pRmApi == GPU_GET_PHYSICAL_RMAPI(pGpu), NV_ERR_INVALID_STATE);
+
+        portMemSet(&newContext, 0, sizeof(newContext));
+        pCallContext = &newContext;
+    }
+
+    if (pCallContext->pControlParams != NULL)
+    {
+        resCtrlFlags = pCallContext->pControlParams->flags;
+    }
+
+    if (resCtrlFlags & NVOS54_FLAGS_FINN_SERIALIZED)
+    {
+        preSerialized = NV_TRUE;
+    }
+    else
+    {
+        status = serverSerializeCtrlDown(pCallContext, cmd, pParamStructPtr, paramsSize, &resCtrlFlags);
+        if (status != NV_OK)
+            goto done;
+    }
+
+    // If this is a serializable API, rpc_params->params is a serialized buffer.
+    // otherwise this is a flat API and paramsSize is the param struct size
+    if (resCtrlFlags & NVOS54_FLAGS_FINN_SERIALIZED)
+    {
+        paramsSize = pCallContext->serializedSize;
         NV_ASSERT_OR_RETURN(!bCacheable, NV_ERR_INVALID_STATE);
     }
 
@@ -1613,24 +1610,11 @@ NV_STATUS rpcRmApiControl_GSP
         message_buffer_remaining = total_size - fixed_param_size;
     }
 
-    // If this is a serializable API, attempt to serialize the param struct using FINN,
-    // otherwise do a flat memcpy
-    if (serializedSize != 0)
+    if (resCtrlFlags & NVOS54_FLAGS_FINN_SERIALIZED)
     {
+        // Copy in serialized buffer from the call context
         rpc_params->serialized = NV_TRUE;
-        pSerBuffer = (NvU8 *)rpc_params->params;
-
-        // Serialize into the first half of the RPC buffer.
-        status = FinnRmApiSerializeDown(interface_id, message_id, pParamStructPtr,
-                                    &pSerBuffer, message_buffer_remaining);
-        if (status != NV_OK)
-        {
-            NV_PRINTF(LEVEL_ERROR,
-                      "GspRmControl: Serialization failed for cmd 0x%x with status %s (0x%x) at index 0x%llx\n",
-                      cmd, nvAssertStatusToString(status), status,
-                      (NvUPtr)(pSerBuffer - (NvU8 *)rpc_params->params));
-            goto done;
-        }
+        portMemCopy(rpc_params->params, message_buffer_remaining, pCallContext->pSerializedParams, pCallContext->serializedSize);
     }
     else
     {
@@ -1697,22 +1681,21 @@ NV_STATUS rpcRmApiControl_GSP
             goto done;
         }
 
-        // If FINN was used to serialize the params, they must be deserialized on the way back,
-        // otherwise do a flat memcpy
-        if (serializedSize != 0)
+        if (resCtrlFlags & NVOS54_FLAGS_FINN_SERIALIZED)
         {
-            NvU8 *pRetBuffer = pSerBuffer;
-
-            // Deserialize from the second half of the RPC buffer.
-            status = FinnRmApiDeserializeUp(&pSerBuffer, paramsSize / 2,
-                                            pParamStructPtr, origParamsSize);
-            if (status != NV_OK)
+            //
+            // If it was preserialized, copy it to call context for deserialization by caller
+            // Otherwise deserialize it because it was serialized here
+            //
+            if (preSerialized)
             {
-                NV_PRINTF(LEVEL_ERROR,
-                          "GspRmControl: Deserialization failed for cmd 0x%x with status %s (0x%x) at index 0x%llx\n",
-                          cmd, nvAssertStatusToString(status), status,
-                          (NvUPtr)(pSerBuffer - pRetBuffer));
-                goto done;
+                portMemCopy(pCallContext->pSerializedParams, pCallContext->serializedSize, rpc_params->params, rpc_params->paramsSize);
+            }
+            else
+            {
+                status = serverDeserializeCtrlUp(pCallContext, cmd, pParamStructPtr, origParamsSize, &resCtrlFlags);
+                if (status != NV_OK)
+                    goto done;
             }
         }
         else
@@ -1731,15 +1714,16 @@ NV_STATUS rpcRmApiControl_GSP
 
     if (status != NV_OK)
     {
-        NvBool bQuiet = NV_FALSE;
+        NvBool bSilentErrorReport = NV_FALSE;
         switch (status)
         {
             case NV_ERR_NOT_SUPPORTED:
             case NV_ERR_OBJECT_NOT_FOUND:
-                bQuiet = NV_TRUE;
+                bSilentErrorReport = NV_TRUE;
                 break;
         }
-        NV_PRINTF_COND((pRpc->bQuietPrints || bQuiet), LEVEL_INFO, LEVEL_WARNING,
+
+        NV_PRINTF_COND(bSilentErrorReport, LEVEL_INFO, LEVEL_WARNING,
             "GspRmControl failed: hClient=0x%08x; hObject=0x%08x; cmd=0x%08x; paramsSize=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
             hClient, hObject, cmd, paramsSize, rpc_params->status, status);
     }
@@ -1752,6 +1736,15 @@ done:
     // Free the local copy we might have allocated above
     portMemFree(large_message_copy);
 
+    //
+    // Free data structures if we serialized/deserialized here
+    // Also check for serialized flag here as we may be called directly from within another control call
+    //
+    if ((resCtrlFlags & NVOS54_FLAGS_FINN_SERIALIZED) && !preSerialized)
+    {
+        serverFreeSerializeStructures(pCallContext, pParamStructPtr);
+    }
+
     return status;
 }
 
@@ -1840,9 +1833,9 @@ NV_STATUS rpcRmApiAlloc_GSP
     }
     else
     {
-        NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-            "GspRmAlloc failed: hClient=0x%08x; hParent=0x%08x; hObject=0x%08x; hClass=0x%08x; paramsSize=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
-            hClient, hParent, hObject, hClass, paramsSize, rpc_params->status, status);
+        NV_PRINTF(LEVEL_ERROR,
+                  "GspRmAlloc failed: hClient=0x%08x; hParent=0x%08x; hObject=0x%08x; hClass=0x%08x; paramsSize=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
+                  hClient, hParent, hObject, hClass, paramsSize, rpc_params->status, status);
         status = rpc_params->status;
     }
 
@@ -1894,9 +1887,9 @@ NV_STATUS rpcRmApiDupObject_GSP
     status = _issueRpcAndWait(pGpu, pRpc);
     if (status != NV_OK)
     {
-        NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-            "GspRmDupObject failed: hClient=0x%08x; hParent=0x%08x; hObject=0x%08x; hClientSrc=0x%08x; hObjectSrc=0x%08x; flags=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
-             hClient, hParent, *phObject, hClientSrc, hObjectSrc, flags, rpc_params->status, status);
+        NV_PRINTF(LEVEL_ERROR,
+                  "GspRmDupObject failed: hClient=0x%08x; hParent=0x%08x; hObject=0x%08x; hClientSrc=0x%08x; hObjectSrc=0x%08x; flags=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
+                  hClient, hParent, *phObject, hClientSrc, hObjectSrc, flags, rpc_params->status, status);
     }
 done:
     if (gpuMaskRelease != 0)
@@ -1939,9 +1932,9 @@ NV_STATUS rpcRmApiFree_GSP
     status = _issueRpcAndWait(pGpu, pRpc);
     if (status != NV_OK)
     {
-        NV_PRINTF_COND(pRpc->bQuietPrints, LEVEL_INFO, LEVEL_ERROR,
-            "GspRmFree failed: hClient=0x%08x; hObject=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
-             hClient, hObject, rpc_params->status, status);
+        NV_PRINTF(LEVEL_ERROR,
+                  "GspRmFree failed: hClient=0x%08x; hObject=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
+                  hClient, hObject, rpc_params->status, status);
     }
 done:
     if (gpuMaskRelease != 0)
