@@ -81,8 +81,6 @@ NVLOG_LOGGER NvLogLogger =
 NV_STATUS
 nvlogInit(void *pData)
 {
-    NV_STATUS status = NV_OK;
-
     nvlogRegRoot = pData;
     portInitialize();
     NvLogLogger.mainLock = portSyncSpinlockCreate(portMemAllocatorGetGlobalNonPaged());
@@ -90,13 +88,8 @@ nvlogInit(void *pData)
     {
         return NV_ERR_INSUFFICIENT_RESOURCES;
     }
-    NvLogLogger.buffersLock = portSyncMutexCreate(portMemAllocatorGetGlobalNonPaged());
-    if (NvLogLogger.buffersLock == NULL)
-    {
-        return NV_ERR_INSUFFICIENT_RESOURCES;
-    }
     tlsInitialize();
-    return status;
+    return NV_OK;
 }
 
 void nvlogUpdate(void) {
@@ -105,30 +98,22 @@ void nvlogUpdate(void) {
 NV_STATUS
 nvlogDestroy(void)
 {
-    NV_STATUS status = NV_OK;
     NvU32 i;
 
+    tlsShutdown();
     for (i = 0; i < NVLOG_MAX_BUFFERS; i++)
     {
         nvlogDeallocBuffer(i, NV_TRUE);
     }
-
     if (NvLogLogger.mainLock != NULL)
     {
         portSyncSpinlockDestroy(NvLogLogger.mainLock);
         NvLogLogger.mainLock = NULL;
     }
-    if (NvLogLogger.buffersLock != NULL)
-    {
-        portSyncMutexDestroy(NvLogLogger.buffersLock);
-        NvLogLogger.buffersLock = NULL;
-    }
 
-    tlsShutdown();
     /// @todo Destructor should return void.
     portShutdown();
-
-    return status;
+    return NV_OK;
 }
 
 static NV_STATUS
@@ -243,7 +228,6 @@ nvlogAllocBuffer
         return status;
     }
 
-    portSyncMutexAcquire(NvLogLogger.buffersLock);
     portSyncSpinlockAcquire(NvLogLogger.mainLock);
 
     if (NvLogLogger.nextFree < NVLOG_MAX_BUFFERS)
@@ -265,7 +249,6 @@ nvlogAllocBuffer
         else break;
     }
     portSyncSpinlockRelease(NvLogLogger.mainLock);
-    portSyncMutexRelease(NvLogLogger.buffersLock);
 
     if (status != NV_OK)
     {
@@ -299,13 +282,11 @@ nvlogDeallocBuffer
                                  _YES, pBuffer->flags);
 
     while (pBuffer->threadCount > 0) { /*spin*/ }
-    portSyncMutexAcquire(NvLogLogger.buffersLock);
     portSyncSpinlockAcquire(NvLogLogger.mainLock);
       NvLogLogger.pBuffers[hBuffer] = NULL;
       NvLogLogger.nextFree = NV_MIN(hBuffer, NvLogLogger.nextFree);
       NvLogLogger.totalFree++;
     portSyncSpinlockRelease(NvLogLogger.mainLock);
-    portSyncMutexRelease(NvLogLogger.buffersLock);
 
     _deallocateNvlogBuffer(pBuffer);
 }
