@@ -46,12 +46,34 @@
 
 static const GPUCHILDPRESENT gpuChildrenPresent_AD102[] =
 {
+    { classId(OBJFUSE), 1, },
+    { classId(OBJSEQ), 1, },
+    { classId(GpuMutexMgr), 1, },
+    { classId(OBJTMR), 1, },
+    { classId(GraphicsManager), 1, },
+    { classId(MIGManager), 1, },
+    { classId(KernelMIGManager), 1, },
+    { classId(KernelGraphicsManager), 1, },
+    { classId(OBJVBIOS), 1, },
+    { classId(OBJDCB), 1, },
+    { classId(OBJGPIO), 1, },
+    { classId(I2c), 1, },
+    { classId(Spi), 1, },
+    { classId(KernelRc), 1, },
+    { classId(OBJRC), 1, },
+    { classId(OBJSTEREO), 1, },
+    { classId(Intr), 1, },
+    { classId(OBJINFOROM), 1, },
+    { classId(NvDebugDump), 1, },
+    { classId(SMDebugger), 1, },
+    { classId(OBJGPULOG), 1, },
+    { classId(OBJGPUMON), 1, },
     {classId(OBJSWENG), 1},
     {classId(OBJUVM), 1},
     {classId(OBJACR), 1},
     {classId(OBJBIF), 1},
     {classId(KernelBif), 1},
-    {classId(OBJNNE), 1},
+    {classId(Nne), 1},
     {classId(OBJBSP), 4},
     {classId(OBJBUS), 1},
     {classId(KernelBus), 1},
@@ -74,18 +96,10 @@ static const GPUCHILDPRESENT gpuChildrenPresent_AD102[] =
     {classId(OBJFIFO), 1},
     {classId(OBJGMMU), 1},
     {classId(KernelGmmu), 1},
-    {classId(OBJGPULOG), 1},
-    {classId(OBJGPUMON), 1},
-    {classId(GraphicsManager), 1},
-    {classId(MIGManager), 1},
-    {classId(KernelMIGManager), 1},
-    {classId(KernelGraphicsManager), 1},
     {classId(Graphics), 1},
     {classId(KernelGraphics), 1},
     {classId(OBJHDACODEC), 1},
     {classId(OBJHWPM), 1},
-    {classId(OBJINFOROM), 1},
-    {classId(Intr), 1},
     {classId(Lpwr   ), 1},
     {classId(OBJLSFM), 1},
     {classId(OBJMC), 1},
@@ -94,7 +108,6 @@ static const GPUCHILDPRESENT gpuChildrenPresent_AD102[] =
     {classId(SwIntr), 1},
     {classId(OBJNVJPG), 4},
     {classId(OBJMSENC), 3},
-    {classId(NvDebugDump), 1},
     {classId(KernelNvlink), 1},
     {classId(Nvlink), 1},
     {classId(Perf), 1},
@@ -105,10 +118,8 @@ static const GPUCHILDPRESENT gpuChildrenPresent_AD102[] =
     {classId(OBJSEC2), 1},
     {classId(Gsp), 1},
     {classId(Therm), 1},
-    {classId(OBJTMR), 1},
     {classId(OBJVOLT), 1},
     {classId(OBJGRIDDISPLAYLESS), 1},
-    {classId(OBJFAS), 1},
     {classId(OBJVMMU), 1},
     {classId(OBJOFA), 1 },
     {classId(KernelSec2), 1},
