@@ -393,10 +393,12 @@ kbusCommitBar2_KERNEL
     NvU32      flags
 )
 {
-    // we will initialize bar2 to the default big page size of the system
-    NV_ASSERT_OK_OR_RETURN(kbusInitVirtualBar2_HAL(pGpu, pKernelBus));
-    NV_ASSERT_OK_OR_RETURN(kbusSetupCpuPointerForBusFlush_HAL(pGpu, pKernelBus));
-
+    if (!KBUS_BAR0_PRAMIN_DISABLED(pGpu))
+    {
+        // we will initialize bar2 to the default big page size of the system
+        NV_ASSERT_OK_OR_RETURN(kbusInitVirtualBar2_HAL(pGpu, pKernelBus));
+        NV_ASSERT_OK_OR_RETURN(kbusSetupCpuPointerForBusFlush_HAL(pGpu, pKernelBus));
+    }
     return NV_OK;
 }
 
@@ -768,9 +770,9 @@ kbusPatchBar2Pdb_GSPCLIENT
 NvBool
 kbusCheckEngine_KERNEL
 (
-    OBJGPU *pGpu,
-    KernelBus *pKernelBus,
-    ENGDESCRIPTOR engDesc
+    OBJGPU        *pGpu,
+    KernelBus     *pKernelBus,
+    ENGDESCRIPTOR  engDesc
 )
 {
     NvU32     rmEngineCaps[NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX] = {0};
@@ -786,15 +788,17 @@ kbusCheckEngine_KERNEL
             return NV_FALSE;
         }
 
-        for (i=0; i<NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX; i++)
+        for (i = 0; i < NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX; i++)
         {
             nv2080EngineCaps[i] = pGSCI->engineCaps[i];
         }
     }
 
     NV_CHECK_OK_OR_ELSE(status, LEVEL_ERROR,
-          gpuGetRmEngineTypeCapMask(nv2080EngineCaps, NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX, rmEngineCaps),
-          return NV_FALSE);
+        gpuGetRmEngineTypeCapMask(nv2080EngineCaps,
+                                  NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX,
+                                  rmEngineCaps),
+        return NV_FALSE);
 
     switch (engDesc)
     {
@@ -810,10 +814,7 @@ kbusCheckEngine_KERNEL
         // (b) RM offload (Kernel RM) where display is supported.
         //
         case ENG_KERNEL_DISPLAY:
-            if (IS_GSP_CLIENT(pGpu))
-                return NV_TRUE;
-            else
-                return NV_FALSE;
+            return IS_GSP_CLIENT(pGpu);
 
         case ENG_BIF:
         case ENG_KERNEL_BIF:
@@ -833,67 +834,51 @@ kbusCheckEngine_KERNEL
             return NV_TRUE;
 
         case ENG_CE(0):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY0) ? NV_TRUE: NV_FALSE);
         case ENG_CE(1):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY1) ? NV_TRUE: NV_FALSE);
         case ENG_CE(2):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY2) ? NV_TRUE: NV_FALSE);
         case ENG_CE(3):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY3) ? NV_TRUE: NV_FALSE);
         case ENG_CE(4):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY4) ? NV_TRUE: NV_FALSE);
         case ENG_CE(5):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY5) ? NV_TRUE: NV_FALSE);
         case ENG_CE(6):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY6) ? NV_TRUE: NV_FALSE);
         case ENG_CE(7):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY7) ? NV_TRUE: NV_FALSE);
         case ENG_CE(8):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY8) ? NV_TRUE: NV_FALSE);
         case ENG_CE(9):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_COPY9) ? NV_TRUE: NV_FALSE);
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                RM_ENGINE_TYPE_COPY(GET_CE_IDX(engDesc)));
+
         case ENG_MSENC(0):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVENC0) ? NV_TRUE: NV_FALSE);
         case ENG_MSENC(1):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVENC1) ? NV_TRUE: NV_FALSE);
         case ENG_MSENC(2):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVENC2) ? NV_TRUE: NV_FALSE);
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                RM_ENGINE_TYPE_NVENC(GET_MSENC_IDX(engDesc)));
         case ENG_SEC2:
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_SEC2) ? NV_TRUE: NV_FALSE);
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                                                RM_ENGINE_TYPE_SEC2);
         case ENG_NVDEC(0):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC0) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(1):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC1) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(2):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC2) ? NV_TRUE: NV_FALSE);
-        case ENG_OFA:
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_OFA) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(3):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC3) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(4):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC4) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(5):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC5) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(6):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC6) ? NV_TRUE: NV_FALSE);
         case ENG_NVDEC(7):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVDEC7) ? NV_TRUE: NV_FALSE);
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                RM_ENGINE_TYPE_NVDEC(GET_NVDEC_IDX(engDesc)));
+
+        case ENG_OFA:
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                                                RM_ENGINE_TYPE_OFA);
         case ENG_NVJPEG(0):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG0) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(1):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG1) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(2):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG2) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(3):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG3) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(4):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG4) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(5):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG5) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(6):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG6) ? NV_TRUE: NV_FALSE);
         case ENG_NVJPEG(7):
-                return (NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps, RM_ENGINE_TYPE_NVJPEG7) ? NV_TRUE: NV_FALSE);
+            return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
+                RM_ENGINE_TYPE_NVJPEG(GET_NVJPEG_IDX(engDesc)));
+
         case ENG_GR(1):
         case ENG_GR(2):
         case ENG_GR(3):
@@ -906,10 +891,10 @@ kbusCheckEngine_KERNEL
 
             NV_ASSERT_OR_RETURN(pKernelFifo != NULL, NV_FALSE);
 
-            if (kfifoCheckEngine_HAL(pGpu, pKernelFifo, engDesc, &bSupported) == NV_OK)
-                return bSupported;
-            else
-                return NV_FALSE;
+            return (kfifoCheckEngine_HAL(pGpu, pKernelFifo,
+                                         engDesc,
+                                         &bSupported) == NV_OK &&
+                    bSupported);
         }
 
         case ENG_INVALID:
@@ -956,21 +941,14 @@ kbusGetDeviceCaps_IMPL
     OBJSYS *pSys = SYS_GET_INSTANCE();
     OBJCL  *pCl  = SYS_GET_CL(pSys);
     NvU8 tempCaps[NV0080_CTRL_HOST_CAPS_TBL_SIZE], temp;
-    NvBool bVirtualP2P;
     NvBool bExplicitCacheFlushRequired;
 
     NV_ASSERT(!gpumgrGetBcEnabledStatus(pGpu));
 
     portMemSet(tempCaps, 0, NV0080_CTRL_HOST_CAPS_TBL_SIZE);
 
-    /*! On KEPLER+, mailbox protocol based P2P transactions goes through virtual to
-     *  physical translation (on request side) */
-    bVirtualP2P = IsdMAXWELLorBetter(pGpu);
-    if (bVirtualP2P)
-        RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _VIRTUAL_P2P);
-
     /*! DMAs to/from cached memory need to have the cache flushed explicitly */
-    bExplicitCacheFlushRequired = NVCPU_IS_ARM && 
+    bExplicitCacheFlushRequired = NVCPU_IS_ARM &&
                                   (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_PLATFORM_MODS_UNIX);
     if (bExplicitCacheFlushRequired ||
         (!pCl->getProperty(pCL, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT)))
@@ -999,8 +977,6 @@ kbusGetDeviceCaps_IMPL
                    NV0080_CTRL_HOST_CAPS, _P2P_4_WAY);
     RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
                    NV0080_CTRL_HOST_CAPS, _P2P_8_WAY);
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _VIRTUAL_P2P);
     RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
                    NV0080_CTRL_HOST_CAPS, _GPU_COHERENT_MAPPING_SUPPORTED);
 
@@ -1242,4 +1218,3 @@ kbusIsGpuP2pAlive_IMPL
 {
     return (pKernelBus->totalP2pObjectsAliveRefCount > 0);
 }
-
