@@ -45,7 +45,6 @@ static NV_STATUS _Class5080DelDeferredApi(DeferredApiObject *pDeferredApiObject,
 
 static NV_STATUS _class5080DeferredApiV2(OBJGPU            *pGpu,
                                          ChannelDescendant *Object,
-                                         METHOD            *pMethod,
                                          NvU32              Offset,
                                          NvU32              Data);
 
@@ -469,7 +468,6 @@ _class5080DeferredApiV2
 (
     OBJGPU *pGpu,
     ChannelDescendant *Object,
-    METHOD *pMethod,
     NvU32 Offset,
     NvU32 Data
 )
@@ -702,7 +700,7 @@ cleanup:
     return rmStatus;
 }
 
-static METHOD Nv50DeferredApi[] =
+static const METHOD Nv50DeferredApi[] =
 {
     { mthdNoOperation,                  0x0100, 0x0103 },
     { _class5080DeferredApiV2,          0x0200, 0x0203 },
@@ -711,7 +709,7 @@ static METHOD Nv50DeferredApi[] =
 NV_STATUS defapiGetSwMethods_IMPL
 (
     DeferredApiObject  *pDeferredApi,
-    METHOD            **ppMethods,
+    const METHOD      **ppMethods,
     NvU32              *pNumMethods
 )
 {
