@@ -195,8 +195,6 @@ deviceDestruct_IMPL
     NV_STATUS               rmStatus = NV_OK;
     NV_STATUS               tmpStatus;
     NvHandle                hClient;
-    NODE                   *pNode;
-    RM_API                 *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
 
     resGetFreeParams(staticCast(pDevice, RsResource), &pCallContext, &pParams);
 
@@ -206,18 +204,6 @@ deviceDestruct_IMPL
 
     LOCK_METER_DATA(FREE_DEVICE, 0, 0, 0);
 
-    // Free all device memory
-    btreeEnumStart(0, &pNode, pDevice->DevMemoryTable);
-    while (pNode != NULL)
-    {
-        Memory *pMemory = pNode->Data;
-        btreeEnumNext(&pNode, pDevice->DevMemoryTable);
-
-        tmpStatus = pRmApi->Free(pRmApi, hClient, RES_GET_HANDLE(pMemory));
-        if ((tmpStatus != NV_OK) && (rmStatus == NV_OK))
-            rmStatus = tmpStatus;
-    }
-
     // free the device
     if (_deviceTeardownRef(pDevice, pCallContext) != NV_OK ||
         _deviceTeardown(pDevice, pCallContext) != NV_OK)
