@@ -232,6 +232,8 @@ gsyncP2060StartupProvider(OBJGSYNC *pGsync)
     pGsync->gsyncHal.gsyncSetInterlaceMode   = gsyncSetInterlaceMode_P2060;
     pGsync->gsyncHal.gsyncGetStereoLockMode  = gsyncGetStereoLockMode_P2060;
     pGsync->gsyncHal.gsyncSetStereoLockMode  = gsyncSetStereoLockMode_P2060;
+    pGsync->gsyncHal.gsyncGetMulDiv          = gsyncGetMulDiv_P2060;
+    pGsync->gsyncHal.gsyncSetMulDiv          = gsyncSetMulDiv_P2060;
 
     pGsync->gsyncHal.gsyncRefSwapBarrier     = gsyncRefSwapBarrier_P2060;
     pGsync->gsyncHal.gsyncSetMosaic          = gsyncSetMosaic_P2060;
@@ -1230,34 +1232,39 @@ gsyncGetStatusParams(OBJGSYNC *pGsync,
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_POLARITY )
     {
-        status |= pGsync->gsyncHal.gsyncGetSyncPolarity(pGpu, pGsync->pExtDev, &SyncPolarity);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetSyncPolarity(pGpu, pGsync->pExtDev, &SyncPolarity));
         pParams->syncPolarity = (NvU32)SyncPolarity;
     }
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_VIDEO_MODE )
     {
-        status |= pGsync->gsyncHal.gsyncGetVideoMode(pGpu, pGsync->pExtDev, &VideoMode);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetVideoMode(pGpu, pGsync->pExtDev, &VideoMode));
         pParams->syncVideoMode = (NvU32)VideoMode;
     }
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_NSYNC )
     {
-        status |= pGsync->gsyncHal.gsyncGetNSync(pGpu, pGsync->pExtDev, &pParams->nSync);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetNSync(pGpu, pGsync->pExtDev, &pParams->nSync));
     }
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_SKEW )
     {
-        status |= pGsync->gsyncHal.gsyncGetSyncSkew(pGpu, pGsync->pExtDev, &pParams->syncSkew);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetSyncSkew(pGpu, pGsync->pExtDev, &pParams->syncSkew));
     }
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_START_DELAY )
     {
-        status |= pGsync->gsyncHal.gsyncGetSyncStartDelay(pGpu, pGsync->pExtDev, &pParams->syncStartDelay);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetSyncStartDelay(pGpu, pGsync->pExtDev, &pParams->syncStartDelay));
     }
 
     if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_USE_HOUSE )
     {
-        status |= pGsync->gsyncHal.gsyncGetUseHouse(pGpu, pGsync->pExtDev, &pParams->useHouseSync);
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetUseHouse(pGpu, pGsync->pExtDev, &pParams->useHouseSync));
+    }
+
+    if ( pParams->which & NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_MULTIPLY_DIVIDE )
+    {
+        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_INFO, pGsync->gsyncHal.gsyncGetMulDiv(pGpu, pGsync->pExtDev, &pParams->syncMulDiv));
     }
 
     return status;
@@ -1308,6 +1315,11 @@ gsyncSetControlParams(OBJGSYNC *pGsync,
         status |= pGsync->gsyncHal.gsyncSetUseHouse(pGpu, pGsync->pExtDev, pParams->useHouseSync);
     }
 
+    if ( pParams->which & NV30F1_CTRL_GSYNC_SET_CONTROL_SYNC_MULTIPLY_DIVIDE )
+    {
+        status |= pGsync->gsyncHal.gsyncSetMulDiv(pGpu, pGsync->pExtDev, &pParams->syncMulDiv);
+    }
+
     return status;
 }
 
@@ -2452,6 +2464,28 @@ gsyncNullSetHouseSyncMode
     return NV_ERR_GENERIC;
 }
 
+static NV_STATUS
+gsyncNullGetMulDiv
+(
+    OBJGPU       *pGpu,
+    DACEXTERNALDEVICE *pExtDev,
+    NV30F1_CTRL_GSYNC_MULTIPLY_DIVIDE_SETTINGS *pMulDivSettings
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+static NV_STATUS
+gsyncNullSetMulDiv
+(
+    OBJGPU       *pGpu,
+    DACEXTERNALDEVICE *pExtDev,
+    NV30F1_CTRL_GSYNC_MULTIPLY_DIVIDE_SETTINGS *pMulDivSettings
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
 static NV_STATUS
 gsyncSetupNullProvider(OBJGSYNCMGR *pGsyncMgr, NvU32 gsyncInst)
 {
@@ -2496,6 +2530,8 @@ gsyncSetupNullProvider(OBJGSYNCMGR *pGsyncMgr, NvU32 gsyncInst)
     pGsync->gsyncHal.gsyncConfigFlashGsync     = gsyncNullConfigFlashGsync;
     pGsync->gsyncHal.gsyncGetHouseSyncMode     = gsyncNullGetHouseSyncMode;
     pGsync->gsyncHal.gsyncSetHouseSyncMode     = gsyncNullSetHouseSyncMode;
+    pGsync->gsyncHal.gsyncGetMulDiv            = gsyncNullGetMulDiv;
+    pGsync->gsyncHal.gsyncSetMulDiv            = gsyncNullSetMulDiv;
 
     return status;
 }
