@@ -33,19 +33,7 @@
 #include "published/ampere/ga100/dev_ram.h"
 #include "published/ampere/ga100/dev_ctrl.h"
 
-/**
- * @brief Translates between 2 engine values
- *
- * To iterate through a value for all engines call with inType of
- * ENGINE_INFO_TYPE_INVALID for 0 through fifoGetNumEngines().
- *
- * @param pGpu
- * @param pKernelFifo
- * @param[in] inType ENGINE_INFO_TYPE_*
- * @param[in] inVal
- * @param[in] outType ENGINE_INFO_TYPE_*
- * @param[out] pOutVal
- */
+
 NV_STATUS
 kfifoEngineInfoXlate_GA100
 (
@@ -148,29 +136,6 @@ kfifoChannelGroupGetLocalMaxSubcontext_GA100
                                                         bLegacyMode);
 }
 
-/*!
- * @brief Clear USERD memory
- */
-void
-kfifoSetupUserD_GA100
-(
-    KernelFifo *pKernelFifo,
-    NvU8       *pUserD
-)
-{
-    NV_ASSERT_OR_RETURN_VOID(pUserD != NULL);
-
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_PUT ),                  0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_GET ),                  0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_REF ),                  0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_PUT_HI ),               0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_TOP_LEVEL_GET ),        0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_TOP_LEVEL_GET_HI ),     0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_GET_HI ),               0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_GP_GET ),               0 );
-    MEM_WR32( pUserD + SF_OFFSET( NV_RAMUSERD_GP_PUT ),               0 );
-}
-
 /*!
  * @brief Update the usermode doorbell register with work submit token to notify
  *        host that work is available on this channel.
