@@ -37,25 +37,15 @@ typedef enum {
     BAD_READ_UNKNOWN,
 } RMCD_BAD_READ_REASON;
 
-static NV_STATUS _allocGpuIODevice(GPU_IO_DEVICE **ppIODevice);
 static void   _gpuCleanRegisterFilterList(DEVICE_REGFILTER_INFO *);
 static NvU32  _gpuHandleReadRegisterFilter(OBJGPU *, DEVICE_INDEX devIndex, NvU32 devInstance, NvU32 addr, NvU32 accessSize, NvU32 *pFlags, THREAD_STATE_NODE *pThreadState);
 static void   _gpuHandleWriteRegisterFilter(OBJGPU *, DEVICE_INDEX devIndex, NvU32 devInstance, NvU32 addr, NvU32 val, NvU32 accessSize, NvU32 *pFlags, THREAD_STATE_NODE *pThreadState);
 
-static void   _gpuApertureWriteRegUnicast(OBJGPU *, IO_APERTURE *pAperture, NvU32 addr, NvV32 val, NvU32 size);
-static NvU32  _gpuApertureReadReg(IO_APERTURE *pAperture, NvU32 addr, NvU32 size);
-
-static NvU8   _gpuApertureReadReg008(IO_APERTURE *a, NvU32 addr);
-static NvU16  _gpuApertureReadReg016(IO_APERTURE *a, NvU32 addr);
-static NvU32  _gpuApertureReadReg032(IO_APERTURE *a, NvU32 addr);
-static void   _gpuApertureWriteReg008(IO_APERTURE *a, NvU32 addr, NvV8  value);
-static void   _gpuApertureWriteReg016(IO_APERTURE *a, NvU32 addr, NvV16 value);
-static void   _gpuApertureWriteReg032(IO_APERTURE *a, NvU32 addr, NvV32 value);
-static void   _gpuApertureWriteReg032Unicast(IO_APERTURE *a, NvU32 addr, NvV32 value);
-static NvBool _gpuApertureValidReg(IO_APERTURE *a, NvU32 addr);
+static void   ioaprtWriteRegUnicast(OBJGPU *, IoAperture *pAperture, NvU32 addr, NvV32 val, NvU32 size);
+static NvU32  ioaprtReadReg(IoAperture *pAperture, NvU32 addr, NvU32 size);
 
 static REGISTER_FILTER * _findGpuRegisterFilter(DEVICE_INDEX devIndex, NvU32 devInstance, NvU32 addr, REGISTER_FILTER *);
-static NV_STATUS            _gpuInitIODeviceAndAperture(OBJGPU *, NvU32, NvU32, RmPhysAddr, NvU32);
+static NV_STATUS _gpuInitIOAperture(OBJGPU *pGpu, NvU32 deviceIndex, DEVICE_MAPPING *pMapping);
 
 NV_STATUS
 regAccessConstruct
@@ -89,10 +79,7 @@ regAccessConstruct
         DEVICE_MAPPING *pMapping = gpuGetDeviceMapping(pGpu, deviceIndex, 0);
         if (pMapping != NULL)
         {
-            rmStatus = _gpuInitIODeviceAndAperture(pGpu, deviceIndex,
-                                                   pMapping->gpuDeviceEnum,
-                                                   pMapping->gpuNvPAddr,
-                                                   pMapping->gpuNvLength);
+            rmStatus = _gpuInitIOAperture(pGpu, deviceIndex, pMapping);
             if (rmStatus != NV_OK)
             {
                 NV_PRINTF(LEVEL_ERROR,
@@ -114,7 +101,7 @@ regAccessDestruct
     OBJGPU         *pGpu = pRegisterAccess->pGpu;
     DEVICE_INDEX    deviceIndex;
     NvU32           mappingNum;
-    IO_APERTURE    *pIOAperture;
+    IoAperture     *pIOAperture;
     REGISTER_FILTER *pNode;
 
     // Ignore attempt to destruct a not-fully-constructed RegisterAccess
@@ -128,8 +115,7 @@ regAccessDestruct
         pIOAperture = pGpu->pIOApertures[deviceIndex];
         if (pIOAperture != NULL)
         {
-            portMemFree(pIOAperture->pDevice);
-            ioaccessDestroyIOAperture(pIOAperture);
+            objDelete(pIOAperture);
         }
     }
 
@@ -153,36 +139,6 @@ regAccessDestruct
     }
 }
 
-/*!
- * @brief Allocates GPU_IO_DEVICE object
- *
- * @param[in] ppIODevice       Pointer to uninitialized GPU_IO_DEVICE
- */
-static NV_STATUS
-_allocGpuIODevice
-(
-    GPU_IO_DEVICE **ppIODevice
-)
-{
-    GPU_IO_DEVICE *pDevice;
-
-    pDevice = portMemAllocNonPaged(sizeof(GPU_IO_DEVICE));
-    if (pDevice == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "memory allocation failed for GPU IO Device\n");
-        DBG_BREAKPOINT();
-        return NV_ERR_NO_MEMORY;
-    }
-
-    portMemSet(pDevice, 0, sizeof(GPU_IO_DEVICE));
-
-    *ppIODevice = pDevice;
-
-    return NV_OK;
-}
-
-
 //
 // The following register I/O functions are organized into two groups;
 // a multi-chip unaware group and a multi-chip aware group.
@@ -254,145 +210,195 @@ _regWriteUnicast
     }
 }
 
+/*!
+ * @brief: Initialize an IoAperture instance in-place.
+ *
+ * @param[out] pAperture        pointer to the IoAperture.
+ * @param[in]  pParentAperture  pointer to the parent of the new IoAperture.
+ * @param[in]  offset           offset from the parent APERTURE's baseAddress.
+ * @param[in]  length           length of the APERTURE.
+ *
+ * @return NV_OK upon success
+ *         NV_ERR* otherwise.
+ */
+NV_STATUS
+ioaprtInit
+(
+    IoAperture     *pAperture,
+    IoAperture     *pParentAperture,
+    NvU32           offset,
+    NvU32           length
+)
+{
+    return objCreateWithFlags(&pAperture, NVOC_NULL_OBJECT, IoAperture, NVOC_OBJ_CREATE_FLAGS_IN_PLACE_CONSTRUCT, pParentAperture, NULL, 0, 0, NULL, 0, offset, length);
+}
+
+/*!
+ * Initialize an IoAperture instance.
+ *
+ * @param[in,out] pAperture        pointer to IoAperture instance to be initialized.
+ * @param[in]     pParentAperture  pointer to parent of the new IoAperture.
+ * @param[in]     deviceIndex      device index
+ * @param[in]     deviceInstance   device instance
+ * @param[in]     pMapping         device register mapping
+ * @param[in]     mappingStartAddr register address corresponding to the start of the mapping
+ * @param[in]     offset           offset from the parent APERTURE's baseAddress.
+ * @param[in]     length           length of the APERTURE.
+ *
+ * @return NV_OK when inputs are valid.
+ */
+NV_STATUS
+ioaprtConstruct_IMPL
+(
+    IoAperture      *pAperture,
+    IoAperture      *pParentAperture,
+    OBJGPU          *pGpu,
+    NvU32            deviceIndex,
+    NvU32            deviceInstance,
+    DEVICE_MAPPING  *pMapping,
+    NvU32            mappingStartAddr,
+    NvU32            offset,
+    NvU32            length
+)
+{
+    if (pParentAperture != NULL)
+    {
+        NV_ASSERT_OR_RETURN(pMapping == NULL, NV_ERR_INVALID_ARGUMENT);
+        NV_ASSERT_OR_RETURN(pGpu == NULL || pGpu == pParentAperture->pGpu, NV_ERR_INVALID_ARGUMENT);
+
+        pAperture->pGpu = pParentAperture->pGpu;
+        pAperture->deviceIndex = pParentAperture->deviceIndex;
+        pAperture->deviceInstance = pParentAperture->deviceInstance;
+        pAperture->pMapping    = pParentAperture->pMapping;
+        pAperture->baseAddress = pParentAperture->baseAddress;
+        pAperture->mappingStartAddr = pParentAperture->mappingStartAddr;
+
+        // Check if the child Aperture strides beyond the parent's boundary.
+        if ((length + offset) > pParentAperture->length)
+        {
+            NV_PRINTF(LEVEL_WARNING,
+                "Child aperture crosses parent's boundary, length %u offset %u, Parent's length %u\n",
+                length, offset, pParentAperture->length);
+        }
+
+    }
+    else
+    {
+        NV_ASSERT_OR_RETURN(pMapping != NULL, NV_ERR_INVALID_ARGUMENT);
+        NV_ASSERT_OR_RETURN(pGpu != NULL, NV_ERR_INVALID_ARGUMENT);
+
+        pAperture->pGpu = pGpu;
+        pAperture->deviceIndex = deviceIndex;
+        pAperture->deviceInstance = deviceInstance;
+        pAperture->pMapping = pMapping;
+        pAperture->baseAddress = 0;
+        pAperture->mappingStartAddr = mappingStartAddr;
+    }
+
+    pAperture->baseAddress += offset;
+    pAperture->length       = length;
+
+    return NV_OK;
+}
+
 static void
-_gpuApertureWriteRegUnicast
+ioaprtWriteRegUnicast
 (
     OBJGPU         *pGpu,
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr,
     NvV32           val,
     NvU32           size
 )
 {
-    NV_ASSERT_OR_RETURN_VOID(pAperture);
-    NV_ASSERT_OR_RETURN_VOID(pAperture->pDevice);
-
-    GPU_IO_DEVICE     *pDevice     = (GPU_IO_DEVICE*) pAperture->pDevice;
-    NvU32              deviceIndex = pDevice->deviceIndex;
-    NvU32              instance    = pDevice->instance;
+    NvU32              deviceIndex = pAperture->deviceIndex;
+    NvU32              instance    = pAperture->deviceInstance;
     NvU32              regAddr     = pAperture->baseAddress + addr;
+    NvU32              mappingRegAddr = regAddr - pAperture->mappingStartAddr;
+    DEVICE_MAPPING    *pMapping    = pAperture->pMapping;
     NvU32              flags       = 0;
     NV_STATUS          status;
     THREAD_STATE_NODE *pThreadState;
-    DEVICE_MAPPING    *pMapping;
 
-    pMapping = gpuGetDeviceMapping(pGpu, deviceIndex, instance);
-
-    if (pMapping == NULL)
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_TEGRA_SOC_NVDISPLAY))
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Could not find mapping for reg %x, deviceIndex=0x%x instance=%d\n",
-                  regAddr, deviceIndex, instance);
-        NV_ASSERT(0);
-        return;
+        status = gpuSanityCheckRegisterAccess(pGpu, regAddr, NULL);
+        if (status != NV_OK)
+        {
+            return;
+        }
+
+        threadStateGetCurrentUnchecked(&pThreadState, pGpu);
+
+        _gpuHandleWriteRegisterFilter(pGpu, deviceIndex, instance, regAddr,
+                                      val, size, &flags, pThreadState);
     }
 
-    status = gpuSanityCheckRegisterAccess(pGpu, regAddr, NULL);
-    if (status != NV_OK)
-    {
-        return;
-    }
-
-    threadStateGetCurrentUnchecked(&pThreadState, pGpu);
-
-    _gpuHandleWriteRegisterFilter(pGpu, deviceIndex, instance, regAddr,
-                                  val, size, &flags, pThreadState);
-
     if (!(flags & REGISTER_FILTER_FLAGS_WRITE))
     {
         switch (size)
         {
             case 8:
-                osDevWriteReg008(pGpu, pMapping, regAddr, 0xFFU & (val));
+                osDevWriteReg008(pGpu, pMapping, mappingRegAddr, 0xFFU & (val));
                 break;
             case 16:
-                osDevWriteReg016(pGpu, pMapping, regAddr, 0xFFFFU & (val));
+                osDevWriteReg016(pGpu, pMapping, mappingRegAddr, 0xFFFFU & (val));
                 break;
             case 32:
-                osDevWriteReg032(pGpu, pMapping, regAddr, val);
+                osDevWriteReg032(pGpu, pMapping, mappingRegAddr, val);
                 break;
         }
     }
 }
 
 void
-_gpuApertureWriteReg008
+ioaprtWriteReg08_IMPL
 (
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr,
     NvV8            val
 )
 {
-    GPU_IO_DEVICE *pDevice = (GPU_IO_DEVICE*)pAperture->pDevice;
-    OBJGPU        *pGpu    = pDevice->pGpu;
+    NV_ASSERT(!gpumgrGetBcEnabledStatus(pAperture->pGpu));
 
-    //
-    // NOTE: The SLI loop below reuses pAperture's values across all iterations
-    // OBJGPU's apertures are initialized to have the same baseAddress and length
-    // on all GPU device instances, so reusing the aperture here is fine.
-    // Device-specific instances are obtained via gpuGetDeviceMapping in the SLI loop.
-    //
-    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY);
-        _gpuApertureWriteRegUnicast(pGpu, pAperture, addr, val, 8 /* size */);
-    SLI_LOOP_END;
+    ioaprtWriteRegUnicast(pAperture->pGpu, pAperture, addr, val, 8 /* size */);
 }
 
 void
-_gpuApertureWriteReg016
+ioaprtWriteReg16_IMPL
 (
-    IO_APERTURE       *pAperture,
+    IoAperture        *pAperture,
     NvU32              addr,
     NvV16              val
 )
 {
-    GPU_IO_DEVICE *pDevice = (GPU_IO_DEVICE*)pAperture->pDevice;
-    OBJGPU        *pGpu    = pDevice->pGpu;
+    NV_ASSERT(!gpumgrGetBcEnabledStatus(pAperture->pGpu));
 
-    //
-    // NOTE: The SLI loop below reuses pAperture's values across all iterations
-    // OBJGPU's apertures are initialized to have the same baseAddress and length
-    // on all GPU device instances, so reusing the aperture here is fine.
-    // Device-specific instances are obtained via gpuGetDeviceMapping in the SLI loop.
-    //
-    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY);
-        _gpuApertureWriteRegUnicast(pGpu, pAperture, addr, val, 16 /* size */);
-    SLI_LOOP_END;
+    ioaprtWriteRegUnicast(pAperture->pGpu, pAperture, addr, val, 16 /* size */);
 }
 
 void
-_gpuApertureWriteReg032
+ioaprtWriteReg32_IMPL
 (
-    IO_APERTURE       *pAperture,
+    IoAperture        *pAperture,
     NvU32              addr,
     NvV32              val
 )
 {
-    GPU_IO_DEVICE *pDevice = (GPU_IO_DEVICE*)pAperture->pDevice;
-    OBJGPU        *pGpu    = pDevice->pGpu;
+    NV_ASSERT(!gpumgrGetBcEnabledStatus(pAperture->pGpu));
 
-    //
-    // NOTE: The SLI loop below reuses pAperture's values across all iterations
-    // OBJGPU's apertures are initialized to have the same baseAddress and length
-    // on all GPU device instances, so reusing the aperture here is fine.
-    // Device-specific instances are obtained via gpuGetDeviceMapping in the SLI loop.
-    //
-    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY);
-        _gpuApertureWriteRegUnicast(pGpu, pAperture, addr, val, 32 /* size */);
-    SLI_LOOP_END;
+    ioaprtWriteRegUnicast(pAperture->pGpu, pAperture, addr, val, 32 /* size */);
 }
 
 void
-_gpuApertureWriteReg032Unicast
+ioaprtWriteReg32Uc_IMPL
 (
-    IO_APERTURE       *pAperture,
+    IoAperture        *pAperture,
     NvU32              addr,
     NvV32              val
 )
 {
-    GPU_IO_DEVICE *pDevice = (GPU_IO_DEVICE*)pAperture->pDevice;
-    OBJGPU        *pGpu    = pDevice->pGpu;
-
-    _gpuApertureWriteRegUnicast(pGpu, pAperture, addr, val, 32 /* size */);
+    ioaprtWriteRegUnicast(pAperture->pGpu, pAperture, addr, val, 32 /* size */);
 }
 
 void
@@ -462,114 +468,108 @@ regWrite032Unicast
 }
 
 static NvU32
-_gpuApertureReadReg
+ioaprtReadReg
 (
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr,
     NvU32           size
 )
 {
-    NV_ASSERT_OR_RETURN(pAperture,          NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pAperture->pDevice, NV_ERR_INVALID_ARGUMENT);
-
     NvU32              flags       = 0;
     NvU32              returnValue = 0;
-    GPU_IO_DEVICE     *pDevice     = (GPU_IO_DEVICE*) pAperture->pDevice;
-    OBJGPU            *pGpu        = pDevice->pGpu;
+    OBJGPU            *pGpu        = pAperture->pGpu;
     NV_STATUS          status      = NV_OK;
     NvU32              regAddr     = pAperture->baseAddress + addr;
-    NvU32              deviceIndex = pDevice->deviceIndex;
-    NvU32              instance    = pDevice->instance;
+    NvU32              mappingRegAddr = regAddr - pAperture->mappingStartAddr;
+    NvU32              deviceIndex = pAperture->deviceIndex;
+    NvU32              instance    = pAperture->deviceInstance;
+    DEVICE_MAPPING    *pMapping    = pAperture->pMapping;
     THREAD_STATE_NODE *pThreadState;
 
     pGpu->registerAccess.regReadCount++;
 
-    DEVICE_MAPPING *pMapping = gpuGetDeviceMapping(pGpu, deviceIndex, instance);
-    if (!pMapping)
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_TEGRA_SOC_NVDISPLAY))
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Could not find mapping for reg %x, deviceIndex=0x%x instance=%d\n",
-                  regAddr, deviceIndex, instance);
-        NV_ASSERT(0);
-        return 0xd0d0d0d0U;
+        status = gpuSanityCheckRegisterAccess(pGpu, regAddr, NULL);
+        if (status != NV_OK)
+        {
+            return (~0);
+        }
+
+        threadStateGetCurrentUnchecked(&pThreadState, pGpu);
+
+        returnValue = _gpuHandleReadRegisterFilter(pGpu, deviceIndex, instance,
+                                                   regAddr, size, &flags, pThreadState);
     }
 
-    status = gpuSanityCheckRegisterAccess(pGpu, regAddr, NULL);
-    if (status != NV_OK)
-    {
-        return (~0);
-    }
-
-    threadStateGetCurrentUnchecked(&pThreadState, pGpu);
-
-    returnValue = _gpuHandleReadRegisterFilter(pGpu, deviceIndex, instance,
-                                               regAddr, size, &flags, pThreadState);
-
     if (!(flags & REGISTER_FILTER_FLAGS_READ))
     {
         switch (size)
         {
             case 8:
-                returnValue = osDevReadReg008(pGpu, pMapping, regAddr);
+                returnValue = osDevReadReg008(pGpu, pMapping, mappingRegAddr);
                 break;
             case 16:
-                returnValue = osDevReadReg016(pGpu, pMapping, regAddr);
+                returnValue = osDevReadReg016(pGpu, pMapping, mappingRegAddr);
                 break;
             case 32:
-                returnValue = osDevReadReg032(pGpu, pMapping, regAddr);
+                returnValue = osDevReadReg032(pGpu, pMapping, mappingRegAddr);
                 break;
         }
     }
 
-    // Make sure the value read is sane before we party on it.
-    gpuSanityCheckRegRead(pGpu, regAddr, size, &returnValue);
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_TEGRA_SOC_NVDISPLAY))
+    {
+        // Make sure the value read is sane before we party on it.
+        gpuSanityCheckRegRead(pGpu, regAddr, size, &returnValue);
+    }
 
     return returnValue;
 }
 
 NvU8
-_gpuApertureReadReg008
+ioaprtReadReg08_IMPL
 (
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr
 )
 {
-    return (NvU8) _gpuApertureReadReg(pAperture, addr, 8 /* size */);
+    return (NvU8) ioaprtReadReg(pAperture, addr, 8 /* size */);
 }
 
-static NvU16
-_gpuApertureReadReg016
+NvU16
+ioaprtReadReg16_IMPL
 (
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr
 )
 {
-    return (NvU16) _gpuApertureReadReg(pAperture, addr, 16 /* size */);
+    return (NvU16) ioaprtReadReg(pAperture, addr, 16 /* size */);
 }
 
-static NvU32
-_gpuApertureReadReg032
+NvU32
+ioaprtReadReg32_IMPL
 (
-    IO_APERTURE       *pAperture,
+    IoAperture        *pAperture,
     NvU32              addr
 
 )
 {
-    return _gpuApertureReadReg(pAperture, addr, 32 /* size */);
+    return ioaprtReadReg(pAperture, addr, 32 /* size */);
 }
 
 /*!
  * Checks if the register address is valid for a particular aperture
  *
- * @param[in]       pAperture       IO_APERTURE pointer
+ * @param[in]       pAperture       IoAperture pointer
  * @param[in]       addr            register address
  *
  * @returns         NV_TRUE         Register offset is valid
  */
-static NvBool
-_gpuApertureValidReg
+NvBool
+ioaprtIsRegValid_IMPL
 (
-    IO_APERTURE    *pAperture,
+    IoAperture     *pAperture,
     NvU32           addr
 )
 {
@@ -748,57 +748,24 @@ regRead032
  * @return NV_OK if IO Aperture is successfully initialized, error otherwise.
  */
 static NV_STATUS
-_gpuInitIODeviceAndAperture
+_gpuInitIOAperture
 (
-    OBJGPU    *pGpu,
-    NvU32      deviceIndex,
-    NvU32      gpuDeviceEnum,
-    RmPhysAddr gpuNvPAddr,
-    NvU32      gpuNvLength
+    OBJGPU         *pGpu,
+    NvU32           deviceIndex,
+    DEVICE_MAPPING *pMapping
 )
 {
     NV_STATUS rmStatus;
-    GPU_IO_DEVICE *pIODevice = NULL;
 
-    // Initialize GPU IO Device
-    rmStatus = _allocGpuIODevice(&pIODevice);
+    rmStatus = objCreate(&pGpu->pIOApertures[deviceIndex], NVOC_NULL_OBJECT, IoAperture,
+                         NULL, // no parent aperture
+                         pGpu,
+                         deviceIndex,
+                         0, // GPU register operations are always on instance 0
+                         pMapping, 0, // mapping, mappingStartAddr
+                         0, pMapping->gpuNvLength); // offset, length
     if (rmStatus != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Failed to initialize pGpu IO device for devIdx %d.\n",
-                  deviceIndex);
-
-        return rmStatus;
-    }
-
-    pIODevice->pGpu          = pGpu;
-    pIODevice->deviceIndex   = deviceIndex;
-    pIODevice->gpuDeviceEnum = gpuDeviceEnum;
-    pIODevice->gpuNvPAddr    = gpuNvPAddr;
-    pIODevice->gpuNvLength   = gpuNvLength;
-    pIODevice->refCount      = 0;
-
-    // GPU register operations are always on instance 0
-    pIODevice->instance      = 0;
-
-    // Initialize register functions in IO_DEVICE
-    pIODevice->parent.pReadReg008Fn    = (ReadReg008Fn*) &_gpuApertureReadReg008;
-    pIODevice->parent.pReadReg016Fn    = (ReadReg016Fn*) &_gpuApertureReadReg016;
-    pIODevice->parent.pReadReg032Fn    = (ReadReg032Fn*) &_gpuApertureReadReg032;
-    pIODevice->parent.pWriteReg008Fn   = (WriteReg008Fn*) &_gpuApertureWriteReg008;
-    pIODevice->parent.pWriteReg016Fn   = (WriteReg016Fn*) &_gpuApertureWriteReg016;
-    pIODevice->parent.pWriteReg032Fn   = (WriteReg032Fn*) &_gpuApertureWriteReg032;
-    pIODevice->parent.pWriteReg032UcFn = (WriteReg032Fn*) &_gpuApertureWriteReg032Unicast;
-    pIODevice->parent.pValidRegFn      = (ValidRegFn*)    &_gpuApertureValidReg;
-
-    rmStatus = ioaccessCreateIOAperture(&pGpu->pIOApertures[deviceIndex],
-                                        NULL,            // no parent aperture
-                                        (IO_DEVICE*) pIODevice,
-                                        0, gpuNvLength); // offset, length
-    if (rmStatus != NV_OK)
-    {
-        portMemFree(pIODevice);
-
         NV_PRINTF(LEVEL_ERROR,
                   "Failed to initialize pGpu IO aperture for devIdx %d.\n",
                   deviceIndex);
@@ -1777,3 +1744,148 @@ gpuSanityCheckRegRead_IMPL
 
     return NV_OK;
 }
+
+
+NV_STATUS swbcaprtConstruct_IMPL
+(
+    SwBcAperture      *pAperture,
+    IoAperture        *pApertures,
+    NvU32              numApertures
+)
+{
+    NV_ASSERT_OR_RETURN(numApertures != 0, NV_ERR_INVALID_ARGUMENT);
+
+    pAperture->pApertures = pApertures;
+    pAperture->numApertures = numApertures;
+
+    return NV_OK;
+}
+
+NvU8
+swbcaprtReadReg08_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr
+)
+{
+    NvU8 val = REG_RD08(&pAperture->pApertures[0], addr);
+
+#if defined(DEBUG)
+    NvU32 i;
+    for (i = 1; i < pAperture->numApertures; i++)
+        NV_ASSERT(REG_RD08(&pAperture->pApertures[i], addr) == val);
+#endif // defined(DEBUG)
+
+    return val;
+}
+
+NvU16
+swbcaprtReadReg16_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr
+)
+{
+    NvU16 val = REG_RD16(&pAperture->pApertures[0], addr);
+
+#if defined(DEBUG)
+    NvU32 i;
+    for (i = 1; i < pAperture->numApertures; i++)
+        NV_ASSERT(REG_RD16(&pAperture->pApertures[i], addr) == val);
+#endif // defined(DEBUG)
+
+    return val;
+}
+
+NvU32
+swbcaprtReadReg32_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr
+)
+{
+    NvU32 val = REG_RD32(&pAperture->pApertures[0], addr);
+
+#if defined(DEBUG)
+    NvU32 i;
+    for (i = 1; i < pAperture->numApertures; i++)
+        NV_ASSERT(REG_RD32(&pAperture->pApertures[i], addr) == val);
+#endif // defined(DEBUG)
+
+    return val;
+}
+
+void
+swbcaprtWriteReg08_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr,
+    NvV8          value
+)
+{
+    NvU32 i;
+
+    for (i = 0; i < pAperture->numApertures; i++)
+        REG_WR08(&pAperture->pApertures[i], addr, value);
+}
+
+void
+swbcaprtWriteReg16_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr,
+    NvV16         value
+)
+{
+    NvU32 i;
+
+    for (i = 0; i < pAperture->numApertures; i++)
+        REG_WR16(&pAperture->pApertures[i], addr, value);
+}
+
+void
+swbcaprtWriteReg32_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr,
+    NvV32         value
+)
+{
+    NvU32 i;
+
+    for (i = 0; i < pAperture->numApertures; i++)
+        REG_WR32(&pAperture->pApertures[i], addr, value);
+}
+
+void
+swbcaprtWriteReg32Uc_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr,
+    NvV32         value
+)
+{
+    NvU32 i;
+
+    for (i = 0; i < pAperture->numApertures; i++)
+        REG_WR32_UC(&pAperture->pApertures[i], addr, value);
+}
+
+NvBool
+swbcaprtIsRegValid_IMPL
+(
+    SwBcAperture *pAperture,
+    NvU32         addr
+)
+{
+
+    NvU32 i;
+
+    for (i = 0; i < pAperture->numApertures; i++)
+    {
+        if (!REG_VALID(&pAperture->pApertures[i], addr))
+            return NV_FALSE;
+    }
+
+    return NV_TRUE;
+}
