@@ -422,18 +422,10 @@ _gpuFabricProbeSend_WORKITEM
     GPU_FABRIC_PROBE_INFO *pGpuFabricProbeInfo = pGpu->pGpuFabricProbeInfo;
     NV_STATUS status;
 
-    if (pGpuFabricProbeInfo == NULL)
-    {
-        NV_PRINTF(LEVEL_INFO,
-                  "GPU%u Invalid Probe info\n",
-                  gpuGetInstance(pGpu));
-        return;
-    }
-
     if (pKernelNvlink->bIsGpuDegraded ||
         (pKernelNvlink->discoveredLinks == 0))
     {
-        NV_PRINTF(LEVEL_INFO,
+        NV_PRINTF(LEVEL_ERROR,
                   "GPU%u Degraded. Not sending probe\n",
                   gpuGetInstance(pGpu));
         _gpuFabricProbeForceCompletionError(pGpuFabricProbeInfo, NV_ERR_NOT_SUPPORTED);
@@ -554,17 +546,24 @@ _gpuFabricProbeSetupFlaRange
 void
 _gpuFabricProbeReceive
 (
-    OBJGPU *pGpu,
+    NvU32 gpuInstance,
     NV2080_CTRL_NVLINK_INBAND_RECEIVED_DATA_PARAMS *pInbandRcvParams
 )
 {
+    OBJGPU *pGpu;
+    NvU32 gpuMaskUnused;
     nvlink_inband_gpu_probe_rsp_msg_t *pProbeRespMsg;
     GPU_FABRIC_PROBE_INFO *pGpuFabricProbeInfo;
     nvlink_inband_gpu_probe_req_msg_t *pProbeReqMsg = NULL;
     NvU8 *pRsvd = NULL;
-    OBJSYS *pSys = SYS_GET_INSTANCE();
 
-    NV_ASSERT(rmDeviceGpuLockIsOwner(gpuGetInstance(pGpu)));
+    if ((pGpu = gpumgrGetGpu(gpuInstance)) == NULL)
+    {
+        NV_ASSERT_FAILED("Invalid GPU instance");
+        return;
+    }
+
+    NV_ASSERT(rmGpuGroupLockIsOwner(gpuInstance, GPU_LOCK_GRP_SUBDEVICE, &gpuMaskUnused));
 
     NV_ASSERT(pInbandRcvParams != NULL);
 
@@ -578,7 +577,7 @@ _gpuFabricProbeReceive
     {
         NV_PRINTF(LEVEL_INFO,
                   "GPU%u Probe resp invalid reqId %lld respId %lld\n",
-                  gpuGetInstance(pGpu),
+                  gpuInstance,
                   pProbeReqMsg->msgHdr.requestId,
                   pProbeRespMsg->msgHdr.requestId);
         return;
@@ -604,15 +603,6 @@ _gpuFabricProbeReceive
                             _gpuFabricProbeFullSanityCheck(pGpuFabricProbeInfo) == NV_OK);
     _gpuFabricProbeSetupGpaRange(pGpu, pGpuFabricProbeInfo);
     _gpuFabricProbeSetupFlaRange(pGpu, pGpuFabricProbeInfo);
-
-    // if MC FLA is disabled, reset the fmCaps
-    if (!pSys->bMulticastFlaEnabled)
-    {
-        pGpuFabricProbeInfo->probeResponseMsg.probeRsp.fmCaps &= 
-                             ~NVLINK_INBAND_FM_CAPS_MC_TEAM_SETUP_V1;
-        pGpuFabricProbeInfo->probeResponseMsg.probeRsp.fmCaps &= 
-                             ~NVLINK_INBAND_FM_CAPS_MC_TEAM_RELEASE_V1;
-    }
 }
 
 void
