@@ -62,18 +62,26 @@ gpuInitSriov_FWCLIENT
     GspStaticConfigInfo *pGSCI = GPU_GET_GSP_STATIC_INFO(pGpu);
     NvU32 totalPcieFns = 0;
 
-    pGpu->sriovState.vfBarSize[0] = pGSCI->sriovCaps.bar0Size;
-    pGpu->sriovState.vfBarSize[1] = pGSCI->sriovCaps.bar1Size;
-    pGpu->sriovState.vfBarSize[2] = pGSCI->sriovCaps.bar2Size;
+    pGpu->sriovState.totalVFs             = pGSCI->sriovCaps.totalVFs;
+    pGpu->sriovState.firstVFOffset        = pGSCI->sriovCaps.firstVfOffset;
+    pGpu->sriovState.firstVFBarAddress[0] = pGSCI->sriovCaps.FirstVFBar0Address;
+    pGpu->sriovState.firstVFBarAddress[1] = pGSCI->sriovCaps.FirstVFBar1Address;
+    pGpu->sriovState.firstVFBarAddress[2] = pGSCI->sriovCaps.FirstVFBar2Address;
+    pGpu->sriovState.vfBarSize[0]         = pGSCI->sriovCaps.bar0Size;
+    pGpu->sriovState.vfBarSize[1]         = pGSCI->sriovCaps.bar1Size;
+    pGpu->sriovState.vfBarSize[2]         = pGSCI->sriovCaps.bar2Size;
+    pGpu->sriovState.b64bitVFBar0         = pGSCI->sriovCaps.b64bitBar0;
+    pGpu->sriovState.b64bitVFBar1         = pGSCI->sriovCaps.b64bitBar1;
+    pGpu->sriovState.b64bitVFBar2         = pGSCI->sriovCaps.b64bitBar2;
     
-    pGpu->sriovState.maxGfid      = pGSCI->sriovMaxGfid;
+    pGpu->sriovState.maxGfid              = pGSCI->sriovMaxGfid;
 
     // note: pGpu->sriovState.virtualRegPhysOffset is initialized separately
 
     // owned by physical RM, so leave uninitialized
-    pGpu->sriovState.pP2PInfo      = NULL;
-    pGpu->sriovState.bP2PAllocated = NV_FALSE;
-    pGpu->sriovState.maxP2pGfid    = 0;
+    pGpu->sriovState.pP2PInfo             = NULL;
+    pGpu->sriovState.bP2PAllocated        = NV_FALSE;
+    pGpu->sriovState.maxP2pGfid           = 0;
 
     // Include Physical function that occupies GFID 0
     totalPcieFns = pGpu->sriovState.totalVFs + 1;
@@ -200,6 +208,8 @@ void gpuInitProperties_FWCLIENT(OBJGPU *pGpu)
 {
     GspStaticConfigInfo *pGSCI = GPU_GET_GSP_STATIC_INFO(pGpu);
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IS_MOBILE, pGSCI->bIsMobile);
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED, pGSCI->bIsGc6Rtd3Allowed);
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_RTD3_GCOFF_SUPPORTED, pGSCI->bIsGcOffRtd3Allowed);
 }
 
 /*!
@@ -246,28 +256,37 @@ gpuConstructDeviceInfoTable_FWCLIENT
     RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
 
     NV2080_CTRL_INTERNAL_GET_DEVICE_INFO_TABLE_PARAMS *pParams;
-    const NvU32 cmd = NV2080_CTRL_CMD_INTERNAL_GET_DEVICE_INFO_TABLE;
 
     if (pGpu->pDeviceInfoTable) // already initialized
         return NV_OK;
 
-    pParams = portMemAllocNonPaged(sizeof(*pParams));
+    pParams = portMemAllocNonPaged(sizeof *pParams);
     NV_ASSERT_OR_RETURN(pParams != NULL, NV_ERR_NO_MEMORY);
 
-    status = pRmApi->Control(pRmApi, pGpu->hInternalClient, pGpu->hInternalSubdevice,
-                             cmd, pParams, sizeof(*pParams));
+    status = pRmApi->Control(pRmApi,
+                             pGpu->hInternalClient,
+                             pGpu->hInternalSubdevice,
+                             NV2080_CTRL_CMD_INTERNAL_GET_DEVICE_INFO_TABLE,
+                             pParams,
+                             sizeof *pParams);
     if (status != NV_OK)
         goto done;
 
     if (pParams->numEntries == 0)
         goto done;
 
-    pGpu->pDeviceInfoTable = portMemAllocNonPaged(pParams->numEntries * sizeof(DEVICE_INFO2_TABLE));
-    NV_ASSERT_TRUE_OR_GOTO(status, pGpu->pDeviceInfoTable != NULL, NV_ERR_NO_MEMORY, done);
+    pGpu->pDeviceInfoTable = portMemAllocNonPaged(
+        pParams->numEntries * (sizeof *pGpu->pDeviceInfoTable));
+    NV_ASSERT_TRUE_OR_GOTO(status,
+                           pGpu->pDeviceInfoTable != NULL,
+                           NV_ERR_NO_MEMORY,
+                           done);
 
     pGpu->numDeviceInfoEntries = pParams->numEntries;
-    portMemCopy(pGpu->pDeviceInfoTable,   pGpu->numDeviceInfoEntries * sizeof(DEVICE_INFO2_TABLE),
-                pParams->deviceInfoTable, pParams->numEntries        * sizeof(DEVICE_INFO2_TABLE));
+    portMemCopy(pGpu->pDeviceInfoTable,
+                pGpu->numDeviceInfoEntries * (sizeof *pGpu->pDeviceInfoTable),
+                pParams->deviceInfoTable,
+                pParams->numEntries * (sizeof pParams->deviceInfoTable[0]));
 
 done:
     portMemFree(pParams);
