@@ -36,7 +36,6 @@
 #include "rmapi/client.h"
 #include "rmapi/resource_fwd_decls.h"
 #include "core/thread_state.h"
-#include "virtualization/hypervisor/hypervisor.h"
 #include "virtualization/kernel_hostvgpudeviceapi.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 #include "kernel/gpu/fifo/kernel_channel.h"
@@ -505,6 +504,9 @@ gpuNotifySubDeviceEvent_IMPL
         Subdevice *pSubdevice = pGpu->pSubdeviceBackReferences[i];
         NV_ASSERT_OR_RETURN_VOID(pSubdevice != NULL);
         INotifier *pNotifier = staticCast(pSubdevice, INotifier);
+        
+        if (inotifyGetNotificationShare(pNotifier) == NULL)
+            continue;
 
         GPU_RES_SET_THREAD_BC_STATE(pSubdevice);
 
@@ -848,8 +850,7 @@ _gpuCollectMemInfo
             (bGlobalInfo || (pMemory->pHeap == pTargetedHeap)) &&
             (RES_GET_HANDLE(pMemory->pDevice) == hDevice) &&
             (pMemory->pMemDesc != NULL) &&
-            ((!bIsGuestProcess && (!memdescGetFlag(pMemory->pMemDesc, MEMDESC_FLAGS_LIST_MEMORY))
-             && !(hypervisorIsVgxHyper() && (pResourceRef->externalClassId == NV01_MEMORY_HW_RESOURCES))) ||
+            ((!bIsGuestProcess && (!memdescGetFlag(pMemory->pMemDesc, MEMDESC_FLAGS_LIST_MEMORY))) ||
              (bIsGuestProcess && (memdescGetFlag(pMemory->pMemDesc, MEMDESC_FLAGS_GUEST_ALLOCATED)) && (pMemory->Type != NVOS32_TYPE_UNUSED))))
         {
 
