@@ -314,8 +314,11 @@ _checkTimeout
         osGetCurrentTick(&timeInNs);
         if (timeInNs >= pTimeout->timeout)
         {
-            NV_PRINTF(LEVEL_INFO, "OS elapsed %llx >= %llx\n",
-                      timeInNs, pTimeout->timeout);
+            if (!(pTimeout->flags & GPU_TIMEOUT_FLAGS_BYPASS_JOURNAL_LOG))
+            {
+                NV_PRINTF(LEVEL_INFO, "OS elapsed %llx >= %llx\n",
+                          timeInNs, pTimeout->timeout);
+            }
             status = NV_ERR_TIMEOUT;
         }
     }
@@ -345,7 +348,10 @@ _checkTimeout
 
         if (pTimeout->timeout == 0)
         {
-            NV_PRINTF(LEVEL_INFO, "OS timeout == 0\n");
+            if (!(pTimeout->flags & GPU_TIMEOUT_FLAGS_BYPASS_JOURNAL_LOG))
+            {
+                NV_PRINTF(LEVEL_INFO, "OS timeout == 0\n");
+            }
             status =  NV_ERR_TIMEOUT;
         }
     }
@@ -424,7 +430,7 @@ timeoutCheck
 
     if (!(pTimeout->flags & GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD))
     {
-        threadStateYieldCpuIfNecessary(pGpu, !!(pTimeout->flags & GPU_TIMEOUT_FLAGS_BYPASS_JOURNAL_LOG));
+        threadStateYieldCpuIfNecessary(pGpu);
     }
 
     //
