@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -78,24 +78,10 @@ _kgspResetIntoRiscv
 )
 {
     KernelFalcon *pKernelFlcn = staticCast(pKernelGsp, KernelFalcon);
-    NV_ASSERT_OK_OR_RETURN(kflcnPreResetWait_HAL(pGpu, pKernelFlcn));
-
+    NV_ASSERT_OK_OR_RETURN(kflcnPreResetWait(pGpu, pKernelFlcn));
     GPU_FLD_WR_DRF_DEF(pGpu, _PGSP, _FALCON_ENGINE, _RESET, _TRUE);
-
-    // Reg read cycles needed for signal propagation.
-    for (NvU32 i = 0; i < FLCN_RESET_PROPAGATION_DELAY_COUNT; i++)
-    {
-        GPU_REG_RD32(pGpu, NV_PGSP_FALCON_ENGINE);
-    }
-
     GPU_FLD_WR_DRF_DEF(pGpu, _PGSP, _FALCON_ENGINE, _RESET, _FALSE);
 
-    // Reg read cycles needed for signal propagation.
-    for (NvU32 i = 0; i < FLCN_RESET_PROPAGATION_DELAY_COUNT; i++)
-    {
-        GPU_REG_RD32(pGpu, NV_PGSP_FALCON_ENGINE);
-    }
-
     NV_ASSERT_OK_OR_RETURN(kflcnWaitForResetToFinish_HAL(pGpu, pKernelFlcn));
 
     kflcnRiscvProgramBcr_HAL(pGpu, pKernelFlcn, NV_TRUE);
