@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,7 +28,6 @@
 #include "rmconfig.h"
 #include "gpu/fsp/kern_fsp.h"
 #include "gpu/gsp/kernel_gsp.h"
-#include "gpu/mem_sys/kern_mem_sys.h"
 #include "gsp/gspifpub.h"
 #include "vgpu/rpc.h"
 
@@ -143,13 +142,14 @@ kgspAllocBootArgs_GH100
     NvP64 pVa = NvP64_NULL;
     NvP64 pPriv = NvP64_NULL;
     NV_STATUS nvStatus = NV_OK;
+    NvU64 flags = MEMDESC_FLAGS_NONE;
 
     // Allocate GSP-FMC arguments
     NV_ASSERT_OK_OR_GOTO(nvStatus,
                           memdescCreate(&pKernelGsp->pGspFmcArgumentsDescriptor,
                                         pGpu, sizeof(GSP_FMC_BOOT_PARAMS), 0x1000,
                                         NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED,
-                                        MEMDESC_FLAGS_NONE),
+                                        flags),
                           _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -200,17 +200,6 @@ kgspFreeBootArgs_GH100
     }
 }
 
-NvBool
-kgspIsWpr2Up_GH100
-(
-    OBJGPU    *pGpu,
-    KernelGsp *pKernelGsp
-)
-{
-
-    return kgspIsWpr2Up_TU102(pGpu, pKernelGsp);
-}
-
 NV_STATUS
 kgspWaitForGfwBootOk_GH100
 (
@@ -314,6 +303,10 @@ kgspCalculateFbLayout_GH100
     //
     pWprMeta->gspFwHeapSize = kgspGetWprHeapSize(pGpu, pKernelGsp);
 
+    // Number of VF partitions allocating sub-heaps from the WPR heap
+    pWprMeta->gspFwHeapVfPartitionCount =
+        pGpu->bVgpuGspPluginOffloadEnabled ? MAX_PARTITIONS_WITH_GFID : 0;
+
     // Fill in the meta-metadata
     pWprMeta->revision = GSP_FW_WPR_META_REVISION;
     pWprMeta->magic = GSP_FW_WPR_META_MAGIC;
@@ -384,38 +377,7 @@ _kgspIsLockdownReleased
                         _UNLOCK, reg);
 }
 
-/*!
- * Determine if PRIV target mask is unlocked for GSP and BAR0 Decoupler allows GSP access.
- *
- * This is temporary WAR for the PRIV target mask bug 3640831 until we have notification
- * protocol in place (there is no HW mechanism for CPU to check if GSP is open other than
- * reading 0xBADF41YY code).
- *
- * Until the programmed BAR0 decoupler settings are cleared, GSP access is blocked from
- * the CPU so all reads will return 0.
- */
-static NvBool
-_kgspIsTargetMaskReleased
-(
-    OBJGPU  *pGpu,
-    void    *pVoid
-)
-{
-    KernelGsp *pKernelGsp               = reinterpretCast(pVoid, KernelGsp *);
-    KernelFalcon *pKernelFalcon         = staticCast(pKernelGsp, KernelFalcon);
-    const NvU32 privErrTargetLocked     = 0xBADF4100U;
-    const NvU32 privErrTargetLockedMask = 0xFFFFFF00U; // Ignore LSB - it has extra error information
-    NvU32 reg;
 
-    //
-    // This register is read with the raw OS read to avoid the 0xbadf sanity checking
-    // done by the usual register read utilities.
-    //
-    reg = osDevReadReg032(pGpu, gpuGetDeviceMapping(pGpu, DEVICE_INDEX_GPU, 0),
-                          pKernelFalcon->registerBase + NV_PFALCON_FALCON_HWCFG2);
-
-    return ((reg != 0) && ((reg & privErrTargetLockedMask) != privErrTargetLocked));
-}
 
 
 static void
@@ -503,7 +465,6 @@ kgspBootstrapRiscvOSEarly_GH100
 {
     KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
     KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
-    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     NV_STATUS     status        = NV_OK;
 
     // Only for GSP client builds
@@ -513,16 +474,8 @@ kgspBootstrapRiscvOSEarly_GH100
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    // Clear ECC errors before attempting to load GSP
-    status = kmemsysClearEccCounts_HAL(pGpu, pKernelMemorySystem);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Issue clearing ECC counts! Status:0x%x\n", status);
-    }
-
     // Setup the descriptors that GSP-FMC needs to boot GSP-RM
-    NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-            kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw), exit);
+    NV_ASSERT_OK_OR_RETURN(kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw));
 
     kgspSetupLibosInitArgs(pGpu, pKernelGsp);
 
@@ -551,8 +504,7 @@ kgspBootstrapRiscvOSEarly_GH100
     {
         NV_PRINTF(LEVEL_ERROR, "Starting to boot GSP via FSP.\n");
         pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_GSP_MODE_GSPRM, NV_TRUE);
-        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                kfspSendBootCommands_HAL(pGpu, pKernelFsp), exit);
+        NV_ASSERT_OK_OR_RETURN(kfspSendBootCommands_HAL(pGpu, pKernelFsp));
     }
     else
     {
@@ -560,7 +512,7 @@ kgspBootstrapRiscvOSEarly_GH100
     }
 
     // Wait for target mask to be released.
-    status = gpuTimeoutCondWait(pGpu, _kgspIsTargetMaskReleased, pKernelGsp, NULL);
+    status = kfspWaitForGspTargetMaskReleased_HAL(pGpu, pKernelFsp);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Timeout waiting for GSP target mask release. "
@@ -573,7 +525,7 @@ kgspBootstrapRiscvOSEarly_GH100
             kfspDumpDebugState_HAL(pGpu, pKernelFsp);
         }
 
-        goto exit;
+        return status;
     }
 
     // Wait for lockdown to be released.
@@ -593,7 +545,7 @@ kgspBootstrapRiscvOSEarly_GH100
                   kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0));
         NV_PRINTF(LEVEL_ERROR, "NV_PGSP_FALCON_MAILBOX1 = 0x%x\n",
                   kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1));
-        goto exit;
+        return status;
     }
 
     // Start polling for libos logs now that lockdown is released
@@ -627,11 +579,6 @@ kgspBootstrapRiscvOSEarly_GH100
     NV_PRINTF(LEVEL_INFO, "GSP FW RM ready.\n");
 
 exit:
-    // If GSP fails to boot, check if there's any DED error.
-    if (status != NV_OK)
-    {
-        kmemsysCheckEccCounts_HAL(pGpu, pKernelMemorySystem);
-    }
     NV_ASSERT(status == NV_OK);
 
     return status;
