@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2017-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2017-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -43,7 +43,6 @@
 #include "published/turing/tu102/dev_riscv_pri.h"
 #include "published/turing/tu102/dev_fbif_v4.h"
 #include "published/turing/tu102/dev_falcon_v4.h"
-#include "published/turing/tu102/dev_fb.h"  // for NV_PFB_PRI_MMU_WPR2_ADDR_HI
 #include "published/turing/tu102/dev_fuse.h"
 #include "published/turing/tu102/dev_ram.h"
 #include "published/turing/tu102/dev_gc6_island.h"
@@ -57,7 +56,6 @@
 #undef RPC_STRUCTURES
 #undef RPC_GENERIC_UNION
 
-
 void
 kgspConfigureFalcon_TU102
 (
@@ -109,25 +107,26 @@ kgspAllocBootArgs_TU102
     NvP64 pVa = NvP64_NULL;
     NvP64 pPriv = NvP64_NULL;
     NV_STATUS nvStatus = NV_OK;
+    NvU64 flags = MEMDESC_FLAGS_NONE;
 
     // Allocate WPR meta data
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescCreate(&pKernelGsp->pWprMetaDescriptor,
-                                        pGpu, 0x1000, 0x1000,
-                                        NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED,
-                                        MEMDESC_FLAGS_NONE),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescCreate(&pKernelGsp->pWprMetaDescriptor,
+                                       pGpu, 0x1000, 0x1000,
+                                       NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED,
+                                       flags),
+                        _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescAlloc(pKernelGsp->pWprMetaDescriptor),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescAlloc(pKernelGsp->pWprMetaDescriptor),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescMap(pKernelGsp->pWprMetaDescriptor, 0,
-                                     memdescGetSize(pKernelGsp->pWprMetaDescriptor),
-                                     NV_TRUE, NV_PROTECT_READ_WRITE,
-                                     &pVa, &pPriv),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescMap(pKernelGsp->pWprMetaDescriptor, 0,
+                                    memdescGetSize(pKernelGsp->pWprMetaDescriptor),
+                                    NV_TRUE, NV_PROTECT_READ_WRITE,
+                                    &pVa, &pPriv),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     pKernelGsp->pWprMeta = (GspFwWprMeta *)NvP64_VALUE(pVa);
     pKernelGsp->pWprMetaMappingPriv = pPriv;
@@ -136,24 +135,24 @@ kgspAllocBootArgs_TU102
     // Setup libos arguments memory
     //
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescCreate(&pKernelGsp->pLibosInitArgumentsDescriptor,
-                                        pGpu,
-                                        LIBOS_INIT_ARGUMENTS_SIZE,
-                                        LIBOS_INIT_ARGUMENTS_SIZE,
-                                        NV_TRUE, ADDR_SYSMEM, NV_MEMORY_UNCACHED,
-                                        MEMDESC_FLAGS_NONE),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescCreate(&pKernelGsp->pLibosInitArgumentsDescriptor,
+                                       pGpu,
+                                       LIBOS_INIT_ARGUMENTS_SIZE,
+                                       LIBOS_INIT_ARGUMENTS_SIZE,
+                                       NV_TRUE, ADDR_SYSMEM, NV_MEMORY_UNCACHED,
+                                       flags),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescAlloc(pKernelGsp->pLibosInitArgumentsDescriptor),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescAlloc(pKernelGsp->pLibosInitArgumentsDescriptor),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescMap(pKernelGsp->pLibosInitArgumentsDescriptor, 0,
-                                     memdescGetSize(pKernelGsp->pLibosInitArgumentsDescriptor),
-                                     NV_TRUE, NV_PROTECT_READ_WRITE,
+                         memdescMap(pKernelGsp->pLibosInitArgumentsDescriptor, 0,
+                                    memdescGetSize(pKernelGsp->pLibosInitArgumentsDescriptor),
+                                    NV_TRUE, NV_PROTECT_READ_WRITE,
                                      &pVa, &pPriv),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         _kgspAllocBootArgs_exit_cleanup);
 
     pKernelGsp->pLibosInitArgumentsCached = (LibosMemoryRegionInitArgument *)NvP64_VALUE(pVa);
     pKernelGsp->pLibosInitArgumentsMappingPriv = pPriv;
@@ -162,22 +161,22 @@ kgspAllocBootArgs_TU102
     NV_ASSERT(sizeof(GSP_ARGUMENTS_CACHED) <= 0x1000);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescCreate(&pKernelGsp->pGspArgumentsDescriptor,
-                                        pGpu, 0x1000, 0x1000,
-                                        NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED,
-                                        MEMDESC_FLAGS_NONE),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescCreate(&pKernelGsp->pGspArgumentsDescriptor,
+                                       pGpu, 0x1000, 0x1000,
+                                       NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED,
+                                       flags),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescAlloc(pKernelGsp->pGspArgumentsDescriptor),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescAlloc(pKernelGsp->pGspArgumentsDescriptor),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescMap(pKernelGsp->pGspArgumentsDescriptor, 0,
-                                     memdescGetSize(pKernelGsp->pGspArgumentsDescriptor),
-                                     NV_TRUE, NV_PROTECT_READ_WRITE,
-                                     &pVa, &pPriv),
-                          _kgspAllocBootArgs_exit_cleanup);
+                         memdescMap(pKernelGsp->pGspArgumentsDescriptor, 0,
+                                    memdescGetSize(pKernelGsp->pGspArgumentsDescriptor),
+                                    NV_TRUE, NV_PROTECT_READ_WRITE,
+                                    &pVa, &pPriv),
+                         _kgspAllocBootArgs_exit_cleanup);
 
     pKernelGsp->pGspArgumentsCached = (GSP_ARGUMENTS_CACHED *)NvP64_VALUE(pVa);
     pKernelGsp->pGspArgumentsMappingPriv = pPriv;
@@ -362,8 +361,11 @@ kgspBootstrapRiscvOSEarly_TU102
             kflcnReset_HAL(pGpu, pKernelFalcon);
 
             NV_ASSERT_OK_OR_GOTO(status,
-                kgspExecuteFwsecFrts_HAL(pGpu, pKernelGsp, pKernelGsp->pFwsecUcode,
-                                        pKernelGsp->pWprMeta->frtsOffset), exit);
+                                 kgspExecuteFwsecFrts_HAL(pGpu,
+                                                          pKernelGsp,
+                                                          pKernelGsp->pFwsecUcode,
+                                                          pKernelGsp->pWprMeta->frtsOffset),
+                                 exit);
         }
     }
 
@@ -427,11 +429,13 @@ kgspBootstrapRiscvOSEarly_TU102
     NV_PRINTF(LEVEL_INFO, "Waiting for GSP fw RM to be ready...\n");
 
     // Link the status queue.
-    NV_ASSERT_OK_OR_GOTO(status, GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo),
-                          exit);
+    NV_ASSERT_OK_OR_GOTO(status,
+                         GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo),
+                         exit);
 
-    NV_ASSERT_OK_OR_GOTO(status, kgspWaitForRmInitDone(pGpu, pKernelGsp),
-                          exit);
+    NV_ASSERT_OK_OR_GOTO(status,
+                         kgspWaitForRmInitDone(pGpu, pKernelGsp),
+                         exit);
 
     NV_PRINTF(LEVEL_INFO, "GSP FW RM ready.\n");
 
@@ -560,24 +564,29 @@ kgspCalculateFbLayout_TU102
 
     //
     // Compute the start of the ELF.  Align to 64K to avoid issues with
-    // inherent alignment constraints (e.g. GC6 buffers which are allocated
-    // just below this offset).
+    // inherent alignment constraints.
     //
     pWprMeta->gspFwOffset = NV_ALIGN_DOWN64(pWprMeta->bootBinOffset - pWprMeta->sizeOfRadix3Elf, 0x10000);
 
-    NvU64 wprHeapSize = kgspGetWprHeapSize(pGpu, pKernelGsp);
+    const NvU64 wprHeapSize = kgspGetWprHeapSize(pGpu, pKernelGsp);
 
-    // Start of WPR region (1Mb aligned)
-    pWprMeta->gspFwWprStart =
-        NV_ALIGN_DOWN64(pWprMeta->gspFwOffset - wprHeapSize, 0x100000);
-
-    // GSP-RM heap in WPR, align to 1Mb
-    pWprMeta->gspFwHeapOffset = NV_ALIGN_UP64(pWprMeta->gspFwWprStart + sizeof *pWprMeta, 0x100000);
+    // GSP-RM heap in WPR, align to 1MB
+    pWprMeta->gspFwHeapOffset = NV_ALIGN_DOWN64(pWprMeta->gspFwOffset - wprHeapSize, 0x100000);
     pWprMeta->gspFwHeapSize = NV_ALIGN_DOWN64(pWprMeta->gspFwOffset - pWprMeta->gspFwHeapOffset, 0x100000);
 
-    // Non WPR heap
+    // Number of VF partitions allocating sub-heaps from the WPR heap
+    pWprMeta->gspFwHeapVfPartitionCount = pGpu->bVgpuGspPluginOffloadEnabled ? MAX_PARTITIONS_WITH_GFID : 0;
+
+    //
+    // Start of WPR region (128K alignment requirement, but 1MB aligned so that
+    // the extra padding sits in WPR instead of in between the end of the
+    // non-WPR heap and the start of WPR).
+    //
+    pWprMeta->gspFwWprStart = NV_ALIGN_DOWN64(pWprMeta->gspFwHeapOffset - sizeof *pWprMeta, 0x100000);
+
+    // Non WPR heap (1MB aligned)
     pWprMeta->nonWprHeapSize = kgspGetNonWprHeapSize(pGpu, pKernelGsp);
-    pWprMeta->nonWprHeapOffset = pWprMeta->gspFwWprStart - pWprMeta->nonWprHeapSize;
+    pWprMeta->nonWprHeapOffset = NV_ALIGN_DOWN64(pWprMeta->gspFwWprStart - pWprMeta->nonWprHeapSize, 0x100000);
 
     pWprMeta->gspFwRsvdStart = pWprMeta->nonWprHeapOffset;
 
@@ -605,15 +614,6 @@ kgspCalculateFbLayout_TU102
     pWprMeta->revision = GSP_FW_WPR_META_REVISION;
     pWprMeta->magic = GSP_FW_WPR_META_MAGIC;
 
-    if (pGpu->bVgpuGspPluginOffloadEnabled)
-    {
-        pKernelGsp->pWprMeta->driverModel = 1;
-    }
-    else
-    {
-        pKernelGsp->pWprMeta->driverModel = 0;
-    }
-
 #if 0
     NV_PRINTF(LEVEL_ERROR, "WPR meta data offset:     0x%016llx\n", pWprMeta->gspFwWprStart);
     NV_PRINTF(LEVEL_ERROR, "  magic:                  0x%016llx\n", pWprMeta->magic);
@@ -731,39 +731,11 @@ kgspResetHw_TU102
 )
 {
     GPU_FLD_WR_DRF_DEF(pGpu, _PGSP, _FALCON_ENGINE, _RESET, _TRUE);
-
-    // Reg read cycles needed for signal propagation.
-    for (NvU32 i = 0; i < FLCN_RESET_PROPAGATION_DELAY_COUNT; i++)
-    {
-        GPU_REG_RD32(pGpu, NV_PGSP_FALCON_ENGINE);
-    }
-
     GPU_FLD_WR_DRF_DEF(pGpu, _PGSP, _FALCON_ENGINE, _RESET, _FALSE);
 
-    // Reg read cycles needed for signal propagation.
-    for (NvU32 i = 0; i < FLCN_RESET_PROPAGATION_DELAY_COUNT; i++)
-    {
-        GPU_REG_RD32(pGpu, NV_PGSP_FALCON_ENGINE);
-    }
-
     return NV_OK;
 }
 
-/*!
- *  Return NV_OK along with the Engine Reset state.
- */
-NvBool
-kgspIsEngineInReset_TU102
-(
-    OBJGPU *pGpu,
-    KernelGsp *pKernelGsp
-)
-{
-    NvU32 val = GPU_REG_RD32(pGpu, NV_PGSP_FALCON_ENGINE);
-
-    return FLD_TEST_DRF(_PGSP_FALCON, _ENGINE, _RESET, _TRUE, val);
-}
-
 void
 kgspHealthCheck_TU102
 (
@@ -774,46 +746,31 @@ kgspHealthCheck_TU102
     NvU32 mb0 = GPU_REG_RD32(pGpu, NV_PGSP_MAILBOX(0));
 
     //
-    // Check for an error message in the GSP mailbox.  Any error reported here is
-    // almost certainly fatal.
+    // Check for an error message in the GSP mailbox.  Any error here is severe
+    // enough that it should be reported as an Xid.  Clear the error so more can
+    // potentially be reported by GSP, if it was able to recover.  In that case,
+    // it's possible that GSP will skip reporting some more errors that happened
+    // before the clear, and it will just update the "skipped" count.
     //
     if (FLD_TEST_DRF(_GSP, _ERROR, _TAG, _VAL, mb0))
     {
         NvU32 mb1 = GPU_REG_RD32(pGpu, NV_PGSP_MAILBOX(1));
-        NvU32 skipped = DRF_VAL(_GSP, _ERROR, _SKIPPED, mb0);
 
-        pKernelGsp->bFatalError = NV_TRUE;
-
-        // Clear the mailbox
         GPU_REG_WR32(pGpu, NV_PGSP_MAILBOX(0), 0);
 
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "********************************* GSP Failure **********************************\n");
 
         nvErrorLog_va((void*)pGpu, GSP_ERROR,
-                      "GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x.  The GPU likely needs to be reset.",
+                      "GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x (%d more errors skipped)",
                       DRF_VAL(_GSP, _ERROR, _TASK, mb0),
                       DRF_VAL(_GSP, _ERROR, _CODE, mb0),
                       DRF_VAL(_GSP, _ERROR, _REASON, mb0),
-                      mb1);
-        NVLOG_PRINTF(NV_PRINTF_MODULE, NVLOG_ROUTE_RM, LEVEL_ERROR, NV_PRINTF_ADD_PREFIX
-                     ("GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x"),
-                     DRF_VAL(_GSP, _ERROR, _TASK, mb0),
-                     DRF_VAL(_GSP, _ERROR, _CODE, mb0),
-                     DRF_VAL(_GSP, _ERROR, _REASON, mb0),
-                     mb1);
+                      mb1,
+                      DRF_VAL(_GSP, _ERROR, _SKIPPED, mb0));
 
-        // Check if GSP had more errors to report (unlikely)
-        if (skipped)
-        {
-            NV_PRINTF(LEVEL_ERROR, "%d more errors skipped\n", skipped);
-        }
-
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "********************************************************************************\n");
-
-        KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
-        kmemsysCheckEccCounts_HAL(pGpu, pKernelMemorySystem);
     }
 }
 
@@ -859,7 +816,7 @@ kgspService_TU102
         // provides RM a chance to handle it so we have better debugability
         // into GSP-RISCV issues.
         //
-        kgspDumpGspLogs(pKernelGsp, NV_FALSE);
+        kgspDumpGspLogs(pGpu, pKernelGsp, NV_FALSE);
         kgspHealthCheck_HAL(pGpu, pKernelGsp);
     }
     if (intrStatus & DRF_DEF(_PFALCON, _FALCON_IRQSTAT, _SWGEN0, _TRUE))
@@ -916,18 +873,6 @@ kgspWaitForProcessorSuspend_TU102
     return gpuTimeoutCondWait(pGpu, _kgspIsProcessorSuspended, pKernelGsp, NULL);
 }
 
-NvBool
-kgspIsWpr2Up_TU102
-(
-    OBJGPU    *pGpu,
-    KernelGsp *pKernelGsp
-)
-{
-    NvU32 data = GPU_REG_RD32(pGpu, NV_PFB_PRI_MMU_WPR2_ADDR_HI);
-    NvU32 wpr2HiVal = DRF_VAL(_PFB, _PRI_MMU_WPR2_ADDR_HI, _VAL, data);
-    return (wpr2HiVal != 0);
-}
-
 #define FWSECLIC_PROG_START_TIMEOUT     50000    // 50ms
 #define FWSECLIC_PROG_COMPLETE_TIMEOUT  2000000  // 2s
 
@@ -972,15 +917,132 @@ kgspWaitForGfwBootOk_TU102
         }
 
         status = gpuCheckTimeout(pGpu, &timeout);
+        if (status == NV_ERR_TIMEOUT)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                      "Timeout waiting for GFW_BOOT to complete\n");
+        }
     }
 
-    // The wait failed if we reach here (as above loop returns upon success).
-    NV_PRINTF(LEVEL_ERROR, "failed to wait for GFW_BOOT: 0x%x (progress 0x%x)\n",
-              status, GPU_REG_RD_DRF(pGpu,
-                        _PGC6,
-                        _AON_SECURE_SCRATCH_GROUP_05_0_GFW_BOOT,
-                        _PROGRESS));
-    NV_PRINTF(LEVEL_ERROR, "(the GPU may be in a bad state and may need to be reset)\n");
-
     return status;
 }
+
+void 
+kgspFreeSuspendResumeData_TU102
+(
+    OBJGPU    *pGpu,
+    KernelGsp *pKernelGsp
+)
+{
+    // release sr meta data resources
+    if (pKernelGsp->pSRMetaDescriptor != NULL)
+    {
+        memdescFree(pKernelGsp->pSRMetaDescriptor);
+        memdescDestroy(pKernelGsp->pSRMetaDescriptor);
+        pKernelGsp->pSRMetaDescriptor = NULL;
+    }
+
+    // release sr meta data resources
+    if (pKernelGsp->pSRRadix3Descriptor != NULL)
+    {
+        memdescFree(pKernelGsp->pSRRadix3Descriptor);
+        memdescDestroy(pKernelGsp->pSRRadix3Descriptor);
+        pKernelGsp->pSRRadix3Descriptor = NULL;
+    }
+}
+
+NV_STATUS
+kgspSavePowerMgmtState_TU102
+(
+    OBJGPU    *pGpu,
+    KernelGsp *pKernelGsp
+)
+{
+    GspFwSRMeta gspfwSRMeta;
+    NvP64 pVa = NvP64_NULL;
+    NvP64 pPriv = NvP64_NULL;
+    NV_STATUS nvStatus = NV_OK;
+
+    // Fill in GspFwSRMeta structure
+    portMemSet(&gspfwSRMeta, 0, sizeof(gspfwSRMeta));
+    gspfwSRMeta.magic                   = GSP_FW_SR_META_MAGIC;
+    gspfwSRMeta.revision                = GSP_FW_SR_META_REVISION;
+    gspfwSRMeta.sizeOfSuspendResumeData = pKernelGsp->pWprMeta->gspFwWprEnd - pKernelGsp->pWprMeta->gspFwWprStart;
+
+    
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         kgspCreateRadix3(pGpu,
+                                          pKernelGsp,
+                                          &pKernelGsp->pSRRadix3Descriptor,
+                                          NULL,
+                                          NULL,
+                                          gspfwSRMeta.sizeOfSuspendResumeData),
+                         exit_fail_cleanup);
+
+    gspfwSRMeta.sysmemAddrOfSuspendResumeData = memdescGetPhysAddr(pKernelGsp->pSRRadix3Descriptor, AT_GPU, 0);
+
+    // Create SR Metadata Area
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         memdescCreate(&pKernelGsp->pSRMetaDescriptor,
+                                       pGpu,
+                                       sizeof(GspFwSRMeta),
+                                       256,
+                                       NV_TRUE,
+                                       ADDR_SYSMEM,
+                                       NV_MEMORY_UNCACHED,
+                                       MEMDESC_FLAGS_NONE),
+                         exit_fail_cleanup);
+
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         memdescAlloc(pKernelGsp->pSRMetaDescriptor),
+                         exit_fail_cleanup);
+
+    // Copy SR Metadata Structure
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         memdescMap(pKernelGsp->pSRMetaDescriptor,
+                                    0,
+                                    memdescGetSize(pKernelGsp->pSRMetaDescriptor),
+                                    NV_TRUE,
+                                    NV_PROTECT_WRITEABLE,
+                                    &pVa,
+                                    &pPriv),
+                         exit_fail_cleanup);
+
+    portMemCopy(pVa, sizeof(gspfwSRMeta), &gspfwSRMeta, sizeof(gspfwSRMeta));
+
+    memdescUnmap(pKernelGsp->pSRMetaDescriptor,
+                 NV_TRUE, osGetCurrentProcess(),
+                 pVa, pPriv);
+
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         kgspExecuteBooterUnloadIfNeeded_HAL(pGpu,
+                                                             pKernelGsp, 
+                                                             memdescGetPhysAddr(pKernelGsp->pSRMetaDescriptor,AT_GPU, 0)),
+                         exit_fail_cleanup);
+
+    return nvStatus;
+
+exit_fail_cleanup:
+    kgspFreeSuspendResumeData_HAL(pGpu, pKernelGsp);
+    return nvStatus;
+}
+
+NV_STATUS
+kgspRestorePowerMgmtState_TU102
+(
+    OBJGPU    *pGpu,
+    KernelGsp *pKernelGsp
+)
+{
+    NV_STATUS nvStatus = NV_OK;
+
+    NV_ASSERT_OK_OR_GOTO(nvStatus,
+                         kgspExecuteBooterLoad_HAL(pGpu,
+                                                   pKernelGsp,
+                                                   memdescGetPhysAddr(pKernelGsp->pSRMetaDescriptor, AT_GPU,0)),
+                         exit_cleanup);
+
+exit_cleanup:
+    kgspFreeSuspendResumeData_HAL(pGpu, pKernelGsp);
+    return nvStatus;
+}
