@@ -183,6 +183,7 @@ intrStateUnload_TU102
     // Disable all interrupts since we're unloading
 
     intrWriteRegTopEnClear_HAL(pGpu, pIntr, 0, 0xFFFFFFFF, NULL);
+    intrWriteRegTopEnClear_HAL(pGpu, pIntr, 1, 0xFFFFFFFF, NULL);
 
     _intrClearLeafEnables_TU102(pGpu, pIntr);
 
@@ -203,39 +204,45 @@ intrCacheIntrFields_TU102
 )
 {
     NV_STATUS status = NV_OK;
-    NvU32 leafEnHi, leafEnLo;
-    NvU32 uvmSharedLeafIdxStart = NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(NV_CPU_INTR_UVM_SHARED_SUBTREE_START);
-    NvU32 uvmSharedLeafIdxEnd = NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST);
-    NvU32 stallSubtreeLast = intrGetStallSubtreeLast_HAL(pGpu, pIntr);
-    NvU32 i;
 
-    //
-    // Compile time assert to make sure we have only one client shared subtree.
-    // The below code assumes that.
-    //
-    ct_assert(NV_CPU_INTR_UVM_SHARED_SUBTREE_START == NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST);
-
-    // Now cache the leaf enable mask for the subtree shared with the client
-    leafEnHi = intrReadRegLeafEnSet_HAL(pGpu, pIntr, uvmSharedLeafIdxStart, NULL);
-    leafEnLo = intrReadRegLeafEnSet_HAL(pGpu, pIntr, uvmSharedLeafIdxEnd, NULL);
-
-    pIntr->uvmSharedCpuLeafEn = ((NvU64)(leafEnHi) << 32) | leafEnLo;
-    pIntr->uvmSharedCpuLeafEnDisableMask = intrGetUvmSharedLeafEnDisableMask_HAL(pGpu, pIntr);
-
-    //
-    // Cache the CPU_INTR_TOP_EN mask to clear when disabling stall interrupts
-    // (other interrupts are either not disabled or disabled selectively at leaf level)
-    //
-    for (i = NV_CPU_INTR_STALL_SUBTREE_START; i <= stallSubtreeLast; i++)
     {
-        pIntr->intrTopEnMask |= NVBIT(i);
+        NV2080_INTR_CATEGORY_SUBTREE_MAP uvmShared;
+        NV_ASSERT_OK_OR_RETURN(
+            intrGetSubtreeRange(pIntr,
+                                NV2080_INTR_CATEGORY_UVM_SHARED,
+                                &uvmShared));
+        //
+        // Assert to make sure we have only one client shared subtree.
+        // The below code assumes that.
+        //
+        NV_ASSERT_OR_RETURN(uvmShared.subtreeStart == uvmShared.subtreeEnd,
+                            NV_ERR_INVALID_STATE);
+
+        // Now cache the leaf enable mask for the subtree shared with the client
+        NvU32 leafEnHi = intrReadRegLeafEnSet_HAL(pGpu, pIntr,
+            NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(uvmShared.subtreeStart),
+            NULL);
+        NvU32 leafEnLo = intrReadRegLeafEnSet_HAL(pGpu, pIntr,
+            NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(uvmShared.subtreeStart),
+            NULL);
+
+        pIntr->uvmSharedCpuLeafEn = ((NvU64)(leafEnHi) << 32) | leafEnLo;
+        pIntr->uvmSharedCpuLeafEnDisableMask =
+            intrGetUvmSharedLeafEnDisableMask_HAL(pGpu, pIntr);
     }
 
+    //
+    // Cache the CPU_INTR_TOP_EN mask to clear when disabling stall
+    // interrupts (other interrupts are either not disabled or disabled
+    // selectively at leaf level)
+    //
+    pIntr->intrTopEnMask |= intrGetIntrTopLockedMask(pGpu, pIntr);
+
     OBJDISP *pDisp = GPU_GET_DISP(pGpu);
 
     // Cache client owned, shared interrupt, and display vectors for ease of use later
     pIntr->accessCntrIntrVector      = intrGetVectorFromEngineId(pGpu, pIntr, MC_ENGINE_IDX_ACCESS_CNTR,      NV_FALSE);
-    if (!gpuIsCCFeatureEnabled(pGpu))
+    if (!gpuIsCCFeatureEnabled(pGpu) || !gpuIsGspOwnedFaultBuffersEnabled(pGpu))
     {
         pIntr->replayableFaultIntrVector = intrGetVectorFromEngineId(pGpu, pIntr, MC_ENGINE_IDX_REPLAYABLE_FAULT, NV_FALSE);
     }
@@ -270,20 +277,30 @@ intrCacheIntrFields_TU102
         }
     }
 
-    //
-    // Now ensure that they're in the expected subtree (check right now so we
-    // don't have to check later in latency critical paths where this is assumed
-    // to be true)
-    //
-    if (NV_CTRL_INTR_GPU_VECTOR_TO_SUBTREE(pIntr->accessCntrIntrVector) != NV_CPU_INTR_UVM_SUBTREE_START)
     {
-        NV_PRINTF(LEVEL_ERROR, "UVM interrupt vectors for replayable fault and "
-            "access counter are in an unexpected subtree. Expected = 0x%x, actual = 0x%x\n",
-            NV_CPU_INTR_UVM_SUBTREE_START,
-            NV_CTRL_INTR_GPU_VECTOR_TO_SUBTREE(pIntr->accessCntrIntrVector));
-        DBG_BREAKPOINT();
-        status = NV_ERR_GENERIC;
-        goto exit;
+        //
+        // Now ensure that they're in the expected subtree (check right now so
+        // we don't have to check later in latency critical paths where this is
+        // assumed to be true)
+        //
+        NV2080_INTR_CATEGORY_SUBTREE_MAP uvmOwned;
+        NvU32 accessCntrSubtree = NV_CTRL_INTR_GPU_VECTOR_TO_SUBTREE(
+            pIntr->accessCntrIntrVector);
+        NV_ASSERT_OK_OR_RETURN(
+            intrGetSubtreeRange(pIntr,
+                                NV2080_INTR_CATEGORY_UVM_OWNED,
+                                &uvmOwned));
+        if (!(uvmOwned.subtreeStart <= accessCntrSubtree &&
+              accessCntrSubtree     <= uvmOwned.subtreeEnd))
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "UVM owned interrupt vector for access counter is in an unexpected subtree\n"
+                "Expected range = [0x%x, 0x%x], actual = 0x%x\n",
+                uvmOwned.subtreeStart, uvmOwned.subtreeEnd, accessCntrSubtree);
+            DBG_BREAKPOINT();
+            status = NV_ERR_GENERIC;
+            goto exit;
+        }
     }
 
 exit:
@@ -439,13 +456,13 @@ _intrEnableStall_TU102
     THREAD_STATE_NODE *pThreadState
 )
 {
-    NvU32 val, idx;
+    NvU32 idx;
 
     //
     // 1. Enable the UVM interrupts that RM currently owns at INTR_LEAF
     // level.
     //
-    val = _intrGetUvmLeafMask_TU102(pGpu, pIntr);
+    NvU32 val = _intrGetUvmLeafMask_TU102(pGpu, pIntr);
     idx = NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_REG(pIntr->accessCntrIntrVector);
     if (val != 0)
     {
@@ -457,12 +474,18 @@ _intrEnableStall_TU102
     // level, based on the cached value.
     //
 
-    //
-    // Compile time assert to make sure we have only one client shared subtree.
-    // The below code assumes that.
-    //
-    ct_assert(NV_CPU_INTR_UVM_SHARED_SUBTREE_START == NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST);
-    idx = NV_CPU_INTR_UVM_SHARED_SUBTREE_START;
+    {
+        NV2080_INTR_CATEGORY_SUBTREE_MAP uvmShared;
+        NV_ASSERT_OK(intrGetSubtreeRange(pIntr,
+                                         NV2080_INTR_CATEGORY_UVM_SHARED,
+                                         &uvmShared));
+        //
+        // Assert to make sure we have only one client shared subtree.
+        // The below code assumes that.
+        //
+        NV_ASSERT(uvmShared.subtreeStart == uvmShared.subtreeEnd);
+        idx = uvmShared.subtreeStart;
+    }
 
     if (NvU64_HI32(pIntr->uvmSharedCpuLeafEn) != 0)
     {
@@ -482,15 +505,30 @@ _intrEnableStall_TU102
     // We use the assumption that 1 == ENABLE below
     ct_assert(NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET_SUBTREE_ENABLE == 1);
 
-    //
-    // 3. Enable all interrupt subtrees (except nonstall) at top level. Nonstall
-    // enablement is handled by a different function.
-    //
-    val = 0xFFFFFFFF;
+    {
+        //
+        // 3. Enable all interrupt subtrees (except nonstall) at top level.
+        // Nonstall enablement is handled by a different function.
+        //
+        NvU64 mask = NV_U64_MAX;
 
-    val &= ~intrGetIntrTopNonStallMask_HAL(pGpu, pIntr);
+        mask &= ~intrGetIntrTopNonStallMask_HAL(pGpu, pIntr);
 
-    intrWriteRegTopEnSet_HAL(pGpu, pIntr, 0, val, pThreadState);
+        if (NvU64_LO32(mask) != 0)
+        {
+            intrWriteRegTopEnSet_HAL(pGpu, pIntr,
+                                     0,
+                                     NvU64_LO32(mask),
+                                     pThreadState);
+        }
+        if (NvU64_HI32(mask) != 0)
+        {
+            intrWriteRegTopEnSet_HAL(pGpu, pIntr,
+                                     1,
+                                     NvU64_HI32(mask),
+                                     pThreadState);
+        }
+    }
 }
 
 /*!
@@ -525,12 +563,18 @@ _intrDisableStall_TU102
     // level, except the ones that can be handled outside the GPU lock.
     //
 
-    //
-    // Compile time assert to make sure we have only one client shared subtree.
-    // The below code assumes that.
-    //
-    ct_assert(NV_CPU_INTR_UVM_SHARED_SUBTREE_START == NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST);
-    idx = NV_CPU_INTR_UVM_SHARED_SUBTREE_START;
+    {
+        NV2080_INTR_CATEGORY_SUBTREE_MAP uvmShared;
+        NV_ASSERT_OK(intrGetSubtreeRange(pIntr,
+                                         NV2080_INTR_CATEGORY_UVM_SHARED,
+                                         &uvmShared));
+        //
+        // Assert to make sure we have only one client shared subtree.
+        // The below code assumes that.
+        //
+        NV_ASSERT(uvmShared.subtreeStart == uvmShared.subtreeEnd);
+        idx = uvmShared.subtreeStart;
+    }
 
     if (!gpuIsStateLoaded(pGpu))
     {
@@ -588,7 +632,20 @@ _intrDisableStall_TU102
     // 3. Disable some interrupt subtrees at top level (information about which
     // ones to disable is cached in pIntr->intrTopEnMask)
     //
-    intrWriteRegTopEnClear_HAL(pGpu, pIntr, 0, pIntr->intrTopEnMask, pThreadState);
+    if (NvU64_LO32(pIntr->intrTopEnMask) != 0)
+    {
+        intrWriteRegTopEnClear_HAL(pGpu, pIntr,
+                                   0,
+                                   NvU64_LO32(pIntr->intrTopEnMask),
+                                   pThreadState);
+    }
+    if (NvU64_HI32(pIntr->intrTopEnMask) != 0)
+    {
+        intrWriteRegTopEnClear_HAL(pGpu, pIntr,
+                                   1,
+                                   NvU64_HI32(pIntr->intrTopEnMask),
+                                   pThreadState);
+    }
 }
 
 /*!
@@ -662,7 +719,7 @@ _intrGetUvmLeafMask_TU102
         NvBool bRmOwnsReplayableFault = !!(pKernelGmmu->uvmSharedIntrRmOwnsMask & RM_UVM_SHARED_INTR_MASK_MMU_REPLAYABLE_FAULT_NOTIFY);
         NvBool bRmOwnsAccessCntr      = !!(pKernelGmmu->uvmSharedIntrRmOwnsMask & RM_UVM_SHARED_INTR_MASK_HUB_ACCESS_COUNTER_NOTIFY);
 
-        if (bRmOwnsReplayableFault && !gpuIsCCFeatureEnabled(pGpu))
+        if (bRmOwnsReplayableFault && (!gpuIsCCFeatureEnabled(pGpu) || !gpuIsGspOwnedFaultBuffersEnabled(pGpu)))
         {
             val |= NVBIT(NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_BIT(pIntr->replayableFaultIntrVector));
         }
@@ -696,6 +753,7 @@ intrGetUvmSharedLeafEnDisableMask_TU102
     NvU32 intrVectorNonReplayableFault;
     NvU32 intrVectorFifoNonstall = NV_INTR_VECTOR_INVALID;
     NvU64 mask = 0;
+    NV2080_INTR_CATEGORY_SUBTREE_MAP uvmShared;
 
     // GSP RM services both MMU non-replayable fault and FIFO interrupts
     if (IS_GSP_CLIENT(pGpu))
@@ -719,22 +777,28 @@ intrGetUvmSharedLeafEnDisableMask_TU102
                 NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_REG(intrVectorFifoNonstall));
     }
 
-    // Ascertain that they're in the first leaf
-    NV_ASSERT(NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_REG(intrVectorNonReplayableFault) ==
-              NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(NV_CPU_INTR_UVM_SHARED_SUBTREE_START));
-
+    NV_ASSERT_OK(intrGetSubtreeRange(pIntr,
+                                     NV2080_INTR_CATEGORY_UVM_SHARED,
+                                     &uvmShared));
     //
-    // Compile-time ascertain that we only have 1 client subtree (we assume
+    // Ascertain that we only have 1 client subtree (we assume
     // this since we cache only 64 bits).
     //
-    ct_assert(NV_CPU_INTR_UVM_SHARED_SUBTREE_START == NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST);
+    NV_ASSERT(uvmShared.subtreeStart == uvmShared.subtreeEnd);
 
     //
-    // Compile-time ascertain that we only have 2 subtrees as this is what we currently support
-    // by only caching 64 bits
+    // Ascertain that we only have 2 subtrees as this is what we currently
+    // support by only caching 64 bits
     //
-    ct_assert((NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST) - 1) ==
-               NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(NV_CPU_INTR_UVM_SHARED_SUBTREE_START));
+    NV_ASSERT(
+        (NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(uvmShared.subtreeEnd) - 1) ==
+        NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(uvmShared.subtreeStart));
+
+
+    // Ascertain that they're in the first leaf
+    NV_ASSERT(
+        NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_REG(intrVectorNonReplayableFault) ==
+        NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(uvmShared.subtreeStart));
 
     mask |= NVBIT64(NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_BIT(intrVectorNonReplayableFault));
 
@@ -770,12 +834,13 @@ intrGetPendingStallEngines_TU102
     INTR_TABLE_ENTRY *pIntrTable;
     KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
     NvU32 intrTableSz, i;
-
-    NvU32 stallSubtreeLast = intrGetStallSubtreeLast_HAL(pGpu, pIntr);
+    NvU64 sanityCheckSubtreeMask = 0;
     NvU32 numIntrLeaves = intrGetNumLeaves_HAL(pGpu, pIntr);
     NV_ASSERT(numIntrLeaves <= NV_MAX_INTR_LEAVES);
     NvU32 intrLeafValues[NV_MAX_INTR_LEAVES];
 
+    sanityCheckSubtreeMask = intrGetIntrTopLegacyStallMask(pIntr);
+
     portMemSet(intrLeafValues, 0, numIntrLeaves * sizeof(NvU32));
     bitVectorClrAll(pEngines);
 
@@ -810,8 +875,9 @@ intrGetPendingStallEngines_TU102
         // is only to catch issues during code development. Should never happen
         // in practice
         //
-        if ((leafIndex < NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(NV_CPU_INTR_UVM_SUBTREE_START)) ||
-            (leafIndex > NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(stallSubtreeLast)))
+
+        if ((sanityCheckSubtreeMask &
+             NVBIT64(NV_CTRL_INTR_LEAF_IDX_TO_SUBTREE(leafIndex))) == 0)
         {
             NV_PRINTF(LEVEL_ERROR, "MC_ENGINE_IDX %u has invalid stall intr vector %u\n", pIntrTable[i].mcEngine, intrVector);
             DBG_BREAKPOINT();
@@ -906,8 +972,7 @@ intrRetriggerTopLevel_TU102
     Intr *pIntr
 )
 {
-    NvU32 val = 0;
-    NvU32 i;
+    NvU64 mask = 0;
 
     // We use the assumption that 1 == DISABLE below
     ct_assert(NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR_SUBTREE_DISABLE == 1);
@@ -923,28 +988,53 @@ intrRetriggerTopLevel_TU102
         // 1. If the alternate tree (nonstall tree) is handled "lockless", it
         // is not disabled during RM lock acquire, so needs re-arming.
         //
-        val |= intrGetIntrTopNonStallMask_HAL(pGpu, pIntr);
+        mask |= intrGetIntrTopNonStallMask_HAL(pGpu, pIntr);
     }
 
-    //
     // 2. UVM-owned interrupt tree (never disabled at top level)
+    mask |= intrGetIntrTopCategoryMask(pIntr, NV2080_INTR_CATEGORY_UVM_OWNED);
+
     // 3. UVM/RM shared interrupt tree (never disabled at top level)
-    //
-    for (i = NV_CPU_INTR_UVM_SUBTREE_START; i <= NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST; i++)
-    {
-        val |= NVBIT(i);
-    }
+    mask |= intrGetIntrTopCategoryMask(pIntr, NV2080_INTR_CATEGORY_UVM_SHARED);
 
     //
     // Bypass GPU_REG_WR32 that requires the GPU lock to be held (for some
     // register filters) by using the OS interface directly.
     //
-    osGpuWriteReg032(pGpu,
-                     GPU_GET_VREG_OFFSET(pGpu, NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR(0)),
-                     val);
-    osGpuWriteReg032(pGpu,
-                     GPU_GET_VREG_OFFSET(pGpu, NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET(0)),
-                     val);
+    // Clear all first, then set
+    //
+    if (NvU64_LO32(mask) != 0 &&
+        0 < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR__SIZE_1)
+    {
+        osGpuWriteReg032(pGpu,
+            GPU_GET_VREG_OFFSET(pGpu,
+                NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR(0)),
+            NvU64_LO32(mask));
+    }
+    if (NvU64_HI32(mask) != 0 &&
+        1 < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR__SIZE_1)
+    {
+        osGpuWriteReg032(pGpu,
+            GPU_GET_VREG_OFFSET(pGpu,
+                NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_CLEAR(1)),
+            NvU64_HI32(mask));
+    }
+    if (NvU64_LO32(mask) != 0 &&
+        0 < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET__SIZE_1)
+    {
+        osGpuWriteReg032(pGpu,
+            GPU_GET_VREG_OFFSET(pGpu,
+                NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET(0)),
+            NvU64_LO32(mask));
+    }
+    if (NvU64_HI32(mask) != 0 &&
+        1 < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET__SIZE_1)
+    {
+        osGpuWriteReg032(pGpu,
+            GPU_GET_VREG_OFFSET(pGpu,
+                NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_EN_SET(1)),
+            NvU64_HI32(mask));
+    }
 }
 
 /*!
@@ -967,11 +1057,8 @@ intrGetLeafStatus_TU102
     NvU32 subtreeIndex;
     NvU32 leafIndex;
 
-    // Read all the stalling interrupt leaf status
-    NvU32 stallSubtreeLast = intrGetStallSubtreeLast_HAL(pGpu, pIntr);
-    NV_ASSERT_OR_RETURN(NV_CPU_INTR_UVM_SUBTREE_START <= stallSubtreeLast, NV_ERR_INVALID_STATE);
-    subtreeIndex = NV_CPU_INTR_UVM_SUBTREE_START;
-    for (; subtreeIndex <= stallSubtreeLast; subtreeIndex++)
+    FOR_EACH_INDEX_IN_MASK(64, subtreeIndex,
+                           intrGetIntrTopLegacyStallMask(pIntr))
     {
         leafIndex = NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(subtreeIndex);
         if (pIntr->getProperty(pIntr, PDB_PROP_INTR_READ_ONLY_EVEN_NUMBERED_INTR_LEAF_REGS))
@@ -989,7 +1076,7 @@ intrGetLeafStatus_TU102
                 pLeafVals[leafIndex] = intrReadRegLeaf_HAL(pGpu, pIntr, leafIndex, pThreadState);
             }
         }
-    }
+    } FOR_EACH_INDEX_IN_MASK_END
 
     return NV_OK;
 }
@@ -1131,15 +1218,38 @@ intrDumpState_TU102
     }
 }
 
-/*!
- * @brief Gets the stall subtree end index
- */
-NvU32
-intrGetStallSubtreeLast_TU102(OBJGPU *pGpu, Intr *pIntr)
+
+NV_STATUS
+intrInitSubtreeMap_TU102
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr
+)
 {
-    return NV_CPU_INTR_STALL_SUBTREE_LAST;
+    NV2080_INTR_CATEGORY_SUBTREE_MAP *pCategoryEngine =
+        &pIntr->subtreeMap[NV2080_INTR_CATEGORY_ESCHED_DRIVEN_ENGINE];
+    pCategoryEngine->subtreeStart = NV_CPU_INTR_STALL_SUBTREE_START;
+    pCategoryEngine->subtreeEnd   = NV_CPU_INTR_STALL_SUBTREE_LAST;
+
+    NV2080_INTR_CATEGORY_SUBTREE_MAP *pCategoryEngineNotification =
+        &pIntr->subtreeMap[NV2080_INTR_CATEGORY_ESCHED_DRIVEN_ENGINE_NOTIFICATION];
+    pCategoryEngineNotification->subtreeStart = NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_SUBTREE(0);
+    pCategoryEngineNotification->subtreeEnd   = NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_SUBTREE(0);
+
+    NV2080_INTR_CATEGORY_SUBTREE_MAP *pCategoryUvmOwned =
+        &pIntr->subtreeMap[NV2080_INTR_CATEGORY_UVM_OWNED];
+    pCategoryUvmOwned->subtreeStart = NV_CPU_INTR_UVM_SUBTREE_START;
+    pCategoryUvmOwned->subtreeEnd   = NV_CPU_INTR_UVM_SUBTREE_LAST;
+
+    NV2080_INTR_CATEGORY_SUBTREE_MAP *pCategoryUvmShared =
+        &pIntr->subtreeMap[NV2080_INTR_CATEGORY_UVM_SHARED];
+    pCategoryUvmShared->subtreeStart = NV_CPU_INTR_UVM_SHARED_SUBTREE_START;
+    pCategoryUvmShared->subtreeEnd   = NV_CPU_INTR_UVM_SHARED_SUBTREE_LAST;
+
+    return NV_OK;
 }
 
+
 /*!
  * @brief Gets the number of leaf registers used
  */
@@ -1159,12 +1269,15 @@ intrGetLeafSize_TU102(OBJGPU *pGpu, Intr *pIntr)
     return NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_LEAF__SIZE_1;
 }
 
-/*!
- * @brief Gets the mask of INTR_TOP that covers nonstall interrupts
- */
-NvU32
-intrGetIntrTopNonStallMask_TU102(OBJGPU *pGpu, Intr *pIntr)
+
+NvU64
+intrGetIntrTopNonStallMask_TU102
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr
+)
 {
+    // TODO Bug 3823562 Remove these asserts
     // Compile-time assert against the highest set bit that will be returned
     #define NV_CPU_INTR_NOSTALL_SUBTREE_HIGHEST NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_SUBTREE(0)
 
@@ -1172,9 +1285,20 @@ intrGetIntrTopNonStallMask_TU102(OBJGPU *pGpu, Intr *pIntr)
     ct_assert(NV_CPU_INTR_NOSTALL_SUBTREE_HIGHEST < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_LEAF_EN_SET__SIZE_1);
     ct_assert(NV_CPU_INTR_NOSTALL_SUBTREE_HIGHEST < NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_LEAF_EN_CLEAR__SIZE_1);
 
-    return NVBIT32(NV_VIRTUAL_FUNCTION_PRIV_CPU_INTR_TOP_SUBTREE(0));
+    NvU64 mask = intrGetIntrTopCategoryMask(pIntr,
+        NV2080_INTR_CATEGORY_ESCHED_DRIVEN_ENGINE_NOTIFICATION);
+
+    //
+    // On all Ampere+ that use this TU102 HAL, Esched notification interrupts
+    // are also included in this if PDB_PROP_GPU_SWRL_GRANULAR_LOCKING is set.
+    //
+
+    // Sanity check that Intr.subtreeMap is initialized
+    NV_ASSERT(mask != 0);
+    return mask;
 }
 
+
 /*!
  * @brief Decode the interrupt mode for SW to use
  *
