@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2005-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2005-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -265,6 +265,7 @@ dmaAllocMapping_GM107
         Memory            *pMemory;
         NvU32              pageArrayGranularity;
         NvU8               pageShift;
+        NvU64              physPageSize;
     } *pLocals = portMemAllocNonPaged(sizeof(*pLocals));
     // Heap Allocate to avoid stack overflow
 
@@ -337,10 +338,43 @@ dmaAllocMapping_GM107
 
     // Get pageSize
     pLocals->pTempMemDesc = memdescGetMemDescFromGpu(pAdjustedMemDesc, pGpu);
-    pLocals->pageSize     = memdescGetPageSize(pLocals->pTempMemDesc, addressTranslation);
-
-    // Get pte granularity
+    
+    // Get physical allocation granularity and page size.
     pLocals->pageArrayGranularity = pLocals->pTempMemDesc->pageArrayGranularity;
+    pLocals->physPageSize = memdescGetPageSize64(pLocals->pTempMemDesc, addressTranslation);
+    
+    // retrieve mapping page size from flags
+    switch(DRF_VAL(OS46, _FLAGS, _PAGE_SIZE, flags))
+    {
+        case NVOS46_FLAGS_PAGE_SIZE_DEFAULT:
+        case NVOS46_FLAGS_PAGE_SIZE_BOTH:
+            pLocals->pageSize = memdescGetPageSize64(pLocals->pTempMemDesc, addressTranslation);
+            break;
+        case NVOS46_FLAGS_PAGE_SIZE_4KB:
+            pLocals->pageSize = RM_PAGE_SIZE;
+            break;
+        case NVOS46_FLAGS_PAGE_SIZE_BIG:    
+            // case for arch specific 128K
+            pLocals->pageSize = pLocals->vaspaceBigPageSize;
+            break;
+        case NVOS46_FLAGS_PAGE_SIZE_HUGE:
+            pLocals->pageSize = RM_PAGE_SIZE_HUGE;
+            break;
+        default:
+            NV_PRINTF(LEVEL_ERROR, "Unknown page size flag encountered during mapping\n");
+            return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    NV_PRINTF(LEVEL_INFO, "Picked Page size based on flags: 0x%llx flagVal: 0x%x\n",
+                           pLocals->pageSize, DRF_VAL(OS46, _FLAGS, _PAGE_SIZE, flags));
+
+    if (pLocals->physPageSize < pLocals->pageSize)
+    {
+        NV_PRINTF(LEVEL_WARNING, "Requested mapping at larger page size than the physical granularity "
+                                 "PhysPageSize = 0x%llx MapPageSize = 0x%llx. Overriding to physical page granularity...\n",
+                                 pLocals->physPageSize, pLocals->pageSize);
+        pLocals->pageSize = pLocals->physPageSize;
+    }
 
     if (memdescGetFlag(pLocals->pTempMemDesc, MEMDESC_FLAGS_DEVICE_READ_ONLY))
     {
@@ -348,30 +382,6 @@ dmaAllocMapping_GM107
             status = NV_ERR_INVALID_ARGUMENT; goto cleanup);
     }
 
-    // For verify purposes we should allow small page override for mapping.
-    // This will be used for testing VASpace interop.
-    if (RMCFG_FEATURE_PLATFORM_MODS &&
-        FLD_TEST_DRF(OS46, _FLAGS, _PAGE_SIZE, _4KB, flags) &&
-        kgmmuIsVaspaceInteropSupported(pKernelGmmu))
-    {
-        pLocals->pageSize = RM_PAGE_SIZE;
-        SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY)
-            memdescSetPageSize(memdescGetMemDescFromGpu(pAdjustedMemDesc, pGpu), addressTranslation, (NvU32)pLocals->pageSize);
-        SLI_LOOP_END
-    }
-    else if (kgmmuIsPerVaspaceBigPageEn(pKernelGmmu) &&
-             (pLocals->pageSize >= RM_PAGE_SIZE_64K))
-    {
-        NV_ASSERT(pLocals->pageSize != RM_PAGE_SIZE_HUGE);
-
-        //
-        // This is a temp WAR till the memdesc->_pageSize is cleaned up
-        // If the memdesc page size is >= the smallest big page size then
-        // we will correct it to the Big page size of the VASpace
-        //
-        pLocals->pageSize = pLocals->vaspaceBigPageSize;
-    }
-
     //
     // Force BAR1 VA pageSize at bigPageSize only if total BAR1 size is less
     // than threshold(default: 256MB) to not waste BAR1.
@@ -412,6 +422,18 @@ dmaAllocMapping_GM107
     if (NV_OK != status)
         goto cleanup;
 
+    //
+    // When compression is enabled mapping at 4K is not supported due to
+    // RM allocating one comptagline per 64KB allocation (From Pascal to Turing).
+    // See bug 3909010
+    //
+    if ((pLocals->pageSize == RM_PAGE_SIZE) &&
+        memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_COMPRESSIBLE, pLocals->kind))
+    {
+        NV_PRINTF(LEVEL_WARNING, "Requested 4K mapping on compressible sufrace. Overriding to physical page granularity...");
+        pLocals->pageSize = pLocals->physPageSize;
+    }
+
 #ifdef DEBUG
     // Check for subdevices consistency if broadcast memdesc is passed in
     if (memdescHasSubDeviceMemDescs(pAdjustedMemDesc))
@@ -458,7 +480,9 @@ dmaAllocMapping_GM107
     {
         // FIXME: This is broken for page size > 4KB and page offset
         //        that crosses a page boundary (can overrun pPteArray).
-        pLocals->pteCount = pLocals->pageCount;
+        // --
+        // page count is one more than integral division in case of presence of offset hence being rounded up
+        pLocals->pteCount = RM_ALIGN_UP((pLocals->pTempMemDesc->Size + pLocals->pageOffset), pLocals->pageArrayGranularity) >> BIT_IDX_32(pLocals->pageArrayGranularity);
     }
 
     // Disable PLC Compression for FLA->PA Mapping because of the HW Bug: 3046774
@@ -496,7 +520,7 @@ dmaAllocMapping_GM107
 
         if (pLocals->bIsMemContiguous)
         {
-            pLocals->overMap = pLocals->pageCount + NvU64_LO32((pLocals->pageOffset + (pLocals->pageArrayGranularity - 1)) / pLocals->pageArrayGranularity);
+            pLocals->overMap = pLocals->pageCount + NvU64_LO32((pLocals->pageOffset + (pLocals->pageSize - 1)) / pLocals->pageSize);
         }
         else
         {
@@ -605,13 +629,13 @@ dmaAllocMapping_GM107
         NvU64           vaAlign   = NV_MAX(pLocals->pageSize, compAlign);
         NvU64           vaSize    = RM_ALIGN_UP(pLocals->mapLength, vaAlign);
         NvU64           pageSizeLockMask = 0;
+        pGVAS = dynamicCast(pVAS, OBJGVASPACE);
 
         if (FLD_TEST_DRF(OS46, _FLAGS, _PAGE_SIZE, _BOTH, flags))
         {
             vaAlign = NV_MAX(vaAlign, pLocals->vaspaceBigPageSize);
             vaSize  = RM_ALIGN_UP(pLocals->mapLength, vaAlign);
         }
-
         //
         // Third party code path, nvidia_p2p_get_pages, expects on BAR1 VA to be
         // always aligned at 64K.
@@ -641,6 +665,14 @@ dmaAllocMapping_GM107
                     goto cleanup;
                 }
             }
+            if (pGVAS != NULL && gvaspaceIsInternalVaRestricted(pGVAS))
+            {
+                if ((pLocals->vaRangeLo >= pGVAS->vaStartInternal && pLocals->vaRangeLo <= pGVAS->vaLimitInternal) ||
+                    (pLocals->vaRangeHi <= pGVAS->vaLimitInternal && pLocals->vaRangeHi >= pGVAS->vaStartInternal))
+                {
+                    return NV_ERR_INVALID_PARAMETER;
+                }
+            }
         }
         else if (pDma->getProperty(pDma, PDB_PROP_DMA_RESTRICT_VA_RANGE))
         {
@@ -666,7 +698,6 @@ dmaAllocMapping_GM107
         // Clients can pass an allocation flag to the device or VA space constructor
         // so that mappings and allocations will fail without an explicit address.
         //
-        pGVAS = dynamicCast(pVAS, OBJGVASPACE);
         if (pGVAS != NULL)
         {
             if ((pGVAS->flags & VASPACE_FLAGS_REQUIRE_FIXED_OFFSET) &&
@@ -676,6 +707,18 @@ dmaAllocMapping_GM107
                 NV_PRINTF(LEVEL_ERROR, "The VA space requires all allocations to specify a fixed address\n");
                 goto cleanup;
             }
+
+            // 
+            // Bug 3610538 clients can allocate GPU VA, during mapping for ctx dma.
+            // But if clients enable RM to map internal buffers in a reserved 
+            // range of VA for unlinked SLI in Linux, we want to tag these 
+            // allocations as "client allocated", so that it comes outside of 
+            // RM internal region.
+            //
+            if (gvaspaceIsInternalVaRestricted(pGVAS))
+            {
+                allocFlags.bClientAllocation = NV_TRUE;
+            }
         }
 
         status = vaspaceAlloc(pVAS, vaSize, vaAlign, pLocals->vaRangeLo, pLocals->vaRangeHi,
@@ -894,23 +937,13 @@ dmaAllocMapping_GM107
 
             if (pPeerGpu != NULL)
             {
-                if (IS_VIRTUAL_WITH_SRIOV(pMappingGpu) &&
-                    !gpuIsWarBug200577889SriovHeavyEnabled(pMappingGpu))
-                {
-                    pLocals->peerNumber = kbusGetNvlinkPeerId_HAL(pMappingGpu,
-                                                                GPU_GET_KERNEL_BUS(pMappingGpu),
-                                                                pPeerGpu);
-                }
-                else
-                {
-                    KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pMappingGpu);
+                KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pMappingGpu);
 
-                    if ((pKernelNvlink != NULL) &&
-                        knvlinkIsNvlinkP2pSupported(pMappingGpu, pKernelNvlink, pPeerGpu))
-                    {
-                        pLocals->peerNumber = kbusGetPeerId_HAL(pMappingGpu, GPU_GET_KERNEL_BUS(pMappingGpu),
-                                                            pPeerGpu);
-                    }
+                if ((pKernelNvlink != NULL) &&
+                     knvlinkIsNvlinkP2pSupported(pMappingGpu, pKernelNvlink, pPeerGpu))
+                {
+                    pLocals->peerNumber = kbusGetPeerId_HAL(pMappingGpu, GPU_GET_KERNEL_BUS(pMappingGpu),
+                                                           pPeerGpu);
                 }
             }
             else
@@ -1027,9 +1060,15 @@ dmaAllocMapping_GM107
         //
         *pVaddr = pLocals->vaLo + pLocals->pageOffset;
 
+        // Fill in the final mapping page size for client mappings.
+        if (pCliMapInfo != NULL)
+        {
+            pCliMapInfo->pDmaMappingInfo->mapPageSize = pLocals->pageSize;
+        }
+
         SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY)
         // This is needed for cliDB tracking of the map.
-        memdescSetPageSize(memdescGetMemDescFromGpu(pAdjustedMemDesc, pGpu), addressTranslation, NvU64_LO32(pLocals->pageSize));
+        memdescSetPageSize(memdescGetMemDescFromGpu(pAdjustedMemDesc, pGpu), addressTranslation, pLocals->pageSize);
         SLI_LOOP_END
     }
     else
@@ -1106,7 +1145,7 @@ dmaFreeMapping_GM107
 
     SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY)
         // ensure the page size has been set before continuing
-        NV_ASSERT(memdescGetPageSize(memdescGetMemDescFromGpu(pMemDesc, pGpu), VAS_ADDRESS_TRANSLATION(pVAS)) != 0);
+        NV_ASSERT(memdescGetPageSize64(memdescGetMemDescFromGpu(pMemDesc, pGpu), VAS_ADDRESS_TRANSLATION(pVAS)) != 0);
     SLI_LOOP_END
 
     if (pCliMapInfo)
@@ -1138,7 +1177,11 @@ dmaFreeMapping_GM107
             }
 
             pTempMemDesc = memdescGetMemDescFromGpu(pMemDesc, pGpu);
-            pageSize     = memdescGetPageSize(pTempMemDesc, VAS_ADDRESS_TRANSLATION(pVAS));
+
+            NV_ASSERT_OR_RETURN(pCliMapInfo != NULL, NV_ERR_INVALID_STATE);
+            NV_ASSERT_OR_RETURN(pCliMapInfo->pDmaMappingInfo->mapPageSize != 0, NV_ERR_INVALID_STATE);
+
+            pageSize     = pCliMapInfo->pDmaMappingInfo->mapPageSize;
             pageOffset   = memdescGetPhysAddr(pTempMemDesc, VAS_ADDRESS_TRANSLATION(pVAS), 0) & (pageSize - 1);
             mapLength    = RM_ALIGN_UP(pageOffset + pTempMemDesc->Size, pageSize);
             vaLo         = RM_ALIGN_DOWN(vAddr, pageSize);
@@ -1165,7 +1208,7 @@ dmaFreeMapping_GM107
                                           NVLINK_INVALID_FABRIC_ADDR,
                                           deferInvalidate,
                                           NV_FALSE,
-					  pageSize);
+                                          pageSize);
             if (status != NV_OK)
             {
                 NV_PRINTF(LEVEL_ERROR, "error updating VA space.\n");
@@ -1602,7 +1645,7 @@ dmaUpdateVASpace_GF100
     NvU64       fabricAddr,
     NvU32       deferInvalidate,
     NvBool      bSparse,
-    NvU32       pageSize
+    NvU64       pageSize
 )
 {
     KernelBus     *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
@@ -1623,8 +1666,8 @@ dmaUpdateVASpace_GF100
     KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
     NvBool      bFillPteMem = !!(flags & DMA_UPDATE_VASPACE_FLAGS_FILL_PTE_MEM);
     NvBool      bUnmap = !bFillPteMem &&
-                          (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
-                          (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE);
+                         (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
+                         (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE);
     NvBool      bIsIndirectPeer;
     VAS_PTE_UPDATE_TYPE update_type;
 
@@ -1635,16 +1678,12 @@ dmaUpdateVASpace_GF100
     readDisable = !!(flags & DMA_UPDATE_VASPACE_FLAGS_SHADER_WRITE_ONLY);
     bIsIndirectPeer = !!(flags & DMA_UPDATE_VASPACE_FLAGS_INDIRECT_PEER);
 
-    if (pGpu->bEnableBar1SparseForFillPteMemUnmap)
+    NV_ASSERT_OR_RETURN(pageSize, NV_ERR_INVALID_ARGUMENT);
+
+    vaSpaceBigPageSize = vaspaceGetBigPageSize(pVAS);
+    if ((pageSize == RM_PAGE_SIZE_64K) || (pageSize == RM_PAGE_SIZE_128K))
     {
-        OBJGVASPACE *pGVAS = dynamicCast(pVAS, OBJGVASPACE);
-        if (bFillPteMem &&
-            (pGVAS->flags & VASPACE_FLAGS_BAR_BAR1) &&
-            (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
-            (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE))
-        {
-            bSparse = NV_TRUE;
-        }
+        NV_ASSERT_OR_RETURN(pageSize == vaSpaceBigPageSize, NV_ERR_INVALID_STATE);
     }
 
     //
@@ -1684,27 +1723,6 @@ dmaUpdateVASpace_GF100
     encrypted = (flags & DMA_UPDATE_VASPACE_FLAGS_DISABLE_ENCRYPTION) ? 0 :
         memdescGetFlag(pMemDesc, MEMDESC_FLAGS_ENCRYPTED);
 
-    vaSpaceBigPageSize = vaspaceGetBigPageSize(pVAS);
-    pageSize = memdescGetPageSize(pMemDesc, VAS_ADDRESS_TRANSLATION(pVAS));
-    NV_ASSERT_OR_RETURN(pageSize, NV_ERR_INVALID_ARGUMENT);
-
-    if (kgmmuIsPerVaspaceBigPageEn(pKernelGmmu) &&
-             (pageSize >= RM_PAGE_SIZE_64K))
-    {
-        NV_ASSERT(pageSize != RM_PAGE_SIZE_HUGE);
-        NV_ASSERT(vaSpaceBigPageSize);
-
-        //
-        // This is a temp WAR till the memdesc->_pageSize is cleaned up
-        // If the memdesc page size is >= the smallest big page size then
-        // we will correct it to the Big page size of the VASpace
-        // This will also set it to the correct size for the memDesc
-        //
-        pageSize = vaSpaceBigPageSize;
-    }
-
-    NV_ASSERT_OR_RETURN(pageSize, NV_ERR_INVALID_ARGUMENT);
-
     // Check this here so we don't have to in the loop(s) below as necessary.
     if ((flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_PADDR) && (pPageArray == NULL))
     {
@@ -1734,43 +1752,51 @@ dmaUpdateVASpace_GF100
     }
 
     //
-    // VMM-TODO: Merge into PL1 traveral.
+    // If we have dynamic granularity page arrays enabled we will never
+    // encounter a case where a larger page granularity physical surface gets
+    // represented by a smaller granularity pageArray.
     //
-    // If the pageSize of the mapping != 4K then be sure that the 4k pages
-    // making up the big physical page are contiguous. This is currently
-    // necessary since pMemDesc->PteArray is always in terms of 4KB pages.
-    // Different large pages do not have to be contiguous with each other.
-    // This check isn't needed for contig allocations.
-    //
-    if (pPageArray && (pageSize != RM_PAGE_SIZE) && (pPageArray->count > 1) &&
-        !(flags & DMA_UPDATE_VASPACE_FLAGS_SKIP_4K_PTE_CHECK))
+    if (!pMemoryManager->bEnableDynamicGranularityPageArrays)
     {
-        NvU32 i, j;
-        RmPhysAddr pageAddr, pagePrevAddr;
-
-        for (i = 0; i < pPageArray->count; i += j)
+        //
+        // VMM-TODO: Merge into PL1 traveral.
+        //
+        // If the pageSize of the mapping != 4K then be sure that the 4k pages
+        // making up the big physical page are contiguous. This is currently
+        // necessary since pMemDesc->PteArray is always in terms of 4KB pages.
+        // Different large pages do not have to be contiguous with each other.
+        // This check isn't needed for contig allocations.
+        //
+        if (pPageArray && (pageSize != RM_PAGE_SIZE) && (pPageArray->count > 1) &&
+        !(flags & DMA_UPDATE_VASPACE_FLAGS_SKIP_4K_PTE_CHECK))
         {
-            for (j = i + 1; j < pPageArray->count; j++)
+            NvU32 i, j;
+            RmPhysAddr pageAddr, pagePrevAddr;
+
+            for (i = 0; i < pPageArray->count; i += j)
             {
-                pagePrevAddr = dmaPageArrayGetPhysAddr(pPageArray, j - 1);
-                pageAddr     = dmaPageArrayGetPhysAddr(pPageArray, j);
-
-                if ((1 + (pagePrevAddr/(RM_PAGE_SIZE))) !=
-                         (pageAddr/(RM_PAGE_SIZE)))
+                for (j = i + 1; j < pPageArray->count; j++)
                 {
-                    NV_PRINTF(LEVEL_ERROR,
-                              "MMU: given non-contig 4KB pages for %dkB mapping\n",
-                              pageSize / 1024);
-                    DBG_BREAKPOINT();
-                    return NV_ERR_GENERIC;
-                }
+                    pagePrevAddr = dmaPageArrayGetPhysAddr(pPageArray, j - 1);
+                    pageAddr     = dmaPageArrayGetPhysAddr(pPageArray, j);
 
-                // Are we at the pageSize boundary yet?
-                if ((pageAddr + RM_PAGE_SIZE)
-                     % pageSize == 0)
-                {
-                    j++;
-                    break;
+                    if ((1 + (pagePrevAddr/(RM_PAGE_SIZE))) !=
+                             (pageAddr/(RM_PAGE_SIZE)))
+                    {
+                        NV_PRINTF(LEVEL_ERROR,
+                                 "MMU: given non-contig 4KB pages for %lldkB mapping\n",
+                                 pageSize / 1024);
+                        DBG_BREAKPOINT();
+                        return NV_ERR_GENERIC;
+                    }
+
+                    // Are we at the pageSize boundary yet?
+                    if ((pageAddr + RM_PAGE_SIZE)
+                        % pageSize == 0)
+                    {
+                        j++;
+                        break;
+                    }
                 }
             }
         }
@@ -1825,7 +1851,7 @@ dmaUpdateVASpace_GF100
 
         // MMU_MAP_CTX
         mapTarget.pLevelFmt            = mmuFmtFindLevelWithPageShift(pFmt->pRoot,
-                                                                BIT_IDX_32(pageSize));
+                                                                BIT_IDX_64(pageSize));
         mapTarget.pIter                = &mapIter;
         mapTarget.MapNextEntries       = _gmmuWalkCBMapNextEntries_RmAperture;
         mapTarget.pageArrayGranularity = pMemDesc->pageArrayGranularity;
@@ -2203,9 +2229,20 @@ dmaMapBuffer_GM107
         vaAlign = NV_MAX(vaAlign, temp);
     }
 
+    // Set this first in case we ignore DMA_ALLOC_VASPACE_USE_RM_INTERNAL_VALIMITS next
     rangeLo = vaspaceGetVaStart(pVAS);
     rangeHi = vaspaceGetVaLimit(pVAS);
 
+    if (flagsForAlloc & DMA_ALLOC_VASPACE_USE_RM_INTERNAL_VALIMITS)
+    {
+        OBJGVASPACE *pGVAS = dynamicCast(pVAS, OBJGVASPACE);
+        if (pGVAS)
+        {
+            rangeLo = pGVAS->vaStartInternal;
+            rangeHi = pGVAS->vaLimitInternal;
+        }
+    }
+
     // If trying to conserve 32bit address space, map RM buffers at 4GB+
     if (pDma->getProperty(pDma, PDB_PROP_DMA_ENFORCE_32BIT_POINTER) &&
         (pVASpaceHeap->free > NVBIT64(32)))
