@@ -551,6 +551,7 @@ NV_STATUS semaphoreFillGPUVATimestamp
 )
 {
     NvU64                 semaphoreGPUVA;
+    NvU64                 semaphoreGPUVAOffset;
     CLI_DMA_MAPPING_INFO *pDmaMappingInfo;
     NvU64                 offset;
     NvU32                 timeHi, timeLo;
@@ -570,8 +571,11 @@ NV_STATUS semaphoreFillGPUVATimestamp
     if (status != NV_OK)
         return status;
 
-    if (!portSafeAddU64(SemaphoreGPUVABase,
-                        (NvU64) (Index * sizeof(NvGpuSemaphore)),
+    if (!portSafeMulU64((NvU64) Index,
+                        (NvU64) sizeof(NvGpuSemaphore),
+                        &semaphoreGPUVAOffset) ||
+        !portSafeAddU64(SemaphoreGPUVABase,
+                        semaphoreGPUVAOffset,
                         &semaphoreGPUVA))
     {
         return NV_ERR_INVALID_ARGUMENT;
