@@ -43,7 +43,6 @@
 /* ------------------ static and helper functions prototypes------------------*/
 static NvU32 translateAllocFlagsToVASpaceFlags(NvU32 allocFlags, NvU32 *translatedFlags);
 static NvU32 translatePageSizeToVASpaceFlags(NV_VASPACE_ALLOCATION_PARAMETERS *pNvVASpaceAllocParams);
-static void destroyMemDesc(Device *pDevice, NvHandle hVASpace);
 static NV_STATUS _vaspaceapiManagePageLevelsForSplitVaSpace(OBJGPU *pGpu, NvHandle hClient, NvU32 gpuMask, NvU32 flags, VASPACEAPI_MANAGE_PAGE_LEVELS_ACTION action);
 
 NvBool
@@ -351,6 +350,18 @@ vaspaceapiConstruct_IMPL
             }
         }
 
+        // 
+        // Bug 3610538 For unlinked SLI, clients want to restrict internal buffers to
+        // Internal VA range. setting internal va range to match what we use for
+        // windows.
+        //
+        if (allocFlags & NV_VASPACE_ALLOCATION_FLAGS_VA_INTERNAL_LIMIT)
+        {
+            vaStartInternal = SPLIT_VAS_SERVER_RM_MANAGED_VA_START;
+            vaLimitInternal = SPLIT_VAS_SERVER_RM_MANAGED_VA_START +
+                              SPLIT_VAS_SERVER_RM_MANAGED_VA_SIZE - 1;
+        }
+
         // Finally call the factory
         status = vmmCreateVaspace(pVmm, pParams->externalClassId,
                                   pNvVASpaceAllocParams->index,
@@ -522,7 +533,6 @@ vaspaceapiDestruct_IMPL(VaSpaceApi *pVaspaceApi)
                                                          VASPACEAPI_MANAGE_PAGE_LEVELS_RELEASE);
     }
 
-    destroyMemDesc(pDevice, hVASpace);
     if ((vaspaceGetFlags(pVaspaceApi->pVASpace) & VASPACE_FLAGS_FLA))
     {
         if (GPU_GET_KERNEL_BUS(pGpu)->flaInfo.pFlaVAS == NULL)
@@ -561,52 +571,6 @@ skip_destroy:
               pCallContext->pResourceRef->pParentRef);
 }
 
-/*--------------------------static and helper functions ----------------------*/
-// move this to Device
-/**
- * @brief Destroy associated memory with this vaspace.
- *
- * @param[in] pDevice  Pointer to Device instance
- * @param[in] hVASpace VASpace handle of the vaspace that will be deleted
- **/
-static void
-destroyMemDesc
-(
-    Device  *pDevice,
-    NvHandle hVASpace
-)
-{
-    VirtualMemory *pVirtualMemory = NULL;
-    RsClient      *pClient = RES_GET_CLIENT(pDevice);
-    NODE          *pNode = NULL;
-    RM_API        *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-
-    //
-    // RS-TODO: Convert to resource server dependency tracking system.
-    // stdmemConstruct calls add refAddDependant() but does it work properly for
-    // sharing (?)
-    //
-
-    // Check if any memory linked to this VASpace hasn't been freed.
-    btreeEnumStart(0, &pNode, pDevice->DevMemoryTable);
-    while (pNode != NULL)
-    {
-        Memory *pMemory = pNode->Data;
-        pVirtualMemory = dynamicCast(pMemory, VirtualMemory);
-        btreeEnumNext(&pNode, pDevice->DevMemoryTable);
-
-        if ((pVirtualMemory != NULL) &&
-            virtmemMatchesVASpace(pVirtualMemory, pClient->hClient, hVASpace))
-        {
-            // Free hMemory
-            pRmApi->Free(pRmApi, pClient->hClient, RES_GET_HANDLE(pVirtualMemory));
-
-            // Restart iteration as memory will be freed.
-            btreeEnumStart(0, &pNode, pDevice->DevMemoryTable);
-        }
-    }
-}
-
 /**
  * @brief Translate flags to vaspace equivalent flags and perform error check.
  *
@@ -667,6 +631,10 @@ static NV_STATUS translateAllocFlagsToVASpaceFlags(NvU32 allocFlags, NvU32 *tran
     {
         flags |= VASPACE_FLAGS_REQUIRE_FIXED_OFFSET;
     }
+    if (allocFlags & NV_VASPACE_ALLOCATION_FLAGS_VA_INTERNAL_LIMIT)
+    {
+        flags |= VASPACE_FLAGS_RESTRICTED_RM_INTERNAL_VALIMITS;
+    }
     flags |= VASPACE_FLAGS_ENABLE_VMM;
 
     // Validate the flag combinations
