@@ -308,13 +308,19 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                 }
                 else
                 {
+                    const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                            kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
                     NV_ASSERT(0 == NvU64_HI32(pMemoryManager->Ram.fbTotalMemSizeMb << 10));
-                    data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10), (pMemoryManager->Ram.fbOverrideSizeMb << 10)));
+                    data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10), 
+                                             (pMemoryManager->Ram.fbOverrideSizeMb << 10))
+                                             - pMemsysConfig->fbOverrideStartKb);
                     break;
                 }
             }
             case NV2080_CTRL_FB_INFO_INDEX_RAM_SIZE:
             {
+                const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                        kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
                 if (pMemoryPartitionHeap != NULL)
                 {
                     NvU32 heapSizeKb;
@@ -336,11 +342,15 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                     break;
                 }
                 NV_ASSERT(0 == NvU64_HI32(pMemoryManager->Ram.fbTotalMemSizeMb << 10));
-                data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10), (pMemoryManager->Ram.fbOverrideSizeMb << 10)));
+                data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10),
+                                         (pMemoryManager->Ram.fbOverrideSizeMb << 10))
+                                         - pMemsysConfig->fbOverrideStartKb);
                 break;
             }
             case NV2080_CTRL_FB_INFO_INDEX_USABLE_RAM_SIZE:
             {
+                const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                        kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
                 if (pMemoryPartitionHeap != NULL)
                 {
                     NvU32 heapSizeKb;
@@ -362,11 +372,15 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                     break;
                 }
                 NV_ASSERT(0 == NvU64_HI32(pMemoryManager->Ram.fbUsableMemSize >> 10));
-                data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbUsableMemSize >> 10), (pMemoryManager->Ram.fbOverrideSizeMb << 10)));
+                data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbUsableMemSize >> 10 ),
+                                         (pMemoryManager->Ram.fbOverrideSizeMb << 10))
+                                         - pMemsysConfig->fbOverrideStartKb);
                 break;
             }
             case NV2080_CTRL_FB_INFO_INDEX_HEAP_SIZE:
             {
+                const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                        kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
                 if (bIsPmaEnabled)
                 {
                     pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
@@ -381,6 +395,7 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                     NV_ASSERT(NvU64_HI32(size >> 10) == 0);
                     data = NvU64_LO32(size >> 10);
                 }
+                data -= pMemsysConfig->fbOverrideStartKb;
                 break;
             }
             case NV2080_CTRL_FB_INFO_INDEX_HEAP_START:
@@ -400,13 +415,23 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                 }
                 else
                 {
-                    //
-                    // Returns start of heap in kbytes. This is zero unless
-                    // VGA display memory is reserved.
-                    //
-                    heapGetBase(pHeap, &heapBase);
-                    data = NvU64_LO32(heapBase >> 10);
-                    NV_ASSERT(((NvU64) data << 10ULL) == heapBase);
+                    const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                            kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
+                    if (pMemsysConfig->fbOverrideStartKb != 0)
+                    {
+                        data = NvU64_LO32(pMemsysConfig->fbOverrideStartKb);
+                        NV_ASSERT(((NvU64) data << 10ULL) == pMemsysConfig->fbOverrideStartKb);
+                    }
+					else
+					{
+                        //
+                        // Returns start of heap in kbytes. This is zero unless
+                        // VGA display memory is reserved.
+                        //
+                        heapGetBase(pHeap, &heapBase);
+                        data = NvU64_LO32(heapBase >> 10);
+                        NV_ASSERT(((NvU64) data << 10ULL) == heapBase);
+                    }
                 }
                 break;
             }
@@ -487,6 +512,8 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
 
             case NV2080_CTRL_FB_INFO_INDEX_MAPPABLE_HEAP_SIZE:
             {
+                const MEMORY_SYSTEM_STATIC_CONFIG *pMemsysConfig = 
+                            kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
                 if (bIsPmaEnabled)
                 {
                     NvU32 heapSizeKb;
@@ -512,6 +539,7 @@ _fbGetFbInfos(OBJGPU *pGpu, NvHandle hClient, NvHandle hObject, NV2080_CTRL_FB_I
                     if (data > heapSizeKb)
                         data = heapSizeKb;
                 }
+                data -= pMemsysConfig->fbOverrideStartKb;
                 break;
             }
             case NV2080_CTRL_FB_INFO_INDEX_BANK_COUNT:
@@ -1070,9 +1098,10 @@ subdeviceCtrlCmdFbSetZbcReferenced_IMPL
     NV2080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS *pParams
 )
 {
-    OBJGPU       *pGpu          = GPU_RES_GET_GPU(pSubdevice);
-    CALL_CONTEXT *pCallContext  = resservGetTlsCallContext();
+    OBJGPU       *pGpu = GPU_RES_GET_GPU(pSubdevice);
+    CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
     NvU32         gfid;
+    NV_STATUS     status = NV_OK;
 
     NV_ASSERT_OK_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid));
 
@@ -1081,7 +1110,6 @@ subdeviceCtrlCmdFbSetZbcReferenced_IMPL
     if (IS_VIRTUAL(pGpu) || IS_GSP_CLIENT(pGpu))
     {
         RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
-        NV_STATUS     status        = NV_OK;
 
         NV_RM_RPC_CONTROL(pGpu,
                           pRmCtrlParams->hClient,
@@ -1094,7 +1122,7 @@ subdeviceCtrlCmdFbSetZbcReferenced_IMPL
         return status;
     }
 
-    return NV_OK;
+    return status;
 }
 
 /*!
