@@ -89,7 +89,7 @@ kgmmuChangeReplayableFaultOwnership_GV100(OBJGPU *pGpu, KernelGmmu *pKernelGmmu,
 }
 
 /*!
- * @brief Creates the shadow fault buffer for client handling of non-replayable faults
+ * @brief Creates the shadow fault buffer for client handling of replayable/non-replayable faults
  *
  * @param[in] pGpu
  * @param[in] pKernelGmmu
@@ -99,8 +99,9 @@ kgmmuChangeReplayableFaultOwnership_GV100(OBJGPU *pGpu, KernelGmmu *pKernelGmmu,
 NV_STATUS
 kgmmuClientShadowFaultBufferAlloc_GV100
 (
-    OBJGPU               *pGpu,
-    KernelGmmu           *pKernelGmmu
+    OBJGPU           *pGpu,
+    KernelGmmu       *pKernelGmmu,
+    FAULT_BUFFER_TYPE index
 )
 {
     if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
@@ -112,7 +113,7 @@ kgmmuClientShadowFaultBufferAlloc_GV100
         NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
     }
 
-    return kgmmuClientShadowFaultBufferNonreplayableAllocate(pGpu, pKernelGmmu);
+    return kgmmuClientShadowFaultBufferAllocate(pGpu, pKernelGmmu, index);
 }
 
 /*!
@@ -126,14 +127,15 @@ kgmmuClientShadowFaultBufferAlloc_GV100
 NV_STATUS
 kgmmuClientShadowFaultBufferFree_GV100
 (
-    OBJGPU               *pGpu,
-    KernelGmmu           *pKernelGmmu
+    OBJGPU           *pGpu,
+    KernelGmmu       *pKernelGmmu,
+    FAULT_BUFFER_TYPE index
 )
 {
     if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
         return NV_OK;
 
-    return kgmmuClientShadowFaultBufferNonreplayableDestroy(pGpu, pKernelGmmu);
+    return kgmmuClientShadowFaultBufferDestroy(pGpu, pKernelGmmu, index);
 }
 
 /*!
