@@ -500,7 +500,6 @@ knvlinkValidateFabricBaseAddress_GV100
 {
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
     NvU64          fbSizeBytes;
-    NvU64          fbUpperLimit;
 
     fbSizeBytes = pMemoryManager->Ram.fbTotalMemSizeMb << 20;
 
@@ -519,14 +518,8 @@ knvlinkValidateFabricBaseAddress_GV100
     // Align fbSize to mapslot size.
     fbSizeBytes = RM_ALIGN_UP(fbSizeBytes, NVBIT64(34));
 
-    // Check for integer overflow
-    if (!portSafeAddU64(fabricBaseAddr, fbSizeBytes, &fbUpperLimit))
-    {
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
     // Make sure the address range doesn't go beyond the limit, (8K * 16GB).
-    if (fbUpperLimit > NVBIT64(47))
+    if ((fabricBaseAddr + fbSizeBytes) > NVBIT64(47))
     {
         return NV_ERR_INVALID_ARGUMENT;
     }
