@@ -589,6 +589,7 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
     NvBool bMIGNvLinkP2PSupported = ((pKernelMIGManager != NULL) &&
                                      kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
     NV_STATUS status = NV_OK;
+    NvBool    bIsNvlinkReady = NV_TRUE;
     NvU8 i = 0;
     struct
     {
@@ -696,12 +697,16 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
             // as not ready
             //
             status = knvlinkCoreGetRemoteDeviceInfo(pGpu, pKernelNvlink);
-            if (status != NV_OK)
+            if (status == NV_ERR_NOT_READY)
             {
                 NV_PRINTF(LEVEL_INFO, "Nvlink is not ready yet!\n");
-                status = NV_ERR_NOT_READY;
+                bIsNvlinkReady = NV_FALSE;
+            }
+            else if (status != NV_OK)
+            {
                 goto done;
             }
+
         }
 
         //
@@ -711,7 +716,8 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
         //
         knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
 
-        pParams->enabledLinkMask = pKernelNvlink->enabledLinks;
+        // If nvlink is not ready don't report back any links as being enabled
+        pParams->enabledLinkMask = (bIsNvlinkReady) ? pKernelNvlink->enabledLinks : 0x0;
 
         pTmpData->nvlinkLinkAndClockInfoParams.linkMask = pParams->enabledLinkMask;
         pTmpData->nvlinkLinkAndClockInfoParams.bSublinkStateInst = pParams->bSublinkStateInst;
