@@ -204,6 +204,11 @@ knvlinkIsNvlinkP2pSupported_IMPL
         return NV_FALSE;
     }
 
+    if (knvlinkIsBandwidthModeOff(pKernelNvlink))
+    {
+        return NV_FALSE;
+    }
+
     // Get the Nvlink P2P connections from the core library
     status = knvlinkGetP2pConnectionStatus(pGpu, pKernelNvlink, pPeerGpu);
 
@@ -367,7 +372,11 @@ knvlinkGetP2pConnectionStatus_IMPL
     }
 
     // Get the remote ends of the links of local GPU from the nvlink core
-    knvlinkCoreGetRemoteDeviceInfo(pGpu0, pKernelNvlink0);
+    status = knvlinkCoreGetRemoteDeviceInfo(pGpu0, pKernelNvlink0);
+    if (status != NV_OK)
+    {
+        return status;
+    }
 
     // Post topology link enable on links of local GPU
     status = knvlinkEnableLinksPostTopology_HAL(pGpu0, pKernelNvlink0,
@@ -404,7 +413,11 @@ knvlinkGetP2pConnectionStatus_IMPL
         if (knvlinkGetNumLinksToPeer(pGpu1, pKernelNvlink1, pGpu0) != numPeerLinks)
         {
             // Get the remote ends of the links of remote GPU from the nvlink core
-            knvlinkCoreGetRemoteDeviceInfo(pGpu1, pKernelNvlink1);
+            status = knvlinkCoreGetRemoteDeviceInfo(pGpu1, pKernelNvlink1);
+            if (status != NV_OK)
+            {
+                return status;
+            }
 
             // Post topology link enable on links of remote GPU
             status = knvlinkEnableLinksPostTopology_HAL(pGpu1, pKernelNvlink1,
@@ -630,20 +643,12 @@ knvlinkInbandMsgCallbackDispatcher_WORKITEM
 
     pHeader = (nvlink_inband_msg_header_t *)pMessage->data;
 
-    if (pKernelNvlink->inbandCallback[pHeader->type].pCallback == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "No Callback Registered for type %d. Dropping the msg\n", 
-                  pHeader->type);
-        return;
-    }
-
     // Assert reserved in msgHdr are zero
     pRsvd = &pHeader->reserved[0];
     NV_ASSERT((pRsvd[0] == 0) && portMemCmp(pRsvd, pRsvd + 1,
               sizeof(pHeader->reserved) - 1) == 0);
 
-    pKernelNvlink->inbandCallback[pHeader->type].pCallback(pGpu, pData);
+    pKernelNvlink->inbandCallback[pHeader->type].pCallback(gpuInstance, pData);
 }
 
 NV_STATUS
@@ -1021,25 +1026,9 @@ knvlinkPrepareForXVEReset_IMPL
         // initializations in between, without hanging up the GPU trying to
         // flush data over links that aren't available anymore.
         //
-        // Starting from Ampere single GPU reset is supported and hence remove
-        // only the nvlink's of the remote GPU's which are connected to the
-        // current GPU.
-        //
-
-        if (IsAMPEREorBetter(pGpu))
-        {
-            NvU32 remPeerId = kbusGetPeerId_HAL(pRemoteGpu, GPU_GET_KERNEL_BUS(pRemoteGpu), pGpu);
-            if (remPeerId != BUS_INVALID_PEER)
-                status = knvlinkRemoveMapping_HAL(pRemoteGpu, pRemoteKernelNvlink, NV_FALSE,
-                                                  NVBIT(remPeerId),
-                                                  NV_FALSE /* bL2Entry */);
-        }
-        else
-        {
-            status = knvlinkRemoveMapping_HAL(pRemoteGpu, pRemoteKernelNvlink, NV_FALSE,
-                                              ((1 << NVLINK_MAX_PEERS_SW) - 1),
-                                              NV_FALSE /* bL2Entry */);
-        }
+        status = knvlinkRemoveMapping_HAL(pRemoteGpu, pRemoteKernelNvlink, NV_FALSE,
+                                          ((1 << NVLINK_MAX_PEERS_SW) - 1),
+                                          NV_FALSE /* bL2Entry */);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR,
@@ -1180,10 +1169,6 @@ knvlinkSetPowerFeatures_IMPL
     {
         case NVLINK_VERSION_22:
         {
-            // Regkeys finally decide whether or not the power state is supported
-            pKernelNvlink->setProperty(pKernelNvlink, PDB_PROP_KNVLINK_SINGLE_LANE_POWER_STATE_ENABLED,
-                                       (pKernelNvlink->bDisableSingleLaneMode ? NV_FALSE : NV_TRUE));
-
             // NVLink L2 is supported only on MODS and Windows LDDM
             if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM || RMCFG_FEATURE_MODS_FEATURES)
             {
@@ -1193,15 +1178,6 @@ knvlinkSetPowerFeatures_IMPL
 
             break;
         }
-        case NVLINK_VERSION_31:
-        case NVLINK_VERSION_30:
-        case NVLINK_VERSION_20:
-        {
-            // Regkeys finally decide whether or not the power state is supported
-            pKernelNvlink->setProperty(pKernelNvlink, PDB_PROP_KNVLINK_SINGLE_LANE_POWER_STATE_ENABLED,
-                                       (pKernelNvlink->bDisableSingleLaneMode ? NV_FALSE : NV_TRUE));
-            break;
-        }
         default:
             break;
     }
