@@ -196,12 +196,13 @@ getGpuInfos(Subdevice *pSubdevice, NV2080_CTRL_GPU_GET_INFO_V2_PARAMS *pParams,
                 }
 
                 portMemSet(&params, 0x0, sizeof(params));
-                status = pRmApi->Control(pRmApi,
-                                         pGpu->hInternalClient,
-                                         pGpu->hInternalSubdevice,
-                                         NV2080_CTRL_CMD_INTERNAL_GPU_GET_SMC_MODE,
-                                         &params,
-                                         sizeof(params));
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                    pRmApi->Control(pRmApi,
+                                    pGpu->hInternalClient,
+                                    pGpu->hInternalSubdevice,
+                                    NV2080_CTRL_CMD_INTERNAL_GPU_GET_SMC_MODE,
+                                    &params,
+                                    sizeof(params)));
                 data = params.smcMode;
                 break;
             }
@@ -1314,7 +1315,7 @@ subdeviceCtrlCmdGpuGetFermiZcullInfo_IMPL
     NvHandle  hClient = RES_GET_CLIENT_HANDLE(pSubdevice);
     NvHandle  hSubdevice = RES_GET_HANDLE(pSubdevice);
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(GPU_RES_GET_GPU(pSubdevice)->gpuInstance));
 
     portMemSet(&zcullMaskParams, 0, sizeof(zcullMaskParams));
     zcullMaskParams.gpcId = pParams->gpcId;
@@ -2062,9 +2063,6 @@ subdeviceCtrlCmdGpuGetNumMmusPerGpc_IMPL
 /*
  * @brief Update/Set the compute policy config for a GPU
  *
- * Lock Requirements:
- *      Assert that API and GPUs lock held on entry
- *
  * @param[in] pSubdevice
  * @param[in] pParams    pointer to control parameters
  *
@@ -2086,8 +2084,6 @@ subdeviceCtrlCmdGpuSetComputePolicyConfig_IMPL
     NvU32 gidFlags;
     NV_STATUS status = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
-
     switch(pParams->config.type)
     {
         case NV2080_CTRL_GPU_COMPUTE_POLICY_TIMESLICE:
@@ -2139,9 +2135,6 @@ ct_assert(NV2080_CTRL_GPU_COMPUTE_POLICY_MAX <= NV2080_CTRL_GPU_COMPUTE_POLICY_C
 /*
  * @brief Get all compute policy configs for a GPU
  *
- * Lock Requirements:
- *      Assert that API and GPUs lock held on entry
- *
  * @param[in] pSubdevice
  * @param[in] pParams    pointer to control parameters
  *
@@ -2162,8 +2155,6 @@ subdeviceCtrlCmdGpuGetComputePolicyConfig_IMPL
     NvU32 gidFlags;
     NV_STATUS status = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
-
     gidFlags = DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _TYPE, _SHA1) |
                DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _FORMAT, _BINARY);
     status = gpuGetGidInfo(pGpu, NULL, NULL, gidFlags);
@@ -2266,7 +2257,7 @@ NV_STATUS subdeviceCtrlCmdValidateMemMapRequest_IMPL
     NvU32         bar0MapSize;
     NvU64         bar0MapOffset;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(GPU_RES_GET_GPU(pSubdevice)->gpuInstance));
 
     pParams->protection = NV_PROTECT_READ_WRITE;
 
@@ -2458,7 +2449,6 @@ subdeviceCtrlCmdUpdateGfidP2pCapability_IMPL
                 if (pP2PInfo[idx].gfid == INVALID_P2P_GFID)
                 {
                     pP2PInfo[idx].gfid = pParams->gfid;
-                    pGpu->sriovState.p2pFabricPartitionId = pParams->fabricPartitionId;
                     bSetP2PAccess = NV_TRUE;
                     break;
                 }
@@ -2468,7 +2458,6 @@ subdeviceCtrlCmdUpdateGfidP2pCapability_IMPL
                 if (pP2PInfo[idx].gfid == pParams->gfid)
                 {
                     pP2PInfo[idx].gfid = INVALID_P2P_GFID;
-                    pGpu->sriovState.p2pFabricPartitionId = INVALID_FABRIC_PARTITION_ID;
                     bSetP2PAccess = NV_TRUE;
                     break;
                 }
