@@ -339,7 +339,6 @@ static NV_STATUS _class9074SetNotifierHi
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -358,7 +357,6 @@ static NV_STATUS _class9074SetNotifierLo
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -394,7 +392,6 @@ static NV_STATUS _class9074SetSemaphoreHi
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -413,7 +410,6 @@ static NV_STATUS _class9074SetSemaphoreLo
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -449,7 +445,6 @@ static NV_STATUS _class9074SetWaitTimestampHi
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -468,7 +463,6 @@ static NV_STATUS _class9074SetWaitTimestampLo
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -497,7 +491,6 @@ static NV_STATUS _class9074SetSemaphoreReleaseValue
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -516,7 +509,6 @@ static NV_STATUS _class9074ScheduleSemaphoreRelease
 (
     OBJGPU *pGpu,
     ChannelDescendant *pObject,
-    PMETHOD pMethod,
     NvU32   Offset,
     NvU32   Data
 )
@@ -599,7 +591,7 @@ static NV_STATUS _class9074TimerCallback
 } // end of _class9074TimerCallback
 
 // GF100_TIMED_SEMAPHORE_SW
-static METHOD GF100TimedSemSwMethods[] =
+static const METHOD GF100TimedSemSwMethods[] =
 {
     { mthdNoOperation,                    0x0100, 0x0103 },
     { _class9074SetNotifierHi,            0x0140, 0x0143 },
@@ -615,7 +607,7 @@ static METHOD GF100TimedSemSwMethods[] =
 NV_STATUS tsemaGetSwMethods_IMPL
 (
     TimedSemaSwObject *pTimedSemSw,
-    METHOD           **ppMethods,
+    const METHOD     **ppMethods,
     NvU32             *pNumMethods
 )
 {
