@@ -133,7 +133,7 @@ clInitMappingPciBusDevice_IMPL
     NvU16 vendorID, deviceID;
     NvBool bFoundDevice = NV_FALSE;
 
-    if (IsT194(pGpu))
+    if (IsT194(pGpu) || IsT234(pGpu))
         return NV0000_CTRL_GPU_INVALID_ID;
 
     // do we already know our domain/bus/device?
@@ -486,7 +486,7 @@ clFindFHBAndGetChipsetInfoIndex_IMPL
                     }
                 }
 
-                if (vendorID == PCI_INVALID_VENDORID)
+                if (!PCI_IS_VENDORID_VALID(vendorID))
                     break;           // skip to the next device
 
                 if ((osPciReadByte(handle, PCI_HEADER_TYPE0_BASECLASS)) != PCI_CLASS_BRIDGE_DEV)
@@ -605,41 +605,6 @@ clIsL1MaskEnabledForUpstreamPort_IMPL
     return bEnable;
 }
 
-//
-// return the First Host Bridge's handle, VendorID and DeviceID
-//
-NV_STATUS
-clGetFHBHandle_IMPL(
-    OBJCL *pCl,
-    void **Handle,
-    NvU16 *VendorID,
-    NvU16 *DeviceID
-)
-{
-    NV_ASSERT(Handle && DeviceID && VendorID); // Avoid Null Pointer
-
-    if (!pCl->FHBAddr.valid)
-        return NV_ERR_GENERIC;
-
-    *Handle   = pCl->FHBAddr.handle;
-    *DeviceID = pCl->FHBBusInfo.deviceID;
-    *VendorID = pCl->FHBBusInfo.vendorID;
-
-    // can this happen, should this be #if 0 out?
-    if (*Handle == NULL)
-    {
-        *Handle = osPciInitHandle(pCl->FHBAddr.domain,
-                                  pCl->FHBAddr.bus,
-                                  pCl->FHBAddr.device,
-                                  pCl->FHBAddr.func,
-                                  VendorID,
-                                  DeviceID);
-    }
-
-    return (*Handle ? NV_OK : NV_ERR_GENERIC);
-}
-
-
 NV_STATUS
 clInit_IMPL(
     OBJGPU *pGpu,
@@ -763,6 +728,7 @@ void clSyncWithGsp_IMPL(OBJCL *pCl, GspSystemInfo *pGSI)
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L1_CHIPSET_DISABLED);
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L0S_CHIPSET_ENABLED_MOBILE_ONLY);
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L1_CHIPSET_ENABLED_MOBILE_ONLY);
+    CL_SYNC_PDB(PDB_PROP_CL_ASPM_UPSTREAM_PORT_L1_MASK_ENABLED);
     CL_SYNC_PDB(PDB_PROP_CL_PCIE_GEN1_GEN2_SWITCH_CHIPSET_DISABLED);
     CL_SYNC_PDB(PDB_PROP_CL_PCIE_GEN1_GEN2_SWITCH_CHIPSET_DISABLED_GEFORCE);
     CL_SYNC_PDB(PDB_PROP_CL_EXTENDED_TAG_FIELD_NOT_CAPABLE);
@@ -787,7 +753,6 @@ void clSyncWithGsp_IMPL(OBJCL *pCl, GspSystemInfo *pGSI)
     CL_SYNC_PDB(PDB_PROP_CL_HAS_RESIZABLE_BAR_ISSUE);
     CL_SYNC_PDB(PDB_PROP_CL_IS_EXTERNAL_GPU);
     CL_SYNC_PDB(PDB_PROP_CL_BUG_3751839_GEN_SPEED_WAR);
-    CL_SYNC_PDB(PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS);
 
 #undef CL_SYNC_PDB
 
