@@ -169,7 +169,7 @@ rmclientConstruct_IMPL
         PUID_TOKEN pUidToken = osGetCurrentUidToken();
         UserInfo *pUserInfo = NULL;
 
-        if (RMCFG_FEATURE_PLATFORM_GSP && IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu))
+        if (RMCFG_FEATURE_PLATFORM_GSP)
         {
             pClient->pSecurityToken = pSecurityToken;
         }
@@ -932,8 +932,6 @@ _registerOSInfo
     if (pInsert == NULL)
         return NV_ERR_NO_MEMORY;
 
-    osAllocatedRmClient(pOSInfo);
-
     *pInsert = pClient;
 
     return NV_OK;
