@@ -224,6 +224,62 @@ rmresGetMemoryMappingDescriptor_IMPL
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS
+rmresControlSerialization_Prologue_IMPL
+(
+    RmResource                     *pResource,
+    CALL_CONTEXT                   *pCallContext,
+    RS_RES_CONTROL_PARAMS_INTERNAL *pParams
+)
+{
+    OBJGPU *pGpu = gpumgrGetGpu(pResource->rpcGpuInstance);
+
+    if (pGpu != NULL &&
+        ((IS_VIRTUAL(pGpu)    && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_VGPU_HOST)) ||
+         (IS_GSP_CLIENT(pGpu) && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_PHYSICAL))))
+    {
+        return serverSerializeCtrlDown(pCallContext, pParams->cmd, pParams->pParams, pParams->paramsSize, &pParams->flags);
+    }
+    else
+    {
+        // Deserialize and replace the pParams->pParams if necessary
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, serverDeserializeCtrlDown(pCallContext, pParams->cmd, pParams->pParams, pParams->paramsSize, &pParams->flags));
+
+        if (pParams->flags & NVOS54_FLAGS_FINN_SERIALIZED)
+        {
+            pParams->pParams = pCallContext->pDeserializedParams;
+            pCallContext->bRestoreParams = NV_TRUE;
+        }
+    }
+
+    return NV_OK;
+}
+
+void
+rmresControlSerialization_Epilogue_IMPL
+(
+    RmResource                     *pResource,
+    CALL_CONTEXT                   *pCallContext,
+    RS_RES_CONTROL_PARAMS_INTERNAL *pParams
+)
+{
+    OBJGPU *pGpu = gpumgrGetGpu(pResource->rpcGpuInstance);
+
+    if (pGpu != NULL &&
+        ((IS_VIRTUAL(pGpu)    && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_VGPU_HOST)) ||
+         (IS_GSP_CLIENT(pGpu) && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_PHYSICAL))))
+    {
+        NV_ASSERT_OK(serverDeserializeCtrlUp(pCallContext, pParams->cmd, pParams->pParams, pParams->paramsSize, &pParams->flags));
+    }
+    else if (pCallContext->bRestoreParams)
+    {
+        pParams->pParams = pCallContext->pSerializedParams;
+    }
+
+    NV_ASSERT_OK(serverSerializeCtrlUp(pCallContext, pParams->cmd, pParams->pParams, pParams->paramsSize, &pParams->flags));
+    serverFreeSerializeStructures(pCallContext, pParams->pParams);
+}
+
 NV_STATUS
 rmresControl_Prologue_IMPL
 (
@@ -235,11 +291,9 @@ rmresControl_Prologue_IMPL
     NV_STATUS status = NV_OK;
     OBJGPU *pGpu = gpumgrGetGpu(pResource->rpcGpuInstance);
 
-    if (pGpu == NULL)
-        return NV_OK;
-
-    if ((IS_VIRTUAL(pGpu)    && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_VGPU_HOST)) ||
-        (IS_GSP_CLIENT(pGpu) && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_PHYSICAL)))
+    if (pGpu != NULL &&
+        ((IS_VIRTUAL(pGpu)    && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_VGPU_HOST)) ||
+         (IS_GSP_CLIENT(pGpu) && (pParams->pCookie->ctrlFlags & RMCTRL_FLAGS_ROUTE_TO_PHYSICAL))))
     {
         //
         // GPU lock is required to protect the RPC buffers. 
@@ -272,6 +326,7 @@ rmresControl_Prologue_IMPL
 
         return (status == NV_OK) ? NV_WARN_NOTHING_TO_DO : status;
     }
+
     return NV_OK;
 }
 
