@@ -330,6 +330,44 @@ char * nvU32ToStr(NvU32 value, char *string, NvU32 radix)
   return string;
 }
 
+/*!
+ * @brief Convert unsigned long long hex int to char*
+ *
+ * @param[in]   value        to be converted to string
+ * @param[in]   targetStrLen Denoted the converted string Length
+ * @param[out] *string       is the char array to be have the converted data
+ * 
+ * @return the converted string
+ */
+char *
+nvU64ToStr
+(
+    NvU64  value,
+    char  *string,
+    NvU32  targetStrLen
+)
+{
+    char  tempBuffer[65];
+    NvU32 base        = 16;
+    NvU32 rem         = 0;
+    NvU32 inx         = 0;
+
+    for (inx = 0; inx < targetStrLen; inx++)
+    {
+        rem             = value % base;
+        value           = value / base;
+        tempBuffer[inx] = (rem > 9)? (rem - 10) + 'a' : rem + '0'; 
+    }
+    
+    for(inx = 0; inx < targetStrLen; inx++)
+    {
+        string[inx] = tempBuffer[targetStrLen - inx - 1];
+    }
+
+    string[targetStrLen] = '\0';
+
+    return string;
+}
 
 /**
  * @brief Get the string length
