@@ -170,17 +170,14 @@ mmuWalkGetUserCtx
     return pWalk->pUserCtx;
 }
 
-NV_STATUS
+void
 mmuWalkSetUserCtx
 (
     MMU_WALK          *pWalk,
     MMU_WALK_USER_CTX *pUserCtx
 )
 {
-    NV_ASSERT_OR_RETURN(NULL != pWalk, NV_ERR_INVALID_STATE);
-
     pWalk->pUserCtx = pUserCtx;
-    return NV_OK;
 }
 
 const MMU_WALK_CALLBACKS *
@@ -302,7 +299,7 @@ NV_STATUS mmuWalkProcessPdes
             NvU32 entryIndexHi = mmuFmtVirtAddrToEntryIndex(pLevel->pFmt, vaHi);
             NvU32 entryIndex;
             NvU32 index;
-            NvU32 entryIndexFillStart;
+            NvU32 entryIndexFillStart = 0;
             NvU32 entryIndexFillEnd;
             NvU32 pendingFillCount = 0;
 
@@ -688,7 +685,7 @@ cleanupIter:
         NvU32        entryIndexHi = mmuFmtVirtAddrToEntryIndex(pLevel->pFmt, vaHi);
         NvU32        entryIndex;
         NvU32        index;
-        NvU32        entryIndexFillStart;
+        NvU32        entryIndexFillStart = 0;
         NvU32        entryIndexFillEnd;
         NvU32        pendingFillCount = 0;
 
