@@ -61,15 +61,6 @@ static NV_STATUS _clientAllocResourceHelper(RsClient *pClient, RsServer *pServer
 static NV_STATUS _clientConstructResourceRef(RsClient *pClient, RsServer *pServer, RsResourceRef *pParentRef,
                                              NvHandle hResource, NvU32 classId, RsResourceRef **ppResourceRef);
 
-/**
- * Release all CPU address mappings for a resource
- *
- * @param[in] pClient Client that owns the resource
- * @param[in] pCallContext Caller information (which includes the resource reference whose mappings will be freed)
- * @param[in] pLockInfo Information about which locks are already held, for recursive calls
- */
-static NV_STATUS _clientUnmapResourceRefMappings(RsClient *pClient, CALL_CONTEXT *pCallContext, RS_LOCK_INFO *pLockInfo);
-
 /**
  * Release all CPU address mappings that reference this resource
  *
@@ -805,7 +796,7 @@ clientFreeResource_IMPL
     resPreDestruct(pResource);
 
     // Remove all CPU mappings
-    _clientUnmapResourceRefMappings(pClient, &callContext, pParams->pLockInfo);
+    clientUnmapResourceRefMappings(pClient, &callContext, pParams->pLockInfo);
     _clientUnmapBackRefMappings(pClient, &callContext, pParams->pLockInfo);
 
     // Remove all inter-mappings
@@ -1080,7 +1071,7 @@ clientDestructResourceRef_IMPL
 }
 
 NV_STATUS
-_clientUnmapResourceRefMappings
+clientUnmapResourceRefMappings
 (
     RsClient *pClient,
     CALL_CONTEXT *pCallContext,
